@@ -40,6 +40,7 @@ static char *unescape(char *buf)
 			ch = ' ';
 
 		if (ch == '%') {
+			long l;
 			if (i + 2 > buflen)
 				fatalx(EXIT_FAILURE, "string too short for escaped char");
 			hex[0] = buf[++i];
@@ -48,7 +49,7 @@ static char *unescape(char *buf)
 			if (!isxdigit((unsigned char) hex[0])
 				|| !isxdigit((unsigned char) hex[1]))
 				fatalx(EXIT_FAILURE, "bad escape char");
-			long l = strtol(hex, NULL, 16);
+			l = strtol(hex, NULL, 16);
 			assert(l>=0);
 			assert(l<=255);
 			ch = (char)l;	/* FIXME: Loophole about non-ASCII symbols in top 128 values, or negatives for signed char... */
