@@ -17,10 +17,26 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#include "config.h"
 #include "nutclient.h"
 
 #include <sstream>
 
+/* TODO: Make it a run-time option like upsdebugx(),
+ * probably with a verbosity level variable in each
+ * class instance */
+#include <iostream>	/* std::cerr debugging */
+#include <cstdint>
+#include <cstdlib>
+#include <stdlib.h>
+
+#ifndef WIN32
+# ifdef HAVE_PTHREAD
+/* this include is needed on AIX to have errno stored in thread local storage */
+#  include <pthread.h>
+# endif
+#endif
+
 #include <errno.h>
 #include <string.h>
 #include <stdio.h>
@@ -29,7 +45,38 @@
 /* Thanks to Benjamin Roux (http://broux.developpez.com/articles/c/sockets/) */
 #ifdef WIN32
 #  include <winsock2.h>
-#else
+#  define SOCK_OPT_CAST (char *)
+
+/* equivalent of W32_NETWORK_CALL_OVERRIDE
+ * invoked by wincompat.h in upsclient.c:
+ */
+static inline int sktconnect(int fh, struct sockaddr * name, int len)
+{
+	int ret = connect(fh,name,len);
+	errno = WSAGetLastError();
+	return ret;
+}
+static inline int sktread(int fh, void *buf, int size)
+{
+	int ret = recv(fh,(char*)buf,size,0);
+	errno = WSAGetLastError();
+	return ret;
+}
+static inline int sktwrite(int fh, const void*buf, int size)
+{
+	int ret = send(fh,(char*)buf,size,0);
+	errno = WSAGetLastError();
+	return ret;
+}
+static inline int sktclose(int fh)
+{
+	int ret = closesocket((SOCKET)fh);
+	errno = WSAGetLastError();
+	return ret;
+}
+
+#else /* not WIN32 */
+
 #  include <sys/types.h>
 #  include <sys/socket.h>
 #  include <netinet/in.h>
@@ -37,28 +84,67 @@
 #  include <unistd.h> /* close */
 #  include <netdb.h> /* gethostbyname */
 #  include <fcntl.h>
-#  define INVALID_SOCKET -1
-#  define SOCKET_ERROR -1
-#  define closesocket(s) close(s)
+#  ifndef INVALID_SOCKET
+#    define INVALID_SOCKET -1
+#  endif
+#  ifndef SOCKET_ERROR
+#    define SOCKET_ERROR -1
+#  endif
+#  ifndef closesocket
+#    define closesocket(s) close(s)
+#  endif
+#  define SOCK_OPT_CAST
    typedef int SOCKET;
    typedef struct sockaddr_in SOCKADDR_IN;
    typedef struct sockaddr SOCKADDR;
    typedef struct in_addr IN_ADDR;
+
+#  define sktconnect(h,n,l)	::connect(h,n,l)
+#  define sktread(h,b,s) 	::read(h,b,s)
+#  define sktwrite(h,b,s) 	::write(h,b,s)
+#  define sktclose(h)		::close(h)
+
+/* WA for Solaris/i386 bug: non-blocking connect sets errno to ENOENT */
+#  if (defined NUT_PLATFORM_SOLARIS)
+#    define SOLARIS_i386_NBCONNECT_ENOENT(status) ( (!strcmp("i386", CPU_TYPE)) ? (ENOENT == (status)) : 0 )
+#  else
+#    define SOLARIS_i386_NBCONNECT_ENOENT(status) (0)
+#  endif  /* end of Solaris/i386 WA for non-blocking connect */
+
+/* WA for AIX bug: non-blocking connect sets errno to 0 */
+#  if (defined NUT_PLATFORM_AIX)
+#    define AIX_NBCONNECT_0(status) (0 == (status))
+#  else
+#    define AIX_NBCONNECT_0(status) (0)
+#  endif  /* end of AIX WA for non-blocking connect */
+
 #endif /* WIN32 */
-/* End of Windows/Linux Socket compatibility layer: */
+/* End of Windows/Linux Socket compatibility layer */
 
 
 /* Include nut common utility functions or define simple ones if not */
 #ifdef HAVE_NUTCOMMON
+/* For C++ code below, we do not actually use the fallback time methods
+ * (on mingw mostly), but in C++ context they happen to conflict with
+ * time.h or ctime headers, while native-C does not. Just disable fallback:
+ */
+# ifndef HAVE_GMTIME_R
+#  define HAVE_GMTIME_R 111
+# endif
+# ifndef HAVE_LOCALTIME_R
+#  define HAVE_LOCALTIME_R 111
+# endif
 #include "common.h"
-#else /* HAVE_NUTCOMMON */
+#else /* not HAVE_NUTCOMMON */
 #include <stdlib.h>
 #include <string.h>
 static inline void *xmalloc(size_t size){return malloc(size);}
 static inline void *xcalloc(size_t number, size_t size){return calloc(number, size);}
 static inline void *xrealloc(void *ptr, size_t size){return realloc(ptr, size);}
 static inline char *xstrdup(const char *string){return strdup(string);}
-#endif /* HAVE_NUTCOMMON */
+#endif /* not HAVE_NUTCOMMON */
+
+#include "nut_stdint.h" /* PRIuMAX etc. */
 
 /* To stay in line with modern C++, we use nullptr (not numeric NULL
  * or shim __null on some systems) which was defined after C++98.
@@ -108,12 +194,12 @@ std::string SystemException::err()
  * https://lgtm.com/rules/2165180572/ like:
  *   NutException& operator=(NutException& rhs) = default;
  */
-NutException::~NutException() {}
-SystemException::~SystemException() {}
-IOException::~IOException() {}
-UnknownHostException::~UnknownHostException() {}
-NotConnectedException::~NotConnectedException() {}
-TimeoutException::~TimeoutException() {}
+NutException::~NutException() noexcept {}
+SystemException::~SystemException() noexcept {}
+IOException::~IOException() noexcept {}
+UnknownHostException::~UnknownHostException() noexcept {}
+NotConnectedException::~NotConnectedException() noexcept {}
+TimeoutException::~TimeoutException() noexcept {}
 
 
 namespace internal
@@ -123,7 +209,7 @@ namespace internal
  * Internal socket wrapper.
  * Provides only client socket functions.
  *
- * Implemented as separate internal class to easily hide plateform specificities.
+ * Implemented as separate internal class to easily hide platform specificities.
  */
 class Socket
 {
@@ -134,6 +220,7 @@ public:
 	void connect(const std::string& host, uint16_t port);
 	void disconnect();
 	bool isConnected()const;
+	void setDebugConnect(bool d);
 
 	void setTimeout(time_t timeout);
 	bool hasTimeout()const{return _tv.tv_sec>=0;}
@@ -147,12 +234,14 @@ public:
 
 private:
 	SOCKET _sock;
+	bool _debugConnect;
 	struct timeval	_tv;
 	std::string _buffer; /* Received buffer, string because data should be text only. */
 };
 
 Socket::Socket():
 _sock(INVALID_SOCKET),
+_debugConnect(false),
 _tv()
 {
 	_tv.tv_sec = -1;
@@ -169,6 +258,11 @@ void Socket::setTimeout(time_t timeout)
 	_tv.tv_sec = timeout;
 }
 
+void Socket::setDebugConnect(bool d)
+{
+	_debugConnect = d;
+}
+
 void Socket::connect(const std::string& host, uint16_t port)
 {
 	int	sock_fd;
@@ -178,40 +272,94 @@ void Socket::connect(const std::string& host, uint16_t port)
 	fd_set 			wfds;
 	int			error;
 	socklen_t		error_size;
-	long			fd_flags;
 
-	_sock = -1;
+#ifndef WIN32
+	long			fd_flags;
+#else
+	HANDLE event = NULL;
+	unsigned long argp;
+
+	WSADATA WSAdata;
+	WSAStartup(2,&WSAdata);
+#endif
+
+	_sock = INVALID_SOCKET;
 
 	if (host.empty()) {
+		if (_debugConnect) std::cerr <<
+			"[D2] Socket::connect(): host.empty()" <<
+			std::endl << std::flush;
 		throw nut::UnknownHostException();
 	}
 
-	snprintf(sport, sizeof(sport), "%ju", static_cast<uintmax_t>(port));
+	snprintf(sport, sizeof(sport), "%" PRIuMAX, static_cast<uintmax_t>(port));
 
 	memset(&hints, 0, sizeof(hints));
+	/* TODO? Port IPv4 vs IPv6 detail from upsclient.c */
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_protocol = IPPROTO_TCP;
 
+	if (_debugConnect) std::cerr <<
+		"[D2] Socket::connect(): getaddrinfo(" <<
+		host << ", " <<
+		sport << ", " <<
+		"...)" << std::endl << std::flush;
+
 	while ((v = getaddrinfo(host.c_str(), sport, &hints, &res)) != 0) {
 		switch (v)
 		{
 		case EAI_AGAIN:
 			continue;
 		case EAI_NONAME:
+			if (_debugConnect) std::cerr <<
+				"[D2] Socket::connect(): " <<
+				"connect not successful: " <<
+				"UnknownHostException" <<
+				std::endl << std::flush;
 			throw nut::UnknownHostException();
-		case EAI_SYSTEM:
-			throw nut::SystemException();
 		case EAI_MEMORY:
+			if (_debugConnect) std::cerr <<
+				"[D2] Socket::connect(): " <<
+				"connect not successful: " <<
+				"Out of memory" <<
+				std::endl << std::flush;
 			throw nut::NutException("Out of memory");
+#ifndef WIN32
+		case EAI_SYSTEM:
+#else
+		case WSANO_RECOVERY:
+#endif
+			if (_debugConnect) std::cerr <<
+				"[D2] Socket::connect(): " <<
+				"connect not successful: " <<
+				"SystemException" <<
+				std::endl << std::flush;
+			throw nut::SystemException();
 		default:
+			if (_debugConnect) std::cerr <<
+				"[D2] Socket::connect(): " <<
+				"connect not successful: " <<
+				"Unknown error" <<
+				std::endl << std::flush;
 			throw nut::NutException("Unknown error");
 		}
 	}
 
 	for (ai = res; ai != nullptr; ai = ai->ai_next) {
 
+		if (_debugConnect) std::cerr <<
+			"[D2] Socket::connect(): socket(" <<
+			ai->ai_family << ", " <<
+			ai->ai_socktype << ", " <<
+			ai->ai_protocol << ")" <<
+			std::endl << std::flush;
+
 		sock_fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+		if (_debugConnect) std::cerr <<
+			"[D2] Socket::connect(): socket(): " <<
+			"sock_fd = " << sock_fd <<
+			std::endl << std::flush;
 
 		if (sock_fd < 0) {
 			switch (errno)
@@ -220,39 +368,94 @@ void Socket::connect(const std::string& host, uint16_t port)
 			case EINVAL:
 				break;
 			default:
+				if (_debugConnect) std::cerr <<
+					"[D2] Socket::connect(): " <<
+					"connect not successful: " <<
+					"SystemException" <<
+					std::endl << std::flush;
 				throw nut::SystemException();
 			}
 			continue;
 		}
 
 		/* non blocking connect */
-		if(hasTimeout()) {
+		if (hasTimeout()) {
+#ifndef WIN32
 			fd_flags = fcntl(sock_fd, F_GETFL);
 			fd_flags |= O_NONBLOCK;
 			fcntl(sock_fd, F_SETFL, fd_flags);
+#else
+			event = CreateEvent(NULL, /* Security */
+					FALSE, /* auto-reset */
+					FALSE, /* initial state */
+					NULL); /* no name */
+
+			/* Associate socket event to the socket via its Event object */
+			WSAEventSelect( sock_fd, event, FD_CONNECT );
+			CloseHandle(event);
+#endif
 		}
 
-		while ((v = ::connect(sock_fd, ai->ai_addr, ai->ai_addrlen)) < 0) {
-			if(errno == EINPROGRESS) {
+		if (_debugConnect) std::cerr <<
+			"[D2] Socket::connect(): sktconnect(" <<
+			sock_fd << ", " <<
+			ai->ai_addr << ", " <<
+			ai->ai_addrlen << ")" <<
+			std::endl << std::flush;
+
+		while ((v = sktconnect(sock_fd, ai->ai_addr, ai->ai_addrlen)) < 0) {
+			if (_debugConnect) std::cerr <<
+				"[D2] Socket::connect(): " <<
+				"sktconnect() < 0" <<
+				"; errno = " << errno <<
+				"; v = " << v <<
+				std::endl << std::flush;
+
+#ifndef WIN32
+			if(errno == EINPROGRESS || SOLARIS_i386_NBCONNECT_ENOENT(errno) || AIX_NBCONNECT_0(errno)) {
+#else
+			if(errno == WSAEWOULDBLOCK) {
+#endif
 				FD_ZERO(&wfds);
 				FD_SET(sock_fd, &wfds);
-				select(sock_fd+1, nullptr, &wfds, nullptr, hasTimeout() ? &_tv : nullptr);
+				select(sock_fd+1, nullptr, &wfds, nullptr,
+					hasTimeout() ? &_tv : nullptr);
 				if (FD_ISSET(sock_fd, &wfds)) {
 					error_size = sizeof(error);
 					getsockopt(sock_fd, SOL_SOCKET, SO_ERROR,
-							&error, &error_size);
+							SOCK_OPT_CAST &error, &error_size);
 					if( error == 0) {
 						/* connect successful */
+						if (_debugConnect) std::cerr <<
+							"[D2] Socket::connect(): " <<
+							"connect-select successful" <<
+							std::endl << std::flush;
 						v = 0;
 						break;
 					}
 					errno = error;
+					if (_debugConnect) std::cerr <<
+						"[D2] Socket::connect(): " <<
+						"connect-select not successful: " <<
+						"errno = " << errno <<
+						std::endl << std::flush;
 				}
 				else {
 					/* Timeout */
 					v = -1;
+					if (_debugConnect) std::cerr <<
+						"[D2] Socket::connect(): " <<
+						"connect-select not successful: timeout" <<
+						std::endl << std::flush;
 					break;
 				}
+			} else {
+				/* WIN32: errno=10061 is actively refusing connection */
+				if (_debugConnect) std::cerr <<
+					"[D2] Socket::connect(): " <<
+					"connect not successful: " <<
+					"errno = " << errno <<
+					std::endl << std::flush;
 			}
 
 			switch (errno)
@@ -271,17 +474,34 @@ void Socket::connect(const std::string& host, uint16_t port)
 		}
 
 		if (v < 0) {
-			close(sock_fd);
+			if (_debugConnect) std::cerr <<
+				"[D2] Socket::connect(): " <<
+				"sktconnect() remains < 0 => sktclose()" <<
+				std::endl << std::flush;
+			sktclose(sock_fd);
 			continue;
 		}
+		if (_debugConnect) std::cerr <<
+			"[D2] Socket::connect(): " <<
+			"sktconnect() > 0, looks promising" <<
+			std::endl << std::flush;
 
 		/* switch back to blocking operation */
-		if(hasTimeout()) {
+		if (hasTimeout()) {
+#ifndef WIN32
 			fd_flags = fcntl(sock_fd, F_GETFL);
 			fd_flags &= ~O_NONBLOCK;
 			fcntl(sock_fd, F_SETFL, fd_flags);
+#else
+			argp = 0;
+			ioctlsocket(sock_fd, FIONBIO, &argp);
+#endif
 		}
 
+		if (_debugConnect) std::cerr <<
+			"[D2] Socket::connect(): " <<
+			"saving sock_fd = " << sock_fd <<
+			std::endl << std::flush;
 		_sock = sock_fd;
 //		ups->upserror = 0;
 //		ups->syserrno = 0;
@@ -290,10 +510,24 @@ void Socket::connect(const std::string& host, uint16_t port)
 
 	freeaddrinfo(res);
 
+#ifndef WIN32
 	if (_sock < 0) {
+#else
+	if (_sock == INVALID_SOCKET) {
+		/* In tracing one may see 18446744073709551615 = "-1" after
+		 * conversion from 'long long unsigned int' to 'int'
+		 * 64-bit WINSOCK API with UINT_PTR , see gory details at e.g.
+		 * https://github.com/openssl/openssl/issues/7282#issuecomment-430633656
+		 */
+#endif
+		if (_debugConnect) std::cerr <<
+			"[D2] Socket::connect(): " <<
+			"invalid _sock = " << _sock <<
+			std::endl << std::flush;
 		throw nut::IOException("Cannot connect to host");
 	}
 
+	/* TODO? See upsclient.c for NSS/SSL connection handling */
 
 #ifdef OLD
 	struct hostent *hostinfo = nullptr;
@@ -315,7 +549,7 @@ void Socket::connect(const std::string& host, uint16_t port)
 	sin.sin_addr = *(IN_ADDR *) hostinfo->h_addr;
 	sin.sin_port = htons(port);
 	sin.sin_family = AF_INET;
-	if(::connect(_sock,(SOCKADDR *) &sin, sizeof(SOCKADDR)) == SOCKET_ERROR)
+	if(sktconnect(_sock,(SOCKADDR *) &sin, sizeof(SOCKADDR)) == SOCKET_ERROR)
 	{
 		_sock = INVALID_SOCKET;
 		throw nut::IOException("Cannot connect to host");
@@ -356,7 +590,7 @@ size_t Socket::read(void* buf, size_t sz)
 		}
 	}
 
-	ssize_t res = ::read(_sock, buf, sz);
+	ssize_t res = sktread(_sock, buf, sz);
 	if(res==-1)
 	{
 		disconnect();
@@ -383,7 +617,7 @@ size_t Socket::write(const void* buf, size_t sz)
 		}
 	}
 
-	ssize_t res = ::write(_sock, buf, sz);
+	ssize_t res = sktwrite(_sock, buf, sz);
 	if(res==-1)
 	{
 		disconnect();
@@ -586,6 +820,11 @@ void TcpClient::connect()
 	_socket->connect(_host, _port);
 }
 
+void TcpClient::setDebugConnect(bool d)
+{
+	_socket->setDebugConnect(d);
+}
+
 std::string TcpClient::getHost()const
 {
 	return _host;
@@ -806,8 +1045,42 @@ TrackingID TcpClient::executeDeviceCommand(const std::string& dev, const std::st
 	return sendTrackingQuery("INSTCMD " + dev + " " + name + " " + param);
 }
 
+std::map<std::string, std::set<std::string>> TcpClient::listDeviceClients(void)
+{
+	/* Lists all clients of all devices (which have at least one client) */
+	std::map<std::string, std::set<std::string>> deviceClientsMap;
+
+	std::set<std::string> devs = getDeviceNames();
+	for(std::set<std::string>::iterator it=devs.begin(); it!=devs.end(); ++it)
+	{
+		std::string dev = *it;
+		std::set<std::string> deviceClients = deviceGetClients(dev);
+		if (!deviceClients.empty()) {
+			deviceClientsMap[dev] = deviceClients;
+		}
+	}
+
+	return deviceClientsMap;
+}
+
+std::set<std::string> TcpClient::deviceGetClients(const std::string& dev)
+{
+	/* Who did a deviceLogin() to this dev? */
+	std::set<std::string> clients;
+
+	std::vector<std::vector<std::string> > res = list("CLIENT", dev);
+	for(size_t n=0; n<res.size(); ++n)
+	{
+		clients.insert(res[n][0]);
+	}
+
+	return clients;
+}
+
 void TcpClient::deviceLogin(const std::string& dev)
 {
+	/* Requires that current session is already logged in with
+	 * an account which has one of "upsmon" roles in `upsd.users` */
 	detectError(sendQuery("LOGIN " + dev));
 }
 
@@ -1332,6 +1605,12 @@ TrackingID Device::executeCommand(const std::string& name, const std::string& pa
 	return getClient()->executeDeviceCommand(getName(), name, param);
 }
 
+std::set<std::string> Device::getClients()
+{
+	if (!isOk()) throw NutException("Invalid device");
+	return getClient()->deviceGetClients(getName());
+}
+
 void Device::login()
 {
 	if (!isOk()) throw NutException("Invalid device");
@@ -1354,6 +1633,8 @@ void Device::primary()
 
 void Device::forcedShutdown()
 {
+	if (!isOk()) throw NutException("Invalid device");
+	getClient()->deviceForcedShutdown(getName());
 }
 
 int Device::getNumLogins()
