@@ -37,6 +37,7 @@
  */
 
 #include "common.h"
+#include "nut_stdint.h"
 #include "upsclient.h"
 #include "cgilib.h"
 #include <stdlib.h>
@@ -49,7 +50,7 @@
 
 static	char	*monhost = NULL, *cmd = NULL;
 
-static	int	port;
+static	uint16_t	port;
 static	char	*upsname, *hostname;
 static	UPSCONN_t	ups;
 
@@ -60,7 +61,7 @@ static	UPSCONN_t	ups;
 
 void parsearg(char *var, char *value)
 {
-	int	i, v;
+	long long	i, v;	/* Be big enough to fit all expected inputs; truncate later */
 
 	/* avoid bogus junk from evil people */
 	if ((strlen(var) > MAX_CGI_STRLEN) || (strlen(value) > MAX_CGI_STRLEN))
@@ -82,17 +83,20 @@ void parsearg(char *var, char *value)
 	for (i = 0; imgarg[i].name != NULL; i++) {
 		if (!strcmp(imgarg[i].name, var)) {
 			if (!strncmp(value, "0x", 2))
-				v = strtoul(value + 2, (char **)NULL, 16);
+				v = (long long)strtoul(value + 2, (char **)NULL, 16);
 			else
-				v = atoi(value);
+				v = (long long)atoi(value);
 
 			/* avoid false numbers from bad people */
 			if (v < imgarg[i].min)
 				imgarg[i].val = imgarg[i].min;
 			else if (v > imgarg[i].max)
 				imgarg[i].val = imgarg[i].max;
-			else
-				imgarg[i].val = v;
+			else {
+				assert (v < INT_MAX);
+				assert (v > INT_MIN);
+				imgarg[i].val = (int)v;
+			}
 			return;
 		}
 	}
@@ -111,6 +115,9 @@ static int get_imgarg(const char *name)
 }
 
 /* write the HTML header then have gd dump the image */
+static void drawimage(gdImagePtr im)
+	__attribute__((noreturn));
+
 static void drawimage(gdImagePtr im)
 {
 	printf("Pragma: no-cache\n");
@@ -191,7 +198,7 @@ static void drawscale(
 			if (level % step5 == 0)
 				gdImageLine(im, 5, y, width - 5, y, col2);
 			else
-		    		gdImageLine(im, 10, y, width - 10, y, col2);
+				gdImageLine(im, 10, y, width - 10, y, col2);
 		}
 	}
 
@@ -200,13 +207,25 @@ static void drawscale(
 		if (level % step10 == 0) {
 			y = scale_height * (lvlhi - level) / range;
 			snprintf(lbltxt, sizeof(lbltxt), "%d", level);
-			gdImageString(im, gdFontMediumBold, width - strlen(lbltxt)*gdFontMediumBold->w, y,
-				(unsigned char *) lbltxt, scale_num_color);
+			gdImageString(im, gdFontMediumBold,
+				width - (int)(strlen(lbltxt)) * gdFontMediumBold->w,
+				y, (unsigned char *) lbltxt, scale_num_color);
 		}
 	}
 }
 
 /* draws the bar style indicator */
+static void drawbar(
+	int lvllo, int lvlhi,			/* min and max numbers on the scale */
+	int step, int step5, int step10,	/* steps for minor, submajor and major dashes */
+	int redlo1, int redhi1,			/* first red zone start and end */
+	int redlo2, int redhi2,			/* second red zone start and end */
+	int grnlo, int grnhi,			/* green zone start and end */
+	double value, 				/* UPS variable value to draw */
+	const char *format			/* printf style format to be used when rendering summary text */
+)
+	__attribute__((noreturn));
+
 static void drawbar(
 	int lvllo, int lvlhi,			/* min and max numbers on the scale */
 	int step, int step5, int step10,	/* steps for minor, submajor and major dashes */
@@ -240,7 +259,7 @@ static void drawbar(
 	summary_color	= color_alloc(im, get_imgarg("summary_col"));
 
 	/* rescale UPS value to fit in the scale */
-	bar_y = (1 - (value - lvllo) / (lvlhi - lvllo)) * scale_height;
+	bar_y = (int)((1.0 - (value - lvllo) / (lvlhi - lvllo)) * scale_height);
 
 	/* sanity checks: */
 
@@ -257,9 +276,21 @@ static void drawbar(
 		bar_color);
 
 	/* stick the text version of the value at the bottom center */
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	snprintf(text, sizeof(text), format, value);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 	gdImageString(im, gdFontMediumBold,
-		(width - strlen(text)*gdFontMediumBold->w)/2,
+		(width - (int)(strlen(text))*gdFontMediumBold->w)/2,
 		height - gdFontMediumBold->h,
 		(unsigned char *) text, summary_color);
 
@@ -269,6 +300,9 @@ static void drawbar(
 }
 
 /* draws the error image */
+static void noimage(const char *fmt, ...)
+	__attribute__((noreturn));
+
 static void noimage(const char *fmt, ...)
 {
 	gdImagePtr	im;
@@ -278,7 +312,19 @@ static void noimage(const char *fmt, ...)
 	va_list		ap;
 
 	va_start(ap, fmt);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	vsnprintf(msg, sizeof(msg), fmt, ap);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 	va_end(ap);
 
 	width = get_imgarg("width");
@@ -293,17 +339,23 @@ static void noimage(const char *fmt, ...)
 
 	if (width > height)
 		gdImageString(im, gdFontMediumBold,
-			(width - strlen(msg)*gdFontMediumBold->w)/2,
+			(width - (int)(strlen(msg))*gdFontMediumBold->w)/2,
 			(height - gdFontMediumBold->h)/2,
 			(unsigned char *) msg, 	summary_color);
 	else
 		gdImageStringUp(im, gdFontMediumBold,
 			(width - gdFontMediumBold->h)/2,
-			(height + strlen(msg)*gdFontMediumBold->w)/2,
+			(height + (int)(strlen(msg))*gdFontMediumBold->w)/2,
 			(unsigned char *) msg, summary_color);
 
 	drawimage(im);
 
+	/* NOTE: Earlier code called noimage() and then exit(EXIT_FAILURE);
+	 * to signal an error via process exit code. Now that drawimage()
+	 * always ends with exit(EXIT_SUCCESS) - which might make webserver
+	 * feel good - the command-line use if any suffers no error returns.
+	 */
+
 	/* NOTREACHED */
 }
 
@@ -311,6 +363,10 @@ static void noimage(const char *fmt, ...)
    UPS variable can be determined.
    deviation < 0 means that values below nom should be grey instead of
    green */
+static void drawgeneralbar(double var, int min, int nom, int max,
+		int deviation, 	const char *format)
+	__attribute__((noreturn));
+
 static void drawgeneralbar(double var, int min, int nom, int max,
 		int deviation, 	const char *format)
 {
@@ -370,6 +426,10 @@ static void drawgeneralbar(double var, int min, int nom, int max,
 }
 
 /* draws input and output voltage bar style indicators */
+static void draw_utility(double var, int min, int nom, int max,
+		int deviation, const char *format)
+	__attribute__((noreturn));
+
 static void draw_utility(double var, int min, int nom, int max,
 		int deviation, const char *format)
 {
@@ -405,7 +465,7 @@ static void draw_utility(double var, int min, int nom, int max,
 
 	/* Acceptable range of voltage is 85%-110% of nominal voltage
 	 * in EU at least. Be conservative and say +-10% */
-	deviation = nom*0.1;
+	deviation = (int)(nom * 0.1);
 
 	drawgeneralbar(var, min, nom, max, deviation, format);
 
@@ -413,9 +473,17 @@ static void draw_utility(double var, int min, int nom, int max,
 }
 
 /* draws battery.percent bar style indicator */
-static void draw_battpct(double var, int min, int nom, int max,
-		int deviation, const char *format)
+static void draw_battpct(double var, int min, int nom,
+		int max, int deviation, const char *format)
+	__attribute__((noreturn));
+
+static void draw_battpct(double var, int min, int nom,
+		int max, int deviation, const char *format)
 {
+	NUT_UNUSED_VARIABLE(nom);
+	NUT_UNUSED_VARIABLE(max);
+	NUT_UNUSED_VARIABLE(deviation);
+
 	if (min < 0) {
 		min = 50;
 	}
@@ -424,6 +492,10 @@ static void draw_battpct(double var, int min, int nom, int max,
 }
 
 /* draws battery.voltage bar style indicator */
+static void draw_battvolt(double var, int min, int nom, int max,
+		int deviation, const char *format)
+	__attribute__((noreturn));
+
 static void draw_battvolt(double var, int min, int nom, int max,
 		int deviation, const char *format)
 {
@@ -450,18 +522,17 @@ static void draw_battvolt(double var, int min, int nom, int max,
 	}
 
 	if(min == -1) {
-		min = nom/2*1.6+1; /* Assume a 2V cell is dead at 1.6V */
+		min = (int)(nom/2*1.6+1); /* Assume a 2V cell is dead at 1.6V */
 	}
 
 	if(max == -1) {
-		max = nom/2*2.3+1; /* Assume 2.3V float charge voltage */
+		max = (int)(nom/2*2.3+1); /* Assume 2.3V float charge voltage */
 	}
 
 	if (nom < min || nom > max)
-	    	nom = -1;
+		nom = -1;
 
-
-	deviation = -(nom*0.05); /* 5% deviation from nominal voltage */
+	deviation = (int)(-nom*0.05); /* 5% deviation from nominal voltage */
 	if(deviation==0) {
 		deviation = -1;
 	}
@@ -470,33 +541,57 @@ static void draw_battvolt(double var, int min, int nom, int max,
 }
 
 /* draws ups.load bar style indicator */
-static void draw_upsload(double var, int min, int nom, int max,
+static void draw_upsload(double var, int min,
+		int nom, int max,
+		int deviation, const char *format)
+	__attribute__((noreturn));
+
+static void draw_upsload(double var, int min,
+		int nom, int max,
 		int deviation, const char *format)
 {
+	NUT_UNUSED_VARIABLE(min);
+	NUT_UNUSED_VARIABLE(nom);
+	NUT_UNUSED_VARIABLE(max);
+	NUT_UNUSED_VARIABLE(deviation);
+
 	drawbar(0, 125, 5, 5, 25, 100, 125, -1, -1, 0, 50, var, format);
 }
 
 /* draws temperature bar style indicator */
+static void draw_temperature(double var, int min, int nom, int max,
+		int deviation, const char *format)
+	__attribute__((noreturn));
+
 static void draw_temperature(double var, int min, int nom, int max,
 		int deviation, const char *format)
 {
 	int	hi = get_imgarg("tempmax");
 	int	lo = get_imgarg("tempmin");
+	NUT_UNUSED_VARIABLE(nom);
+	NUT_UNUSED_VARIABLE(deviation);
 
 	drawbar(lo, hi, 1, 5, 10, lo, min, max, hi, -1, -1, var, format);
 }
 
 /* draws humidity bar style indicator */
+static void draw_humidity(double var, int min, int nom, int max,
+		int deviation, const char *format)
+	__attribute__((noreturn));
+
 static void draw_humidity(double var, int min, int nom, int max,
 		int deviation, const char *format)
 {
+	NUT_UNUSED_VARIABLE(nom);
+	NUT_UNUSED_VARIABLE(deviation);
+
 	drawbar(0, 100, 2, 10, 20, 0, min, max, 100, -1, -1, var, format);
 }
 
 static int get_var(const char *var, char *buf, size_t buflen)
 {
 	int	ret;
-	unsigned int	numq, numa;
+	size_t	numq, numa;
 	const	char	*query[4];
 	char	**answer;
 
@@ -523,6 +618,8 @@ int main(int argc, char **argv)
 	char	str[SMALLBUF];
 	int	i, min, nom, max;
 	double	var = 0;
+	NUT_UNUSED_VARIABLE(argc);
+	NUT_UNUSED_VARIABLE(argv);
 
 	extractcgiargs();
 
@@ -537,13 +634,17 @@ int main(int argc, char **argv)
 
 	if (upscli_splitname(monhost, &upsname, &hostname, &port) != 0) {
 		noimage("Invalid UPS definition (upsname[@hostname[:port]])\n");
-		exit(EXIT_FAILURE);
+#ifndef HAVE___ATTRIBUTE__NORETURN
+		exit(EXIT_FAILURE);	/* Should not get here in practice, but compiler is afraid we can fall through */
+#endif
 	}
 
 	if (upscli_connect(&ups, hostname, port, 0) < 0) {
 		noimage("Can't connect to server:\n%s\n",
 			upscli_strerror(&ups));
-		exit(EXIT_FAILURE);
+#ifndef HAVE___ATTRIBUTE__NORETURN
+		exit(EXIT_FAILURE);	/* Should not get here in practice, but compiler is afraid we can fall through */
+#endif
 	}
 
 	for (i = 0; imgvar[i].name; i++)
@@ -553,7 +654,9 @@ int main(int argc, char **argv)
 			   registered with this variable */
 			if (!imgvar[i].drawfunc) {
 				noimage("Draw function N/A");
-				exit(EXIT_FAILURE);
+#ifndef HAVE___ATTRIBUTE__NORETURN
+				exit(EXIT_FAILURE);	/* Should not get here in practice, but compiler is afraid we can fall through */
+#endif
 			}
 
 			/* get the variable value */
@@ -564,7 +667,9 @@ int main(int argc, char **argv)
 				snprintf(str, sizeof(str), "%s N/A",
 					imgvar[i].name);
 				noimage(str);
-				exit(EXIT_FAILURE);
+#ifndef HAVE___ATTRIBUTE__NORETURN
+				exit(EXIT_FAILURE);	/* Should not get here in practice, but compiler is afraid we can fall through */
+#endif
 			}
 
 			/* when getting minimum, nominal and maximum values,
@@ -617,7 +722,9 @@ int main(int argc, char **argv)
 		}
 
 	noimage("Unknown display");
+#ifndef HAVE___ATTRIBUTE__NORETURN
 	exit(EXIT_FAILURE);
+#endif
 }
 
 imgvar_t imgvar[] = {
