@@ -1,5 +1,8 @@
 /* upslog.h - table of functions for handling various logging functions */
 
+#ifndef NUT_UPSLOG_H_SEEN
+#define NUT_UPSLOG_H_SEEN 1
+
 #ifdef __cplusplus
 /* *INDENT-OFF* */
 extern "C" {
@@ -20,7 +23,10 @@ static void do_time(const char *arg);
 static void do_var(const char *arg);
 static void do_etime(const char *arg);
 
-struct {
+/* This is only used in upslog.c, but refers to routines declared here...
+ * To move or not to move?..
+ */
+static struct {
 	const	char	*name;
 	void	(*func)(const char *arg);
 }	logcmds[] =
@@ -31,7 +37,7 @@ struct {
 	{ "TIME",	do_time			},
 	{ "VAR",	do_var			},
 	{ "ETIME",	do_etime		},
-	{ NULL,		(void(*)())(NULL)	}
+	{ NULL,		(void(*)(const char*))(NULL)	}
 };
 
 #ifdef __cplusplus
@@ -40,3 +46,4 @@ struct {
 /* *INDENT-ON* */
 #endif
 
+#endif	/* NUT_UPSLOG_H_SEEN */
