@@ -19,11 +19,12 @@
  */
 
 #include "common.h"
+#include "nut_stdint.h"
+#include "timehead.h"
 #include "upsclient.h"
 #include "status.h"
 #include "cgilib.h"
 #include "parseconf.h"
-#include "timehead.h"
 #include "upsstats.h"
 #include "upsimagearg.h"
 
@@ -36,7 +37,7 @@ static int	use_celsius = 1, refreshdelay = -1, treemode = 0;
 	/* from cgilib's checkhost() */
 static char	*monhostdesc = NULL;
 
-static int	port;
+static uint16_t	port;
 static char	*upsname, *hostname;
 static char	*upsimgpath="upsimage.cgi", *upsstatpath="upsstats.cgi";
 static UPSCONN_t	ups;
@@ -103,7 +104,7 @@ static int check_ups_fd(int do_report)
 static int get_var(const char *var, char *buf, size_t buflen, int verbose)
 {
 	int	ret;
-	unsigned int	numq, numa;
+	size_t	numq, numa;
 	const	char	*query[4];
 	char	**answer;
 
@@ -134,7 +135,7 @@ static int get_var(const char *var, char *buf, size_t buflen, int verbose)
 	if (numa < numq) {
 		if (verbose)
 			printf("[Invalid response]\n");
-		
+
 		return 0;
 	}
 
@@ -195,9 +196,10 @@ static int do_date(const char *buf)
 {
 	char	datebuf[SMALLBUF];
 	time_t	tod;
+	struct tm tmbuf;
 
 	time(&tod);
-	if (strftime(datebuf, sizeof(datebuf), buf, localtime(&tod))) {
+	if (strftime(datebuf, sizeof(datebuf), buf, localtime_r(&tod, &tmbuf))) {
 		printf("%s", datebuf);
 		return 1;
 	}
@@ -291,12 +293,12 @@ static int do_img(char *buf)
 
 	/* only allow known types through */
 
-	if (!strcmp(type, "input.voltage") 
-			|| !strcmp(type, "input.L1-N.voltage") 
-			|| !strcmp(type, "input.L2-N.voltage") 
+	if (!strcmp(type, "input.voltage")
+			|| !strcmp(type, "input.L1-N.voltage")
+			|| !strcmp(type, "input.L2-N.voltage")
 			|| !strcmp(type, "input.L3-N.voltage")
-			|| !strcmp(type, "input.L1-L2.voltage") 
-			|| !strcmp(type, "input.L2-L3.voltage") 
+			|| !strcmp(type, "input.L1-L2.voltage")
+			|| !strcmp(type, "input.L2-L3.voltage")
 			|| !strcmp(type, "input.L3-L1.voltage")) {
 		return get_img_val(type, "Input voltage", imgargs);
 	}
@@ -308,11 +310,11 @@ static int do_img(char *buf)
 		return get_img_val(type, "Battery charge", imgargs);
 
 	if (!strcmp(type, "output.voltage")
-			|| !strcmp(type, "output.L1-N.voltage") 
-			|| !strcmp(type, "output.L2-N.voltage") 
+			|| !strcmp(type, "output.L1-N.voltage")
+			|| !strcmp(type, "output.L2-N.voltage")
 			|| !strcmp(type, "output.L3-N.voltage")
-			|| !strcmp(type, "output.L1-L2.voltage") 
-			|| !strcmp(type, "output.L2-L3.voltage") 
+			|| !strcmp(type, "output.L1-L2.voltage")
+			|| !strcmp(type, "output.L2-L3.voltage")
 			|| !strcmp(type, "output.L3-L1.voltage")) {
 		return get_img_val(type, "Output voltage", imgargs);
 	}
@@ -349,7 +351,7 @@ static void ups_connect(void)
 {
 	static ulist_t	*lastups = NULL;
 	char	*newups, *newhost;
-	int	newport;
+	uint16_t	newport;
 
 	/* try to minimize reconnects */
 	if (lastups) {
@@ -363,13 +365,13 @@ static void ups_connect(void)
 		/* see if it's just on the same host */
 		newups = newhost = NULL;
 
-		if (upscli_splitname(currups->sys, &newups, &newhost, 
+		if (upscli_splitname(currups->sys, &newups, &newhost,
 			&newport) != 0) {
 			printf("Unusable UPS definition [%s]\n", currups->sys);
-			fprintf(stderr, "Unusable UPS definition [%s]\n", 
+			fprintf(stderr, "Unusable UPS definition [%s]\n",
 				currups->sys);
 			exit(EXIT_FAILURE);
-		}		
+		}
 
 		if ((!strcmp(newhost, hostname)) && (port == newport)) {
 			free(upsname);
@@ -564,7 +566,7 @@ static void do_upsimgpath(const char *s) {
 static void do_temp(const char *var)
 {
 	char	tempc[SMALLBUF];
-	float	tempf;
+	double	tempf;
 
 	if (!get_var(var, tempc, sizeof(tempc), 1))
 		return;
@@ -807,7 +809,8 @@ static int do_command(char *cmd)
 static void parse_line(const char *buf)
 {
 	char	cmd[SMALLBUF];
-	int	i, len, do_cmd = 0;
+	size_t	i, len;
+	char	do_cmd = 0;
 
 	for (i = 0; buf[i]; i += len) {
 
@@ -824,9 +827,10 @@ static void parse_line(const char *buf)
 			i++;	/* skip over the '@' character */
 			continue;
 		}
+		assert (len < INT_MAX);
 
 		if (do_cmd) {
-			snprintf(cmd, sizeof(cmd), "%.*s", len, &buf[i]);
+			snprintf(cmd, sizeof(cmd), "%.*s", (int)len, &buf[i]);
 			continue;
 		}
 
@@ -836,13 +840,13 @@ static void parse_line(const char *buf)
 		}
 
 		/* pass it trough */
-		printf("%.*s", len, &buf[i]);
+		printf("%.*s", (int)len, &buf[i]);
 	}
 }
 
 static void display_template(const char *tfn)
 {
-	char	fn[SMALLBUF], buf[LARGEBUF];	
+	char	fn[SMALLBUF], buf[LARGEBUF];
 
 	snprintf(fn, sizeof(fn), "%s/%s", confpath(), tfn);
 
@@ -865,7 +869,7 @@ static void display_template(const char *tfn)
 
 static void display_tree(int verbose)
 {
-	unsigned int	numq, numa;
+	size_t	numq, numa;
 	const	char	*query[4];
 	char	**answer;
 
@@ -890,7 +894,7 @@ static void display_tree(int verbose)
 	printf("<HTML>\n");
 	printf("<HEAD><TITLE>upsstat: data tree of %s</TITLE></HEAD>\n", currups->desc);
 
-	printf("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000EE\" VLINK=\"#551A8B\">\n"); 
+	printf("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000EE\" VLINK=\"#551A8B\">\n");
 
 	printf("<TABLE BGCOLOR=\"#50A0A0\" ALIGN=\"CENTER\">\n");
 	printf("<TR><TD>\n");
@@ -910,12 +914,12 @@ static void display_tree(int verbose)
 		if (numa < 4) {
 			if (verbose)
 				printf("[Invalid response]\n");
-		
+
 			return;
 		}
 
 		printf("<TR BGCOLOR=\"#60B0B0\" ALIGN=\"LEFT\">\n");
-	
+
 		printf("<TD>%s</TD>\n", answer[2]);
 		printf("<TD>:</TD>\n");
 		printf("<TD>%s<br></TD>\n", answer[3]);
@@ -936,7 +940,7 @@ static void add_ups(char *sys, char *desc)
 
 	tmp = last = ulhead;
 
-	while (tmp) { 
+	while (tmp) {
 		last = tmp;
 		tmp = tmp->next;
 	}
@@ -1041,9 +1045,12 @@ static void display_single(void)
 
 int main(int argc, char **argv)
 {
+	NUT_UNUSED_VARIABLE(argc);
+	NUT_UNUSED_VARIABLE(argv);
+
 	extractcgiargs();
 
-	printf("Content-type: text/html\n"); 
+	printf("Content-type: text/html\n");
 	printf("Pragma: no-cache\n");
 	printf("\n");
 
