@@ -395,6 +395,12 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
         break;
       case 'p':
         strvalue = va_arg (args, void *);
+        /* FIXME: in 64-bit (and Windows-targeted) builds this code yields:
+         * warning: cast from pointer to integer of different size
+         * so probably prints a truncated pointer value. Should check
+         * if sizeof(void*) == sizeof(long) and output e.g. two pieces
+         * (lower and upper long for bytes of the pointer).
+         */
         fmtint (buffer, &currlen, maxlen, (long) strvalue, 16, min, max, flags);
         break;
       case 'n':
@@ -593,11 +599,15 @@ static LDOUBLE abs_val (LDOUBLE value)
 }
 #endif
 
-#ifndef HAVE_FCVT
+#ifndef WIN32
+# ifndef HAVE_FCVT
 /* The two routines that may get defined below are only used if we also don't
  * have a fcvt() in the system. Defining and not using the routines may be a
- * warning (fatal with -Werror), so we hide them here. */
-# ifndef HAVE_POW10
+ * warning (fatal with -Werror), so we hide them here.
+ * FIXME: They are blindly expected (assumed?) to be available on Windows,
+ * maybe better trust configure script macros on this?
+ */
+#  if ! HAVE_DECL_POW10
 static LDOUBLE pow10 (int exp)
 {
   LDOUBLE result = 1;
@@ -610,9 +620,9 @@ static LDOUBLE pow10 (int exp)
 
   return result;
 }
-# endif
+#  endif /* HAVE_DECL_POW10 */
 
-# ifndef HAVE_ROUND
+#  if ! HAVE_DECL_ROUND
 static long round (LDOUBLE value)
 {
   long intpart;
@@ -624,8 +634,9 @@ static long round (LDOUBLE value)
 
   return intpart;
 }
-# endif
-#endif /* HAVE_FCVT */
+#  endif /* HAVE_DECL_ROUND */
+# endif /* HAVE_FCVT */
+#endif /* WIN32 */
 
 static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
                    LDOUBLE fvalue, int min, int max, int flags)
@@ -962,4 +973,3 @@ static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c)
   printf ("%d tests failed out of %d.\n", fail, num);
 }
 #endif /* SNPRINTF_TEST */
-
