@@ -144,7 +144,7 @@ static snmp_info_t apc_ats_mib[] = {
 	{ "outlet.group.%i.realpower", 0, 1, ".1.3.6.1.4.1.318.1.1.8.5.4.5.1.15.%i", NULL, SU_FLAG_NEGINVALID | SU_OUTLET_GROUP, NULL },
 
 
-#if 0 /* FIXME: Remaining data to be processed */
+#if WITH_UNMAPPED_DATA_POINTS /* FIXME: Remaining data to be processed */
 	/* atsIdentDeviceRating.0 = INTEGER: 32 */
 	{ "unmapped.atsIdentDeviceRating", 0, 1, ".1.3.6.1.4.1.318.1.1.8.1.9.0", NULL, SU_FLAG_OK, NULL },
 	/* atsCalibrationNumInputs.0 = INTEGER: 2 */
@@ -444,7 +444,7 @@ static snmp_info_t apc_ats_mib[] = {
 	{ "unmapped.atsOutputBankMinPercentPower", 0, 1, ".1.3.6.1.4.1.318.1.1.8.5.4.5.1.20.2", NULL, SU_FLAG_OK, NULL },
 	/* atsOutputBankMinPercentPower.3 = INTEGER: -1 */
 	{ "unmapped.atsOutputBankMinPercentPower", 0, 1, ".1.3.6.1.4.1.318.1.1.8.5.4.5.1.20.3", NULL, SU_FLAG_OK, NULL },
-#endif /* 0 */
+#endif	/* if WITH_UNMAPPED_DATA_POINTS */
 
 	/* end of structure. */
 	{ NULL, 0, 0, NULL, NULL, 0, NULL }
