@@ -23,6 +23,7 @@ upsdrv_info_t comm_upsdrv_info = {
 };
 
 #define MAX_TRY 4
+#define MAX_TRY_OPENUSB 32
 
 /* Powerware */
 #define POWERWARE 0x0592
@@ -243,7 +244,7 @@ ssize_t get_answer(unsigned char *data, unsigned char command)
 
 		/* Check data length byte (remove the header length) */
 		length = my_buf[2];
-		upsdebugx(3, "get_answer: data length = %zu", length);
+		upsdebugx(3, "get_answer: data length = %" PRIuSIZE, length);
 		if (bytes_read < (length + PW_HEADER_SIZE)) {
 			if (need_data < 0) --need_data; /* count zerro byte too */
 			need_data += length + 1; /* packet lenght + checksum */
@@ -305,7 +306,7 @@ ssize_t get_answer(unsigned char *data, unsigned char command)
 		else if (tail == 0)
 			my_buf = &buf[0];
 		else { /* if (tail < 0) */
-			upsdebugx(1, "get_answer(): did not expect to get negative tail size: %zd", tail);
+			upsdebugx(1, "get_answer(): did not expect to get negative tail size: %" PRIiSIZE, tail);
 			return -1;
 		}
 
@@ -384,13 +385,18 @@ void upsdrv_cleanup(void)
 
 void upsdrv_reconnect(void)
 {
+	dstate_setinfo("driver.state", "reconnect.trying");
+
 	upsdebugx(4, "==================================================");
 	upsdebugx(4, "= device has been disconnected, try to reconnect =");
 	upsdebugx(4, "==================================================");
 
 	nutusb_close(upsdev, "USB");
 	upsdev = NULL;
+
 	upsdrv_initups();
+
+	dstate_setinfo("driver.state", "quiet");
 }
 
 /* USB functions */
@@ -417,7 +423,8 @@ static usb_dev_handle *open_powerware_usb(void)
 	ssize_t devcount = 0;
 	libusb_device_handle *udev;
 	struct libusb_device_descriptor dev_desc;
-	uint8_t bus;
+	uint8_t bus_num;
+	/* TODO: consider device_addr */
 	int i;
 
 	devcount = libusb_get_device_list(NULL, &devlist);
@@ -435,13 +442,13 @@ static usb_dev_handle *open_powerware_usb(void)
 
 		curDevice.VendorID = dev_desc.idVendor;
 		curDevice.ProductID = dev_desc.idProduct;
-		bus = libusb_get_bus_number(device);
+		bus_num = libusb_get_bus_number(device);
 		curDevice.Bus = (char *)malloc(4);
 		if (curDevice.Bus == NULL) {
 			libusb_free_device_list(devlist, 1);
 			fatal_with_errno(EXIT_FAILURE, "Out of memory");
 		}
-		sprintf(curDevice.Bus, "%03d", bus);
+		sprintf(curDevice.Bus, "%03d", bus_num);
 
 		/* FIXME: we should also retrieve
 		 * dev->descriptor.iManufacturer
@@ -513,6 +520,7 @@ usb_dev_handle *nutusb_open(const char *port)
 	usb_find_devices();
 #endif /* WITH_LIBUSB_1_0 */
 
+	/* for (retry = 0; dev_h == NULL && retry < MAX_TRY_OPENUSB; retry++) */
 	for (retry = 0; retry < MAX_TRY ; retry++)
 	{
 		dev_h = open_powerware_usb();
@@ -524,6 +532,13 @@ usb_dev_handle *nutusb_open(const char *port)
 			upsdebugx(1, "device %s opened successfully", curDevice.Bus);
 			errout = 0;
 
+#ifdef WIN32
+			if ((ret = usb_set_configuration(dev_h, 1)) < 0)
+			{
+				upsdebugx(1, "Can't set POWERWARE USB configuration: %s", nut_usb_strerror(ret));
+				errout = 1;
+			}
+#endif
 			if ((ret = usb_claim_interface(dev_h, 0)) < 0)
 			{
 				upsdebugx(1, "Can't claim POWERWARE USB interface: %s", nut_usb_strerror(ret));
