@@ -94,7 +94,7 @@
 #include "serial.h"
 
 #define DRIVER_NAME	"Belkin 'Universal UPS' driver"
-#define DRIVER_VERSION	"0.08"
+#define DRIVER_VERSION	"0.09"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -159,7 +159,7 @@ upsdrv_info_t upsdrv_info = {
 #define BS_REPLACE   0x80
 
 /* size of an array */
-#define asize(x) ((int)(sizeof(x)/sizeof(x[0])))
+#define asize(x) ((int)(SIZEOF_ARRAY(x)))
 
 static const char *upstype[3] = {
 	"ONLINE",
@@ -447,17 +447,19 @@ static int belkin_nut_write_int(unsigned char reg, int val) {
    will be discarded. After this call, the device is ready for reading
    and writing via read(2) and write(2). Return a valid file
    descriptor on success, or else -1 with errno set. */
-static int belkin_std_open_tty(const char *device) {
-	int fd;
+static TYPE_FD_SER belkin_std_open_tty(const char *device) {
+	TYPE_FD_SER fd;
 	struct termios tios;
+#ifndef WIN32
 	struct flock flock;
+#endif
 	char buf[128];
 	ssize_t r;
 
 	/* open the device */
 	fd = open(device, O_RDWR | O_NONBLOCK);
-	if (fd == -1) {
-		return -1;
+	if (INVALID_FD_SER(fd)) {
+		return ERROR_FD_SER;
 	}
 
 	/* set communications parameters: 2400 baud, 8 bits, 1 stop bit, no
@@ -470,7 +472,7 @@ static int belkin_std_open_tty(const char *device) {
 	r = tcsetattr(fd, TCSANOW, &tios);
 	if (r == -1) {
 		close(fd);
-		return -1;
+		return ERROR_FD_SER;
 	}
 
 	/* signal the UPS to enter "smart" mode. This is done by setting RTS
@@ -487,17 +489,20 @@ static int belkin_std_open_tty(const char *device) {
 	r = ser_flush_io(fd);
 	if (r == -1) {
 		close(fd);
-		return -1;
+		return ERROR_FD_SER;
 	}
 
+/* TODO: port to WIN32 */
+#ifndef WIN32
 	/* lock the port */
 	memset(&flock, 0, sizeof(flock));
 	flock.l_type = F_RDLCK;
 	r = fcntl(fd, F_SETLK, &flock);
 	if (r == -1) {
 		close(fd);
-		return -1;
+		return ERROR_FD_SER;
 	}
+#endif
 
 	/* sleep at least 0.25 seconds for the UPS to wake up. Belkin's own
 	   software sleeps 1 second, so that's what we do, too. */
@@ -508,13 +513,20 @@ static int belkin_std_open_tty(const char *device) {
 	r = tcflush(fd, TCIFLUSH);
 	if (r == -1) {
 		close(fd);
-		return -1;
+		return ERROR_FD_SER;
 	}
 
+#ifndef WIN32
 	r = read(fd, buf, 127);
+#else
+/* WIN32 : w32_serial_read is blocking, using select_read with 0ms timeout
+ * is non-blocking */
+	r = select_read(fd, buf, 127, 0, 0);
+#endif
+
 	if (r == -1 && errno != EAGAIN) {
 		close(fd);
-		return -1;
+		return ERROR_FD_SER;
 	}
 
 	/* leave port in non-blocking state */
@@ -523,13 +535,19 @@ static int belkin_std_open_tty(const char *device) {
 }
 
 /* blocking read with 1-second timeout (use non-blocking i/o) */
-static int belkin_std_upsread(int fd, unsigned char *buf, int n) {
+static int belkin_std_upsread(TYPE_FD_SER fd, unsigned char *buf, int n) {
 	int count = 0;
 	ssize_t r;
 	int tries = 0;
 
 	while (count < n) {
+#ifndef WIN32
 		r = read(fd, &buf[count], (size_t)(n-count));
+#else
+		/* WIN32 : w32_serial_read is blocking, using select_read
+		 * with 0ms timeout is non-blocking */
+		r = select_read(fd, buf, (size_t)(n-count), 0, 0);
+#endif
 		if (r==-1 && errno==EAGAIN) {
 			/* non-blocking i/o, no data available */
 			usleep(100000);
@@ -547,7 +565,7 @@ static int belkin_std_upsread(int fd, unsigned char *buf, int n) {
 }
 
 /* blocking write with 1-second timeout (use non-blocking i/o) */
-static int belkin_std_upswrite(int fd, unsigned char *buf, int n) {
+static int belkin_std_upswrite(TYPE_FD_SER fd, unsigned char *buf, int n) {
 	int count = 0;
 	ssize_t r;
 	int tries = 0;
@@ -573,7 +591,7 @@ static int belkin_std_upswrite(int fd, unsigned char *buf, int n) {
 /* receive Belkin message from UPS, check for well-formedness (leading
    byte, checksum). Return length of message, or -1 if not
    well-formed */
-static int belkin_std_receive(int fd, unsigned char *buf, int bufsize) {
+static int belkin_std_receive(TYPE_FD_SER fd, unsigned char *buf, int bufsize) {
 	int r;
 	int n=0;
 	int len;
@@ -619,7 +637,7 @@ static int belkin_std_receive(int fd, unsigned char *buf, int bufsize) {
 
 /* read the value of an integer register from UPS. Return -1 on
    failure. */
-static int belkin_std_read_int(int fd, unsigned char reg) {
+static int belkin_std_read_int(TYPE_FD_SER fd, unsigned char reg) {
 	unsigned char buf[MAXMSGSIZE];
 	int len, r;
 
@@ -661,7 +679,7 @@ static int belkin_std_read_int(int fd, unsigned char reg) {
 
 /* write the value of an integer register to UPS. Return -1 on
    failure, else 0 */
-static int belkin_std_write_int(int fd, unsigned char reg, int val) {
+static int belkin_std_write_int(TYPE_FD_SER fd, unsigned char reg, int val) {
 	unsigned char buf[MAXMSGSIZE];
 	int r;
 
@@ -766,7 +784,7 @@ static int belkin_wait(void)
 	char *val;
 	int failcount = 0;  /* count consecutive failed connection attempts */
 	int failerrno = 0;
-	int fd;
+	TYPE_FD_SER fd;
 	int r;
 	int bs, ov, bl, st;
 
@@ -792,7 +810,7 @@ static int belkin_wait(void)
 
 	updatestatus(smode, "Connecting to UPS...");
 	failcount = 0;
-	fd = -1;
+	fd = ERROR_FD_SER;
 
 	while (1) {
 		if (failcount >= 3 && nohang) {
@@ -802,10 +820,10 @@ static int belkin_wait(void)
 		} else if (failcount >= 3) {
 			updatestatus(smode, "UPS is not responding, will keep trying: %s", strerror(failerrno));
 		}
-		if (fd == -1) {
+		if (INVALID_FD_SER(fd)) {
 			fd = belkin_std_open_tty(device_path);
 		}
-		if (fd == -1) {
+		if (INVALID_FD_SER(fd)) {
 			failcount++;
 			failerrno = errno;
 			sleep(1);
@@ -819,7 +837,7 @@ static int belkin_wait(void)
 			failcount++;
 			failerrno = errno;
 			close(fd);
-			fd = -1;
+			fd = ERROR_FD_SER;
 			sleep(1);
 			continue;
 		}
@@ -828,7 +846,7 @@ static int belkin_wait(void)
 			failcount++;
 			failerrno = errno;
 			close(fd);
-			fd = -1;
+			fd = ERROR_FD_SER;
 			sleep(1);
 			continue;
 		}
@@ -837,7 +855,7 @@ static int belkin_wait(void)
 			failcount++;
 			failerrno = errno;
 			close(fd);
-			fd = -1;
+			fd = ERROR_FD_SER;
 			sleep(1);
 			continue;
 		}
