@@ -35,7 +35,7 @@
 #endif
 
 #define DRIVER_NAME     "Best Fortress UPS driver"
-#define DRIVER_VERSION  "0.06"
+#define DRIVER_VERSION  "0.08"
 
 /* driver description structure */
 upsdrv_info_t   upsdrv_info = {
@@ -47,12 +47,39 @@ upsdrv_info_t   upsdrv_info = {
 	{ NULL }
 };
 
+/*
+ * Choose 20s for off delay, reading the tea leaves from other drivers.
+ */
+static const char *shutdown_delay = "20";
+
+/*
+ * Logging plan:
+ * CRIT: executing poweroff
+ * ERR: indicates a bug in nut
+ * WARNING: indicates a serious misconfiguration that will cause nut
+ *           to fail to function
+ * NOTICE: rare situations without a planned reporting path (none so
+ *         far)
+ * INFO: One message each for startup, first establishment of comms
+ * DEBUG: Loss/restore of comms, and perhaps things that are normally
+ *        reported via other daemons (on battery, low battery)
+ * D1: Things that happen rarely (startup/shutdown, param set, instant
+ *     commands)
+ * D2: ?
+ * D3: trace of communications
+ * D4: more io than necessary to see working communications
+ * D5: log even succesful acks
+ */
+
 static int instcmd (const char *cmdname, const char *extra);
 static int upsdrv_setvar (const char *varname, const char *val);
 
-/* rated VA load if known */
+/* Rated maximum VA output as configured by the user. */
 static int maxload = 0;
 
+/*
+ * NB: Not called for shutdown.
+ */
 void upsdrv_initinfo(void)
 {
 	dstate_setinfo("ups.mfr", "Best Power");
@@ -64,7 +91,6 @@ void upsdrv_initinfo(void)
 	if (maxload)
 		dstate_setinfo("ups.load", "0");
 	dstate_setinfo("output.voltamps", "0");
-	dstate_setinfo("ups.delay.shutdown", "10");	/* write only */
 
 	/* tunable via front panel: (european voltage level)
 	   parameter        factory default  range
@@ -197,12 +223,23 @@ static int upssend(const char *fmt,...) {
 #endif
 	va_end(ap);
 
+	/* Why do we not upsflushin here? */
+
+	upsdebugx(3, "%s: sending %d bytes <%s>", __func__, ret, buf);
+
 	if ((ret < 1) || (ret >= (int) sizeof(buf)))
-		upslogx(LOG_WARNING, "ser_send_pace: vsnprintf needed more "
-				"than %d bytes", (int)sizeof(buf));
+		upslogx(LOG_ERR, "%s: vsnprintf needed more than %d bytes",
+			__func__, (int) sizeof(buf));
 
 	for (p = buf; *p && sent < INT_MAX - 1; p++) {
+#ifndef WIN32
 		if (write(upsfd, p, 1) != 1)
+#else
+		DWORD bytes_written;
+		BOOL res;
+		res = WriteFile(upsfd, p, 1, &bytes_written,NULL);
+		if (res == 0 || bytes_written == 0)
+#endif
 			return -1;
 
 		/* Note: LGTM.com analysis warns that here
@@ -217,22 +254,35 @@ static int upssend(const char *fmt,...) {
 
 		sent++;
 		if (sent >= INT_MAX) {
-			upslogx(LOG_WARNING, "ser_send_pace: sent more than INT_MAX, aborting");
+			upslogx(LOG_ERR, "%s: sent >= INT_MAX, aborting",
+				__func__);
 		}
 	}
 
+	if (ret != (int) sent) {
+		upsdebugx(1, "%s: ret %d != sent %u", __func__, ret, sent);
+	}
+
 	return (int)sent;
 }
 
 static ssize_t upsrecv(char *buf,size_t bufsize,char ec,const char *ic)
 {
-	return ser_get_line(upsfd, buf, bufsize - 1, ec, ic,
-	                    SER_WAIT_SEC, SER_WAIT_USEC);
+	ssize_t nread;
+
+	nread = ser_get_line(upsfd, buf, bufsize - 1, ec, ic,
+			     SER_WAIT_SEC, SER_WAIT_USEC);
+
+	/* \todo is buf null terminated? */
+	upsdebugx(4, "%s: read %" PRIiSIZE " <%s>", __func__, nread, buf);
+
+	return nread;
 }
 
 static ssize_t upsflushin(int f, int verbose, const char *ignset)
 {
 	NUT_UNUSED_VARIABLE(f);
+	upsdebugx(4, "%s: begin", __func__);
 	return ser_flush_in(upsfd, ignset, verbose);
 }
 
@@ -248,7 +298,7 @@ void upsdrv_updateinfo(void)
 	char ch;
 	int checksum_ok = -1, is_online = 1, is_off, low_batt, trimming, boosting;
 
-	upsdebugx(1, "upsdrv_updateinfo");
+	upsdebugx(2, "%s: begin", __func__);
 
 	for (retry = 0; retry < 5; ++retry) {
 		upsflushin (0, 0, "\r ");
@@ -257,14 +307,16 @@ void upsdrv_updateinfo(void)
 		temp[2] = 0;
 		do {
 			if ((recv = upsrecv (temp+2, sizeof temp - 2, ENDCHAR, IGNCHARS)) <= 0) {
+				upsdebugx(1, "%s: upsrecv failed, "
+					  "retrying without counting", __func__);
 				upsflushin (0, 0, "\r ");
 				upssend ("f\r");
 				while (ser_get_char(upsfd, &ch, 0, UPSDELAY) > 0 && ch != '\n'); /* response starts with \r\n */
 			}
 		} while (temp[2] == 0);
 
-		upsdebugx(1, "upsdrv_updateinfo: received %zi bytes (try %i)", recv, retry);
-		upsdebug_hex(5, "buffer", temp, (size_t)recv);
+		upsdebugx(3, "%s: received %" PRIiSIZE " bytes (try %i)",
+			  __func__, recv, retry);
 
 		/* syslog (LOG_DAEMON | LOG_NOTICE,"ups: got %d chars '%s'\n", recv, temp + 2); */
 		/* status example:
@@ -282,6 +334,16 @@ void upsdrv_updateinfo(void)
 		checksum_ok = ( (checksum (temp+2) & 0xff) == 0 );
 		/* setinfo (INFO_, ""); */
 
+		if (!checksum_ok) {
+			upsdebug_hex(5, "upsdrv_updateinfo: "
+				     "checksum failure buffer hex",
+				     temp, (size_t)recv);
+			upsdebug_ascii(5, "upsdrv_updateinfo: "
+				       "checksum failure buffer ascii",
+				       temp, (size_t)recv);
+		}
+
+
 		/* I can't figure out why this is missing the first two chars.
 		   But the first two chars are not used, so just set them to zero
 		   when missing. */
@@ -294,23 +356,29 @@ void upsdrv_updateinfo(void)
 		else if (len != 80)
 			checksum_ok = 0;
 		if (checksum_ok) break;
+
+		upsdebugx(1, "%s: failed to read status try %d",
+			  __func__, retry);
 		sleep(SER_WAIT_SEC);
 	}
 
 	if (!p || len < 1 || checksum_ok < 0) {
-		upsdebugx(2, "pointer to data not initialized after processing");
+		/* \todo: Analyze/fix code and rewrite message. */
+		upsdebugx(2, "%s: pointer to data not initialized after processing",
+			__func__);
 		dstate_datastale();
 		return;
 	}
 
 	if (!checksum_ok) {
-		upsdebugx(2, "checksum corruption");
+		upsdebugx(2, "%s: checksum corruption", __func__);
 		upsdebug_hex(3, "buffer", temp, (size_t)len);
 		dstate_datastale();
 		return;
 	}
 
-	/* upslogx(LOG_INFO, "updateinfo: %s", p); */
+	/* Log assuming ASCII which it 99% is.  \todo Improve. */
+	upsdebugx(3, "%s: %s", __func__, p);
 
 	setinfo_int ("input.voltage", p+24,4);
 	setinfo_int ("output.voltage", p+28,4);
@@ -361,11 +429,23 @@ void upsdrv_updateinfo(void)
 static int setparam (int parameter, int dlen, const char * data)
 {
 	char reply[80];
+
+	upsdebugx(2, "%s: begin", __func__);
+
 	/* Note the use of "%*s" - parameter (int)dlen specifies
 	 * the string width reserved for data */
 	upssend ("p%d=%*s\r", parameter, dlen, data);
-	if (upsrecv (reply, sizeof(reply), ENDCHAR, "") < 0) return 0;
-	return strncmp (reply, "OK", 2) == 0;
+	if (upsrecv (reply, sizeof(reply), ENDCHAR, "") < 0) {
+		upsdebugx(1, "%s: did not get reply", __func__);
+		return 0;
+	}
+	if (strncmp (reply, "OK", 2) == 0) {
+		upsdebugx(5, "%s: reply OK", __func__);
+		return 1;
+	} else {
+		upsdebugx(1, "%s: reply NOT ok", __func__);
+		return 0;
+	}
 }
 
 /* ups_setsuper: set super-user access
@@ -390,7 +470,7 @@ static void autorestart (int restart)
 static int upsdrv_setvar (const char *var, const char * data) {
 	int parameter;
 	size_t len = strlen(data);
-	upsdebugx(1, "Setvar: %s %s", var, data);
+	upsdebugx(1, "%s: %s %s (%" PRIuSIZE " bytes)", __func__, var, data, len);
 	if (strcmp("input.transfer.low", var) == 0) {
 		parameter = 7;
 	}
@@ -401,7 +481,13 @@ static int upsdrv_setvar (const char *var, const char * data) {
 		parameter = 2;
 	}
 	else {
-		upslogx(LOG_INFO, "Setvar: unsettable variable %s", var);
+		/*
+		 * \todo Figure out if we get here by a code error or
+		 * by the user asking for a variable that does not
+		 * exist.  If the former, change to LOG_ERR and if the
+		 * latter change to LOG_DEBUG.
+		 */
+		upslogx(LOG_INFO, "%s: unsettable variable %s", __func__, var);
 		return STAT_SET_UNKNOWN;
 	}
 	ups_setsuper (1);
@@ -413,42 +499,60 @@ static int upsdrv_setvar (const char *var, const char * data) {
 	return STAT_SET_HANDLED;
 }
 
+/*
+ * The "power down and maybe return command" is "OFF %d\r", with a
+ * delay in seconds before poweroff.  As a special case, "OFF 0" does
+ * not shut down.  The UPS will power on the load when power returns
+ * (or after a delay if power is not out), according to the front panel
+ * parameter, or the value set via `autorestart()`.
+ */
+
+/*
+ * This is equivalent to the `shutdown.return` instant command, but
+ * invoked with `-k`.
+ * \todo Reduce duplication.
+ */
 void upsdrv_shutdown(void)
 {
 	const	char	*grace;
 
+	upsdebugx(2, "%s: begin", __func__);
+
 	grace = dstate_getinfo("ups.delay.shutdown");
+	if (!grace) {
+		upsdebugx(1, "%s: ups.delay.shutdown is NULL!", __func__);
+		/* Pick a different value than 20 so we can see it in the logs. */
+		grace = "30";
+	}
 
-	if (!grace)
-		grace = "1"; /* apparently, OFF0 does not work */
+	upslogx(LOG_CRIT, "%s: OFF/restart in %s seconds", __func__, grace);
 
-	printf ("shutdown in %s seconds\n", grace);
-	/* make power return when utility power returns */
+	/* Start again, overriding front panel setting. */
 	autorestart (1);
+
 	upssend ("OFF%s\r", grace);
 	/* I'm nearly dead, Jim */
-	/* OFF will powercycle when line power is available again */
+
+	upsdebugx(2, "%s: end", __func__);
 }
 
 static int instcmd (const char *cmdname, const char *extra)
 {
-	const char *p;
-
 	if (!strcasecmp(cmdname, "load.off")) {
-		printf ("powering off\n");
+		upslogx(LOG_CRIT, "%s: %s: OFF/stayoff in 1s",
+			__func__, cmdname);
 		autorestart (0);
 		upssend ("OFF1\r");
 		return STAT_INSTCMD_HANDLED;
 	}
 	else if (!strcasecmp(cmdname, "shutdown.return")) {
-		p = dstate_getinfo ("ups.delay.shutdown");
-		if (!p) p = "1";
-		printf ("shutdown in %s seconds\n", p);
-		autorestart (1);
-		upssend ("OFF%s\r", p);
+		upsdebugx(2, "%s: %s: start", __func__, cmdname);
+		upsdrv_shutdown();
 		return STAT_INSTCMD_HANDLED;
 	}
-	upslogx(LOG_INFO, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	/* \todo Software error or user error? */
+	upslogx(LOG_ERR, "%s: unknown command [%s] [%s]",
+		__func__, cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -474,6 +578,9 @@ static struct {
 	{NULL, B1200},
 };
 
+/*
+ * Called first, for normal operation and for shutdown.
+ */
 void upsdrv_initups(void)
 {
 	speed_t speed = B1200;
@@ -481,6 +588,8 @@ void upsdrv_initups(void)
 	char * speed_val = getval("baudrate");
 	char * max_load = getval("max_load");
 
+	upsdebugx(1, "%s: begin", __func__);
+
 	if (max_load) maxload = atoi(max_load);
 
 	if (speed_val) {
@@ -493,14 +602,25 @@ void upsdrv_initups(void)
 	}
 
 	upsfd = ser_open(device_path);
-	ser_set_speed(upsfd, device_path, speed);
-	/* TODO: probe ups type */
+	if (INVALID_FD(upsfd)) {
+		upslogx(LOG_WARNING, "%s: failed to open %s",
+			__func__, device_path);
+		/* \todo: Deal with the failure */
+	}
 
-	/* the upsh handlers can't be done here, as they get initialized
-	 * shortly after upsdrv_initups returns to main.
-	 */
+	/* ser_set_speed returns int 0 always; fatal if ioctl fails */
+	ser_set_speed(upsfd, device_path, speed);
+
+	upsdebugx(1, "%s: opened %s speed %s upsfd %d",
+		  __func__, device_path, speed_val ? speed_val : "DEFAULT", upsfd);
+
+	/* Set early so that it is in place for shutdown. */
+	dstate_setinfo("ups.delay.shutdown", "%s", shutdown_delay);
+
+	upsdebugx(1, "%s: end", __func__);
 }
 
 void upsdrv_cleanup(void)
 {
+	upsdebugx(1, "%s: begin/end", __func__);
 }
