@@ -7,7 +7,7 @@
  * device support from such legacy drivers over time.
  *
  * A document describing the protocol implemented by this driver can be
- * found online at "http://www.networkupstools.org/protocols/megatec.html".
+ * found online at "https://www.networkupstools.org/protocols/megatec.html".
  *
  * Copyright (C) 2008 - Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
@@ -31,7 +31,7 @@
 #include "blazer.h"
 
 #define DRIVER_NAME	"Megatec/Q1 protocol serial driver"
-#define DRIVER_VERSION	"1.58"
+#define DRIVER_VERSION	"1.60"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -121,6 +121,7 @@ void upsdrv_makevartable(void)
 void upsdrv_initups(void)
 {
 #ifndef TESTING
+#ifndef WIN32 /* TODO : Correctly set the port parameters for WIN32 */
 	const struct {
 		const char	*val;
 		const int	dtr;
@@ -187,7 +188,10 @@ void upsdrv_initups(void)
 	 * Allow some time to settle for the cablepower
 	 */
 	usleep(100000);
-#endif
+#else
+	upsdebugx(0, "blazer_ser: upsdrv_init(): serial port setup for WIN32 currently has not been ported (TODO)");
+#endif /* WIN32 */
+#endif /* TESTING */
 	blazer_initups();
 }
 
