@@ -23,11 +23,13 @@
 #include "nut_stdint.h"
 
 #include <sys/types.h>
+#ifndef WIN32
 #include <sys/socket.h>
 #include <sys/un.h>
+#endif
 
 #define DRIVER_NAME	"clone outlet UPS Driver"
-#define DRIVER_VERSION	"0.02"
+#define DRIVER_VERSION	"0.03"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -65,7 +67,15 @@ static struct {
 static int	dumpdone = 0;
 
 static PCONF_CTX_t	sock_ctx;
-static time_t	last_heard = 0, last_ping = 0, last_connfail = 0;
+static time_t	last_heard = 0, last_ping = 0;
+
+#ifdef WIN32
+static char     	read_buf[SMALLBUF];
+static OVERLAPPED	read_overlapped;
+#else
+/* TODO: Why not built in WIN32? */
+static time_t	last_connfail = 0;
+#endif
 
 static int parse_args(size_t numargs, char **arg)
 {
@@ -141,10 +151,13 @@ static int parse_args(size_t numargs, char **arg)
 }
 
 
-static int sstate_connect(void)
+static TYPE_FD sstate_connect(void)
 {
+	TYPE_FD	fd;
+
+#ifndef WIN32
 	ssize_t	ret;
-	int	fd, len;
+	int	len;
 	const char	*dumpcmd = "DUMPALL\n";
 	struct sockaddr_un	sa;
 
@@ -159,8 +172,8 @@ static int sstate_connect(void)
 	if ((uintmax_t)len >= (uintmax_t)sizeof(sa.sun_path)) {
 		fatalx(EXIT_FAILURE,
 			"Can't create a unix domain socket: pathname '%s/%s' "
-			"is too long (%zu) for 'struct sockaddr_un->sun_path' "
-			"on this system (%zu)",
+			"is too long (%" PRIuSIZE ") for 'struct sockaddr_un->sun_path' "
+			"on this system (%" PRIuSIZE ")",
 			dflt_statepath(), device_path,
 			strlen(dflt_statepath()) + 1 + strlen(device_path),
 			sizeof(sa.sun_path));
@@ -168,9 +181,9 @@ static int sstate_connect(void)
 
 	fd = socket(AF_UNIX, SOCK_STREAM, 0);
 
-	if (fd < 0) {
+	if (INVALID_FD(fd)) {
 		upslog_with_errno(LOG_ERR, "Can't create socket for UPS [%s]", device_path);
-		return -1;
+		return ERROR_FD;
 	}
 
 	ret = connect(fd, (struct sockaddr *) &sa, sizeof(sa));
@@ -184,13 +197,13 @@ static int sstate_connect(void)
 		time(&now);
 
 		if (difftime(now, last_connfail) < 60) {
-			return -1;
+			return ERROR_FD;
 		}
 
 		last_connfail = now;
 
 		upslog_with_errno(LOG_ERR, "Can't connect to UPS [%s]", device_path);
-		return -1;
+		return ERROR_FD;
 	}
 
 	ret = fcntl(fd, F_GETFL, 0);
@@ -198,7 +211,7 @@ static int sstate_connect(void)
 	if (ret < 0) {
 		upslog_with_errno(LOG_ERR, "fcntl get on UPS [%s] failed", device_path);
 		close(fd);
-		return -1;
+		return ERROR_FD;
 	}
 
 	ret = fcntl(fd, F_SETFL, ret | O_NDELAY);
@@ -206,7 +219,7 @@ static int sstate_connect(void)
 	if (ret < 0) {
 		upslog_with_errno(LOG_ERR, "fcntl set O_NDELAY on UPS [%s] failed", device_path);
 		close(fd);
-		return -1;
+		return ERROR_FD;
 	}
 
 	/* get a dump started so we have a fresh set of data */
@@ -215,9 +228,55 @@ static int sstate_connect(void)
 	if (ret != (int)strlen(dumpcmd)) {
 		upslog_with_errno(LOG_ERR, "Initial write to UPS [%s] failed", device_path);
 		close(fd);
-		return -1;
+		return ERROR_FD;
 	}
 
+	/* continued below... */
+#else /* WIN32 */
+	char		pipename[SMALLBUF];
+	const char	*dumpcmd = "DUMPALL\n";
+	BOOL		result = FALSE;
+
+	snprintf(pipename, sizeof(pipename), "\\\\.\\pipe\\%s/%s", dflt_statepath(), device_path);
+
+	result = WaitNamedPipe(pipename,NMPWAIT_USE_DEFAULT_WAIT);
+
+	if (result == FALSE) {
+		return ERROR_FD;
+	}
+
+	fd = CreateFile(
+			pipename,	/* pipe name */
+			GENERIC_READ |	/* read and write access */
+			GENERIC_WRITE,
+			0,		/* no sharing */
+			NULL,		/* default security attributes FIXME */
+			OPEN_EXISTING,	/* opens existing pipe */
+			FILE_FLAG_OVERLAPPED,	/*  enable async IO */
+			NULL);		/* no template file */
+
+	if (fd == INVALID_HANDLE_VALUE) {
+		upslog_with_errno(LOG_ERR, "Can't connect to UPS [%s]", device_path);
+		return ERROR_FD;
+	}
+
+	/* get a dump started so we have a fresh set of data */
+	DWORD bytesWritten = 0;
+
+	result = WriteFile (fd,dumpcmd,strlen(dumpcmd),&bytesWritten,NULL);
+	if (result == 0 || bytesWritten != strlen(dumpcmd)) {
+		upslog_with_errno(LOG_ERR, "Initial write to UPS [%s] failed", device_path);
+		CloseHandle(fd);
+		return ERROR_FD;
+	}
+
+	/* Start a read IO so we could wait on the event associated with it */
+	ReadFile(fd, read_buf,
+		sizeof(read_buf) - 1, /*-1 to be sure to have a trailling 0 */
+		NULL, &(read_overlapped));
+#endif
+
+	/* sstate_connect() continued for both platforms: */
 	pconf_init(&sock_ctx, NULL);
 
 	time(&last_heard);
@@ -234,14 +293,20 @@ static int sstate_connect(void)
 
 static void sstate_disconnect(void)
 {
-	if (upsfd < 0) {
+	if (INVALID_FD(upsfd)) {
+		/* Already disconnected... or not yet? ;) */
 		return;
 	}
 
 	pconf_finish(&sock_ctx);
 
+#ifndef WIN32
 	close(upsfd);
-	upsfd = -1;
+#else
+	CloseHandle(upsfd);
+#endif
+
+	upsfd = ERROR_FD;
 }
 
 
@@ -249,11 +314,25 @@ static int sstate_sendline(const char *buf)
 {
 	ssize_t	ret;
 
-	if (upsfd < 0) {
+	if (INVALID_FD(upsfd)) {
 		return -1;	/* failed */
 	}
 
+#ifndef WIN32
 	ret = write(upsfd, buf, strlen(buf));
+#else
+	DWORD bytesWritten = 0;
+	BOOL  result = FALSE;
+
+	result = WriteFile (upsfd,buf,strlen(buf),&bytesWritten,NULL);
+
+	if( result == 0 ) {
+		ret = 0;
+	}
+	else  {
+		ret = (int)bytesWritten;
+	}
+#endif
 
 	if (ret == (int)strlen(buf)) {
 		return 0;
@@ -268,9 +347,10 @@ static int sstate_readline(void)
 {
 	int	i;
 	ssize_t	ret;
+#ifndef WIN32
 	char	buf[SMALLBUF];
 
-	if (upsfd < 0) {
+	if (INVALID_FD(upsfd)) {
 		return -1;	/* failed */
 	}
 
@@ -279,33 +359,44 @@ static int sstate_readline(void)
 	if (ret < 0) {
 		switch(errno)
 		{
-		case EINTR:
-		case EAGAIN:
-			return 0;
+			case EINTR:
+			case EAGAIN:
+				return 0;
 
-		default:
-			upslog_with_errno(LOG_WARNING, "Read from UPS [%s] failed", device_path);
-			return -1;
+			default:
+				upslog_with_errno(LOG_WARNING, "Read from UPS [%s] failed", device_path);
+				return -1;
 		}
 	}
+#else
+	if (INVALID_FD(upsfd)) {
+		return -1;	/* failed */
+	}
+
+	/* FIXME? I do not see this buf or read_buf filled below */
+	char *buf = read_buf;
+	DWORD bytesRead;
+	GetOverlappedResult(upsfd, &read_overlapped, &bytesRead, FALSE);
+	ret = bytesRead;
+#endif
 
 	for (i = 0; i < ret; i++) {
 
 		switch (pconf_char(&sock_ctx, buf[i]))
 		{
-		case 1:
-			if (parse_args(sock_ctx.numargs, sock_ctx.arglist)) {
-				time(&last_heard);
-			}
-			continue;
+			case 1:
+				if (parse_args(sock_ctx.numargs, sock_ctx.arglist)) {
+					time(&last_heard);
+				}
+				continue;
 
-		case 0:
-			continue;	/* haven't gotten a line yet */
+			case 0:
+				continue;	/* haven't gotten a line yet */
 
-		default:
-			/* parse error */
-			upslogx(LOG_NOTICE, "Parse error on sock: %s", sock_ctx.errmsg);
-			return -1;
+			default:
+				/* parse error */
+				upslogx(LOG_NOTICE, "Parse error on sock: %s", sock_ctx.errmsg);
+				return -1;
 		}
 	}
 
@@ -319,7 +410,7 @@ static int sstate_dead(int maxage)
 	double	elapsed;
 
 	/* an unconnected ups is always dead */
-	if (upsfd < 0) {
+	if (INVALID_FD(upsfd)) {
 		upsdebugx(3, "sstate_dead: connection to driver socket for UPS [%s] lost", device_path);
 		return -1;	/* dead */
 	}
@@ -388,6 +479,11 @@ void upsdrv_updateinfo(void)
 
 void upsdrv_shutdown(void)
 {
+	/* replace with a proper shutdown function */
+/*
+	upslogx(LOG_ERR, "shutdown not supported");
+	set_exit_flag(-1);
+ */
 }
 
 
