@@ -2,6 +2,7 @@
 
    Copyright (C)
        2005 - 2015  Arnaud Quette <http://arnaud.quette.free.fr/contact.html>
+       2014 - 2023  Jim Klimov <jimklimov+nut@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,9 +32,12 @@
 
 #include "config.h" /* must be the first header */
 
+#ifndef WIN32
 #include <netdb.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
+#endif
+
 #include <sys/stat.h>
 #include <string.h>
 
@@ -44,7 +48,7 @@
 #include "dummy-ups.h"
 
 #define DRIVER_NAME	"Device simulation and repeater driver"
-#define DRIVER_VERSION	"0.15"
+#define DRIVER_VERSION	"0.18"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info =
@@ -95,7 +99,7 @@ static struct stat	datafile_stat;
 
 static int setvar(const char *varname, const char *val);
 static int instcmd(const char *cmdname, const char *extra);
-static int parse_data_file(int upsfd);
+static int parse_data_file(TYPE_FD arg_upsfd);
 static dummy_info_t *find_info(const char *varname);
 static int is_valid_data(const char* varname);
 static int is_valid_value(const char* varname, const char *value);
@@ -107,6 +111,9 @@ static char		*client_upsname = NULL, *hostname = NULL;
 static UPSCONN_t	*ups = NULL;
 static uint16_t	port;
 
+/* repeater mode parameters */
+static int repeater_disable_strict_start = 0;
+
 /* Driver functions */
 
 void upsdrv_initinfo(void)
@@ -127,7 +134,7 @@ void upsdrv_initinfo(void)
 
 					/* Set max length for strings, if needed */
 					if (item->info_flags & ST_FLAG_STRING)
-						dstate_setaux(item->info_type, item->info_len);
+						dstate_setaux(item->info_type, (long)item->info_len);
 				}
 			}
 
@@ -152,7 +159,17 @@ void upsdrv_initinfo(void)
 			ups = xmalloc(sizeof(*ups));
 			if (upscli_connect(ups, hostname, port, UPSCLI_CONN_TRYSSL) < 0)
 			{
-				fatalx(EXIT_FAILURE, "Error: %s", upscli_strerror(ups));
+				if(repeater_disable_strict_start == 1)
+				{
+					upslogx(LOG_WARNING, "Warning: %s", upscli_strerror(ups));
+				}
+				else
+				{
+					fatalx(EXIT_FAILURE, "Error: %s. "
+					"Any errors encountered starting the repeater mode result in driver termination, "
+					"perhaps you want to set the 'repeater_disable_strict_start' option?"
+					, upscli_strerror(ups));
+				}
 			}
 			else
 			{
@@ -165,7 +182,18 @@ void upsdrv_initinfo(void)
 				{
 					fatalx(EXIT_FAILURE, "Error: upsd is too old to support this query");
 				}
-				fatalx(EXIT_FAILURE, "Error: %s", upscli_strerror(ups));
+
+				if(repeater_disable_strict_start == 1)
+				{
+					upslogx(LOG_WARNING, "Warning: %s", upscli_strerror(ups));
+				}
+				else
+				{
+					fatalx(EXIT_FAILURE, "Error: %s. "
+					"Any errors encountered starting the repeater mode result in driver termination, "
+					"perhaps you want to set the 'repeater_disable_strict_start' option?"
+					, upscli_strerror(ups));
+				}
 			}
 			/* FIXME: commands and settable variable! */
 			break;
@@ -204,6 +232,36 @@ void upsdrv_initinfo(void)
 	dstate_addcmd("load.off");
 }
 
+static int prepare_filepath(char *fn, size_t buflen)
+{
+	/* Note: device_path is a global variable,
+	 * the "port=..." value parsed in main.c */
+	if (device_path[0] == '/'
+#ifdef WIN32
+	||  device_path[1] == ':'	/* "C:\..." */
+#endif
+	) {
+		/* absolute path */
+		return snprintf(fn, buflen, "%s", device_path);
+	} else if (device_path[0] == '.') {
+		/* "./" or "../" e.g. via CLI, relative to current working
+		 * directory of the driver process... at this moment */
+		if (getcwd(fn, buflen)) {
+			return snprintf(fn + strlen(fn), buflen - strlen(fn), "/%s", device_path);
+		} else {
+			return snprintf(fn, buflen, "%s", device_path);
+		}
+	} else {
+		/* assumed to be a filename in NUT config file path
+		 * (possibly under, with direct use of dirname without dots)
+		 * Note that we do not fiddle with file-path separator,
+		 * modern Windows (at least via MinGW/MSYS2) supports
+		 * the POSIX slash.
+		 */
+		return snprintf(fn, buflen, "%s/%s", confpath(), device_path);
+	}
+}
+
 void upsdrv_updateinfo(void)
 {
 	upsdebugx(1, "upsdrv_updateinfo...");
@@ -224,19 +282,35 @@ void upsdrv_updateinfo(void)
 				struct stat	fs;
 				char fn[SMALLBUF];
 
-				if (device_path[0] == '/')
-					snprintf(fn, sizeof(fn), "%s", device_path);
-				else
-					snprintf(fn, sizeof(fn), "%s/%s", confpath(), device_path);
+				prepare_filepath(fn, sizeof(fn));
 
-				if (0 != fstat (upsfd, &fs) && 0 != stat (fn, &fs)) {
-					upsdebugx(2, "Can't open %s currently", fn);
+				/* Determine if file modification timestamp has changed
+				 * since last use (so we would want to re-read it) */
+#ifndef WIN32
+				/* Either successful stat (zero return) is OK to
+				 * fill the "fs" struct. Note that currently
+				 * "upsfd" is a no-op for files, they are re-opened
+				 * and re-parsed every time so callers can modify
+				 * the data without complications.
+				 */
+				if ( (INVALID_FD(upsfd) || 0 != fstat (upsfd, &fs)) && 0 != stat (fn, &fs))
+#else
+				/* Consider GetFileAttributesEx() for WIN32_FILE_ATTRIBUTE_DATA?
+				 *   https://stackoverflow.com/questions/8991192/check-the-file-size-without-opening-file-in-c/8991228#8991228
+				 */
+				if (0 != stat (fn, &fs))
+#endif
+				{
+					upsdebugx(2, "%s: MODE_DUMMY_ONCE: Can't stat %s currently", __func__, fn);
 					/* retry ASAP until we get a file */
 					memset(&datafile_stat, 0, sizeof(struct stat));
 					next_update = 1;
 				} else {
-					if (datafile_stat.st_mtim.tv_sec != fs.st_mtim.tv_sec) {
-						upsdebugx(2, "upsdrv_updateinfo: input file was already read once to the end, but changed later - re-reading");
+					if (datafile_stat.st_mtime != fs.st_mtime) {
+						upsdebugx(2,
+							"%s: MODE_DUMMY_ONCE: input file was already read once "
+							"to the end, but changed later - re-reading: %s",
+							__func__, fn);
 						/* updated file => retry ASAP */
 						next_update = 1;
 						datafile_stat = fs;
@@ -245,7 +319,7 @@ void upsdrv_updateinfo(void)
 			}
 
 			if (ctx == NULL && next_update == -1) {
-				upsdebugx(2, "upsdrv_updateinfo: NO-OP: input file was already read once to the end");
+				upsdebugx(2, "%s: MODE_DUMMY_ONCE: NO-OP: input file was already read once to the end", __func__);
 				dstate_dataok();
 			} else {
 				/* initial parsing interrupted by e.g. TIMER line */
@@ -306,12 +380,11 @@ void upsdrv_updateinfo(void)
 	}
 }
 
-void upsdrv_shutdown(void)
-	__attribute__((noreturn));
-
 void upsdrv_shutdown(void)
 {
-	fatalx(EXIT_FAILURE, "shutdown not supported");
+	/* replace with a proper shutdown function */
+	upslogx(LOG_ERR, "shutdown not supported");
+	set_exit_flag(-1);
 }
 
 static int instcmd(const char *cmdname, const char *extra)
@@ -338,6 +411,7 @@ void upsdrv_help(void)
 void upsdrv_makevartable(void)
 {
 	addvar(VAR_VALUE,	"mode",	"Specify mode instead of guessing it from port value (dummy = dummy-loop, dummy-once, repeater)"); /* meta */
+	addvar(VAR_FLAG,    "repeater_disable_strict_start", "Do not terminate the driver encountering errors when starting the repeater mode");
 }
 
 void upsdrv_initups(void)
@@ -449,15 +523,33 @@ void upsdrv_initups(void)
 #endif
 		}
 
-		if (device_path[0] == '/')
-			snprintf(fn, sizeof(fn), "%s", device_path);
-		else
-			snprintf(fn, sizeof(fn), "%s/%s", confpath(), device_path);
+		prepare_filepath(fn, sizeof(fn));
 
-		if (0 != fstat (upsfd, &datafile_stat) && 0 != stat (device_path, &datafile_stat)) {
-			upsdebugx(2, "Can't open %s currently", device_path);
+		/* Update file modification timestamp (and other data) */
+#ifndef WIN32
+		/* Either successful stat (zero return) is OK to fill the
+		 * "datafile_stat" struct. Note that currently "upsfd" is
+		 * a no-op for files, they are re-opened and re-parsed
+		 * every time so callers can modify the data without
+		 * complications.
+		 */
+		if ( (INVALID_FD(upsfd) || 0 != fstat (upsfd, &datafile_stat)) && 0 != stat (fn, &datafile_stat))
+#else
+		/* Consider GetFileAttributesEx() for WIN32_FILE_ATTRIBUTE_DATA?
+		 *   https://stackoverflow.com/questions/8991192/check-the-file-size-without-opening-file-in-c/8991228#8991228
+		 */
+		if (0 != stat (fn, &datafile_stat))
+#endif
+		{
+			upsdebugx(2, "%s: Can't stat %s (%s) currently", __func__, device_path, fn);
+		} else {
+			upsdebugx(2, "Located %s for device simulation data: %s", device_path, fn);
 		}
 	}
+	if (testvar("repeater_disable_strict_start"))
+	{
+		repeater_disable_strict_start = 1;
+	}
 }
 
 void upsdrv_cleanup(void)
@@ -531,7 +623,7 @@ static int setvar(const char *varname, const char *val)
 
 			/* Set max length for strings, if needed */
 			if (item->info_flags & ST_FLAG_STRING)
-				dstate_setaux(item->info_type, item->info_len);
+				dstate_setaux(item->info_type, (long)item->info_len);
 		}
 		else
 		{
@@ -571,7 +663,7 @@ static int upsclient_update_vars(void)
 		/* VAR <upsname> <varname> <val> */
 		if (numa < 4)
 		{
-			upsdebugx(1, "Error: insufficient data (got %zu args, need at least 4)", numa);
+			upsdebugx(1, "Error: insufficient data (got %" PRIuSIZE " args, need at least 4)", numa);
 		}
 
 		upsdebugx(5, "Received: %s %s %s %s",
@@ -595,7 +687,7 @@ static dummy_info_t *find_info(const char *varname)
 			return item;
 	}
 
-	upsdebugx(2, "find_info: unknown variable: %s\n", varname);
+	upsdebugx(2, "find_info: unknown variable: %s", varname);
 
 	return NULL;
 }
@@ -649,7 +741,7 @@ static void upsconf_err(const char *errmsg)
 /* for dummy mode
  * parse the definition file and process its content
  */
-static int parse_data_file(int arg_upsfd)
+static int parse_data_file(TYPE_FD arg_upsfd)
 {
 	char	fn[SMALLBUF];
 	char	*ptr, var_value[MAX_STRING_SIZE];
@@ -672,11 +764,7 @@ static int parse_data_file(int arg_upsfd)
 	{
 		ctx = (PCONF_CTX_t *)xmalloc(sizeof(PCONF_CTX_t));
 
-		if (device_path[0] == '/')
-			snprintf(fn, sizeof(fn), "%s", device_path);
-		else
-			snprintf(fn, sizeof(fn), "%s/%s", confpath(), device_path);
-
+		prepare_filepath(fn, sizeof(fn));
 		pconf_init(ctx, upsconf_err);
 
 		if (!pconf_file_begin(ctx, fn))
