@@ -19,7 +19,11 @@
 
 #include "config.h" /* must be first */
 
+#ifndef WIN32
 #include <sys/ioctl.h>
+#else
+#include "wincompat.h"
+#endif
 
 #include "main.h"
 #include "serial.h"
@@ -27,7 +31,7 @@
 #include "nut_stdint.h"
 
 #define DRIVER_NAME	"Generic contact-closure UPS driver"
-#define DRIVER_VERSION	"1.38"
+#define DRIVER_VERSION	"1.39"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -218,7 +222,11 @@ void upsdrv_updateinfo(void)
 {
 	int	flags, ol, bl, rb, bypass, ret;
 
+#ifndef WIN32
 	ret = ioctl(upsfd, TIOCMGET, &flags);
+#else
+	ret = w32_getcomm( upsfd, &flags );
+#endif
 
 	if (ret != 0) {
 		upslog_with_errno(LOG_INFO, "ioctl failed");
@@ -305,34 +313,49 @@ void upsdrv_shutdown(void)
 	int	flags, ret;
 
 	if (upstype == -1) {
-		fatalx(EXIT_FAILURE, "No upstype set - see help text / man page!");
+		upslogx(LOG_ERR, "No upstype set - see help text / man page!");
+		set_exit_flag(-1);
+	        return;
 	}
 
 	flags = upstab[upstype].line_sd;
 
 	if (flags == -1) {
-		fatalx(EXIT_FAILURE, "No shutdown command defined for this model!");
+		upslogx(LOG_ERR, "No shutdown command defined for this model!");
+		set_exit_flag(-1);
+	        return;
 	}
 
 	if (flags == TIOCM_ST) {
 
+#ifndef WIN32
 #ifndef HAVE_TCSENDBREAK
-		fatalx(EXIT_FAILURE, "Need to send a BREAK, but don't have tcsendbreak!");
+		upslogx(LOG_ERR, "Need to send a BREAK, but don't have tcsendbreak!");
+		set_exit_flag(-1);
+	        return;
+#endif
 #endif
 
 		ret = tcsendbreak(upsfd, 4901);
 
 		if (ret != 0) {
-			fatal_with_errno(EXIT_FAILURE, "tcsendbreak");
+			upslog_with_errno(LOG_ERR, "tcsendbreak");
+			set_exit_flag(-1);
 		}
 
 		return;
 	}
 
+#ifndef WIN32
 	ret = ioctl(upsfd, TIOCMSET, &flags);
+#else
+	ret = w32_setcomm(upsfd,&flags);
+#endif
 
 	if (ret != 0) {
-		fatal_with_errno(EXIT_FAILURE, "ioctl TIOCMSET");
+		upslog_with_errno(LOG_ERR, "ioctl TIOCMSET");
+		set_exit_flag(-1);
+	        return;
 	}
 
 	if (getval("sdtime")) {
@@ -426,7 +449,11 @@ void upsdrv_initups(void)
 		upsdebugx(2, "parse_output_signals: SD overridden with %s\n", v);
 	}
 
+#ifndef WIN32
 	if (ioctl(upsfd, TIOCMSET, &upstab[upstype].line_norm)) {
+#else
+	if (w32_setcomm(upsfd,&upstab[upstype].line_norm)) {
+#endif
 		fatal_with_errno(EXIT_FAILURE, "ioctl TIOCMSET");
 	}
 }
@@ -435,4 +462,3 @@ void upsdrv_cleanup(void)
 {
 	ser_close(upsfd, device_path);
 }
-
