@@ -10,7 +10,7 @@
  *
  *      The logic of this file is ripped from mge-shut driver (also from
  *      Arnaud Quette), which is a "HID over serial link" UPS driver for
- *      Network UPS Tools <http://www.networkupstools.org/>
+ *      Network UPS Tools <https://www.networkupstools.org/>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -32,9 +32,12 @@
 #include "common.h" /* for xmalloc, upsdebugx prototypes */
 #include "usb-common.h"
 #include "nut_libusb.h"
+#ifdef WIN32
+#include "wincompat.h"
+#endif
 
 #define USB_DRIVER_NAME		"USB communication driver (libusb 0.1)"
-#define USB_DRIVER_VERSION	"0.43"
+#define USB_DRIVER_VERSION	"0.45"
 
 /* driver description structure */
 upsdrv_info_t comm_upsdrv_info = {
@@ -48,6 +51,14 @@ upsdrv_info_t comm_upsdrv_info = {
 #define MAX_REPORT_SIZE         0x1800
 #define MAX_RETRY               3
 
+#if (!HAVE_STRCASESTR) && (HAVE_STRSTR && HAVE_STRLWR && HAVE_STRDUP)
+/* Only used in this file of all NUT codebase, so not in str.{c,h}
+ * where it happens to conflict with netsnmp-provided variant for
+ * some of our build products.
+ */
+static char *strcasestr(const char *haystack, const char *needle);
+#endif
+
 static void libusb_close(usb_dev_handle *udev);
 
 /*! Add USB-related driver variables with addvar() and dstate_setinfo().
@@ -65,6 +76,22 @@ void nut_usb_addvars(void)
 
 	addvar(VAR_VALUE, "bus", "Regular expression to match USB bus name");
 	addvar(VAR_VALUE, "device", "Regular expression to match USB device name");
+	/* Not supported by libusb0, but let's not crash config
+	 * parsing on unknown keywords due to such nuances! :) */
+	addvar(VAR_VALUE, "busport", "Regular expression to match USB bus port name"
+		" (tolerated but ignored in this build)"
+	);
+
+	/* Warning: this feature is inherently non-deterministic!
+	 * If you only care to know that at least one of your no-name UPSes is online,
+	 * this option can help. If you must really know which one, it will not!
+	 */
+	addvar(VAR_FLAG, "allow_duplicates",
+		"If you have several UPS devices which may not be uniquely "
+		"identified by options above, allow each driver instance with this "
+		"option to take the first match if available, or try another "
+		"(association of driver to device may vary between runs)");
+
 	addvar(VAR_VALUE, "usb_set_altinterface", "Force redundant call to usb_set_altinterface() (value=bAlternateSetting; default=0)");
 
 	dstate_setinfo("driver.version.usb", "libusb-0.1 (or compat)");
@@ -188,11 +215,23 @@ static int libusb_open(usb_dev_handle **udevp,
 	usb_ctrl_char	rdbuf[MAX_REPORT_SIZE];
 	usb_ctrl_charbufsize		rdlen;
 
+	struct usb_bus *busses;
+
 	/* libusb base init */
 	usb_init();
 	usb_find_busses();
 	usb_find_devices();
 
+#ifdef WIN32
+	busses = usb_get_busses();
+#else
+	/* libusb built-in; not sure why original NUT for WIN32
+	 * code differed or if it is actually better? Or why
+	 * this was not tackled in a few other files for USB?..
+	 */
+	busses = usb_busses;
+#endif
+
 #ifndef __linux__ /* SUN_LIBUSB (confirmed to work on Solaris and FreeBSD) */
 	/* Causes a double free corruption in linux if device is detached! */
 	libusb_close(*udevp);
@@ -200,7 +239,7 @@ static int libusb_open(usb_dev_handle **udevp,
 
 	upsdebugx(3, "usb_busses=%p", (void*)usb_busses);
 
-	for (bus = usb_busses; bus; bus = bus->next) {
+	for (bus = busses; bus; bus = bus->next) {
 		for (dev = bus->devices; dev; dev = dev->next) {
 			/* int	if_claimed = 0; */
 
@@ -254,14 +293,29 @@ static int libusb_open(usb_dev_handle **udevp,
 			free(curDevice->Serial);
 			free(curDevice->Bus);
 			free(curDevice->Device);
+#if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+			free(curDevice->BusPort);
+#endif
 			memset(curDevice, '\0', sizeof(*curDevice));
 
+			/* Keep the list of items in sync with those matched by
+			 * drivers/libusb1.c and tools/nut-scanner/scan_usb.c:
+			 */
 			curDevice->VendorID = dev->descriptor.idVendor;
 			curDevice->ProductID = dev->descriptor.idProduct;
 			curDevice->Bus = xstrdup(bus->dirname);
 			curDevice->Device = xstrdup(dev->filename);
 			curDevice->bcdDevice = dev->descriptor.bcdDevice;
 
+#if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+			curDevice->BusPort = (char *)malloc(4);
+			if (curDevice->BusPort == NULL) {
+				fatal_with_errno(EXIT_FAILURE, "Out of memory");
+			}
+			upsdebugx(2, "%s: NOTE: BusPort is always zero with libusb0", __func__);
+			sprintf(curDevice->BusPort, "%03d", 0);
+#endif
+
 			if (dev->descriptor.iManufacturer) {
 				retries = MAX_RETRY;
 				while (retries > 0) {
@@ -311,6 +365,9 @@ static int libusb_open(usb_dev_handle **udevp,
 			upsdebugx(2, "- Serial Number: %s", curDevice->Serial ? curDevice->Serial : "unknown");
 			upsdebugx(2, "- Bus: %s", curDevice->Bus ? curDevice->Bus : "unknown");
 			upsdebugx(2, "- Device: %s", curDevice->Device ? curDevice->Device : "unknown");
+#if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+			upsdebugx(2, "- Bus Port: %s", curDevice->BusPort ? curDevice->BusPort : "unknown");
+#endif
 			upsdebugx(2, "- Device release number: %04x", curDevice->bcdDevice);
 
 			/* FIXME: extend to Eaton OEMs (HP, IBM, ...) */
@@ -341,6 +398,9 @@ static int libusb_open(usb_dev_handle **udevp,
 					goto next_device;
 				}
 			}
+
+			/* If we got here, none of the matchers said
+			 * that the device is not what we want. */
 			upsdebugx(2, "Device matches");
 
 			/* Now we have matched the device we wanted. Claim it. */
@@ -350,11 +410,19 @@ static int libusb_open(usb_dev_handle **udevp,
 			 * it force device claiming by unbinding
 			 * attached driver... From libhid */
 			retries = MAX_RETRY;
-			while (usb_claim_interface(udev, usb_subdriver.hid_rep_index) < 0) {
+#ifdef WIN32
+			usb_set_configuration(udev, 1);
+#endif
 
+			while ((ret = usb_claim_interface(udev, usb_subdriver.hid_rep_index)) < 0) {
 				upsdebugx(2, "failed to claim USB device: %s",
 					usb_strerror());
 
+				if (ret == LIBUSB_ERROR_BUSY && testvar("allow_duplicates")) {
+					upsdebugx(2, "Configured to allow_duplicates so looking for another similar device");
+					goto next_device;
+				}
+
 				if (usb_detach_kernel_driver_np(udev, usb_subdriver.hid_rep_index) < 0) {
 					upsdebugx(2, "failed to detach kernel driver from USB device: %s",
 						usb_strerror());
@@ -374,7 +442,12 @@ static int libusb_open(usb_dev_handle **udevp,
 					usb_strerror());
 			}
 #else
-			if (usb_claim_interface(udev, usb_subdriver.hid_rep_index) < 0) {
+			if ((ret = usb_claim_interface(udev, usb_subdriver.hid_rep_index)) < 0) {
+				if (ret == LIBUSB_ERROR_BUSY && testvar("allow_duplicates")) {
+					upsdebugx(2, "Configured to allow_duplicates so looking for another similar device");
+					goto next_device;
+				}
+
 				fatalx(EXIT_FAILURE,
 					"Can't claim USB device [%04x:%04x]@%d/%d: %s",
 					curDevice->VendorID, curDevice->ProductID,
@@ -414,9 +487,9 @@ static int libusb_open(usb_dev_handle **udevp,
 				upsdebugx(2, "Unable to get HID descriptor (%s)",
 					usb_strerror());
 			} else if (res < 9) {
-				upsdebugx(2, "HID descriptor too short (expected %d, got %d)", 8, res);
+				upsdebugx(2, "HID descriptor too short (expected %d, got %d)", 9, res);
 			} else {
-
+				upsdebugx(2, "Retrieved HID descriptor (expected %d, got %d)", 9, res);
 				upsdebug_hex(3, "HID descriptor, method 1", buf, 9);
 
 				rdlen1 = ((uint8_t)buf[7]) | (((uint8_t)buf[8]) << 8);
@@ -491,7 +564,7 @@ static int libusb_open(usb_dev_handle **udevp,
 			if ((uintmax_t)rdlen > sizeof(rdbuf)) {
 				upsdebugx(2,
 					"HID descriptor too long %" PRI_NUT_USB_CTRL_CHARBUFSIZE
-					" (max %zu)",
+					" (max %" PRIuSIZE ")",
 					rdlen, sizeof(rdbuf));
 				goto next_device;
 			}
@@ -514,9 +587,23 @@ static int libusb_open(usb_dev_handle **udevp,
 
 			if (res < rdlen)
 			{
+#ifndef WIN32
 				upsdebugx(2, "Warning: report descriptor too short "
 					"(expected %" PRI_NUT_USB_CTRL_CHARBUFSIZE
 					", got %d)", rdlen, res);
+#else
+				/* https://github.com/networkupstools/nut/issues/1690#issuecomment-1455206002 */
+				upsdebugx(0, "Warning: report descriptor too short "
+					"(expected %" PRI_NUT_USB_CTRL_CHARBUFSIZE
+					", got %d)", rdlen, res);
+				upsdebugx(0, "Please check your Windows Device Manager: "
+					"perhaps the UPS was recognized by default OS\n"
+					"driver such as HID UPS Battery (hidbatt.sys, "
+					"hidusb.sys or similar). It could have been\n"
+					"\"restored\" by Windows Update. You can try "
+					"https://zadig.akeo.ie/ to handle it with\n"
+					"either WinUSB, libusb0.sys or libusbK.sys.");
+#endif	/* WIN32 */
 				rdlen = res; /* correct rdlen if necessary */
 			}
 
@@ -593,12 +680,16 @@ static int libusb_strerror(const int ret, const char *desc)
 		upsdebugx(2, "%s: Connection timed out", desc);
 		return 0;
 
+/* libusb-win32 does not know EPROTO and EOVERFLOW,
+ * it only returns EIO for any IO errors */
+#ifndef WIN32
 	case -EOVERFLOW:	/* Value too large for defined data type */
-#ifdef EPROTO
+# ifdef EPROTO
 	case -EPROTO:	/* Protocol error */
-#endif
+# endif
 		upsdebugx(2, "%s: %s", desc, usb_strerror());
 		return 0;
+#endif	/* WIN32 */
 
 	default:	/* Undetermined, log only */
 		upslogx(LOG_DEBUG, "%s: %s", desc, usb_strerror());
@@ -635,6 +726,10 @@ static int libusb_get_report(
 		usb_subdriver.hid_rep_index,
 		raw_buf, ReportSize, USB_TIMEOUT);
 
+#ifdef WIN32
+	errno = -ret;
+#endif
+
 	/* Ignore "protocol stall" (for unsupported request) on control endpoint */
 	if (ret == -EPIPE) {
 		return 0;
@@ -666,6 +761,10 @@ static int libusb_set_report(
 		usb_subdriver.hid_rep_index,
 		raw_buf, ReportSize, USB_TIMEOUT);
 
+#ifdef WIN32
+	errno = -ret;
+#endif
+
 	/* Ignore "protocol stall" (for unsupported request) on control endpoint */
 	if (ret == -EPIPE) {
 		return 0;
@@ -714,6 +813,10 @@ static int libusb_get_string(
 
 	ret = usb_get_string_simple(udev, StringIdx, buf, (size_t)buflen);
 
+#ifdef WIN32
+	errno = -ret;
+#endif
+
 	return libusb_strerror(ret, __func__);
 }
 
@@ -736,6 +839,10 @@ static int libusb_get_interrupt(
 	/* Interrupt EP is USB_ENDPOINT_IN with offset defined in hid_ep_in, which is 0 by default, unless overridden in subdriver. */
 	ret = usb_interrupt_read(udev, USB_ENDPOINT_IN + usb_subdriver.hid_ep_in, (char *)buf, bufsize, timeout);
 
+#ifdef WIN32
+	errno = -ret;
+#endif
+
 	/* Clear stall condition */
 	if (ret == -EPIPE) {
 		ret = usb_clear_halt(udev, 0x81);
@@ -757,6 +864,37 @@ static void libusb_close(usb_dev_handle *udev)
 	usb_close(udev);
 }
 
+#if (!HAVE_STRCASESTR) && (HAVE_STRSTR && HAVE_STRLWR && HAVE_STRDUP)
+static char *strcasestr(const char *haystack, const char *needle) {
+	/* work around "const char *" and guarantee the original is not
+	 * touched... not efficient but we have few uses for this method */
+	char * dH = NULL, *dN = NULL, *lH = NULL, *lN = NULL, *first = NULL;
+
+	dH = strdup(haystack);
+	if (dH == NULL) goto err;
+	dN = strdup(needle);
+	if (dN == NULL) goto err;
+	lH = strlwr(dH);
+	if (lH == NULL) goto err;
+	lN = strlwr(dN);
+	if (lN == NULL) goto err;
+	first = strstr(lH, lN);
+
+err:
+	if (dH != NULL) free(dH);
+	if (dN != NULL) free(dN);
+	/* Does this implementation of strlwr() change original buffer? */
+	if (lH != dH && lH != NULL) free(lH);
+	if (lN != dN && lN != NULL) free(lN);
+	if (first == NULL) {
+		return NULL;
+	}
+
+	/* Pointer to first char of the needle found in original haystack */
+	return (char *)(haystack + (first - lH));
+}
+#endif
+
 usb_communication_subdriver_t usb_subdriver = {
 	USB_DRIVER_NAME,
 	USB_DRIVER_VERSION,
