@@ -35,6 +35,10 @@
 #include "mge-xml.h"
 #include "main.h" /* for testvar() */
 
+#ifdef WIN32
+#include "wincompat.h"
+#endif
+
 #define MGE_XML_VERSION		"MGEXML/0.36"
 
 #define MGE_XML_INITUPS		"/"
@@ -544,7 +548,7 @@ static const char *mge_sensitivity_info(const char *arg_val)
 
 static const char *mge_test_result_info(const char *arg_val)
 {
-	STATUS_CLR(CAL);
+	STATUS_CLR(CALIB);
 	switch (atoi(arg_val))
 	{
 	case 1:
@@ -556,7 +560,7 @@ static const char *mge_test_result_info(const char *arg_val)
 	case 4:
 		return "aborted";
 	case 5:
-		STATUS_SET(CAL);
+		STATUS_SET(CALIB);
 		return "in progress";
 	case 6:
 		return "no test initiated";
@@ -1533,10 +1537,10 @@ subdriver_t mge_xml_subdriver = {
 };
 
 const char *vname_nut2mge_xml(const char *name) {
-	assert(NULL != name);
-
 	size_t i = 0;
 
+	assert(NULL != name);
+
 	for (; i < sizeof(mge_xml2nut) / sizeof(xml_info_t); ++i) {
 		xml_info_t *info = mge_xml2nut + i;
 
@@ -1549,10 +1553,10 @@ const char *vname_nut2mge_xml(const char *name) {
 }
 
 const char *vname_mge_xml2nut(const char *name) {
-	assert(NULL != name);
-
 	size_t i = 0;
 
+	assert(NULL != name);
+
 	for (; i < sizeof(mge_xml2nut) / sizeof(xml_info_t); ++i) {
 		xml_info_t *info = mge_xml2nut + i;
 
@@ -1565,10 +1569,10 @@ const char *vname_mge_xml2nut(const char *name) {
 }
 
 char *vvalue_mge_xml2nut(const char *name, const char *value, size_t len) {
-	assert(NULL != name);
-
 	size_t i = 0;
 
+	assert(NULL != name);
+
 	for (; i < sizeof(mge_xml2nut) / sizeof(xml_info_t); ++i) {
 		xml_info_t *info = mge_xml2nut + i;
 
