@@ -207,7 +207,7 @@ static int	voltronic_qs_hex_preprocess_qs_answer(item_t *item, const int len)
 	snprintf(refined, sizeof(refined), "%s", "#");
 
 	/* e.g.: item->answer = "#\x6C\x01 \x35 \x6C\x01 \x35 \x03 \x51\x9A \x28\x02\x12\xD0 \xE6 \x1E \x09\r" */
-	upsdebug_hex(4, "read", item->answer, len);
+	upsdebug_hex(4, "read", item->answer, (size_t)len);
 
 	for (i = 1, token = 1; i < len; i++) {
 
@@ -219,7 +219,7 @@ static int	voltronic_qs_hex_preprocess_qs_answer(item_t *item, const int len)
 		}
 
 		/* 'Unescape' raw data */
-		if (item->answer[i] == 0x28 && i < len) {
+		if (i < len && item->answer[i] == 0x28) {
 
 			switch (item->answer[i + 1])
 			{
@@ -323,7 +323,19 @@ static int	voltronic_qs_hex_protocol(item_t *item, char *value, const size_t val
 		return -1;
 	}
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	snprintf(value, valuelen, item->dfl, item->value);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 
 	/* Unskip items supported only by devices that implement 'T' protocol */
 
@@ -344,7 +356,7 @@ static int	voltronic_qs_hex_protocol(item_t *item, char *value, const size_t val
 /* Input/Output voltage */
 static int	voltronic_qs_hex_input_output_voltage(item_t *item, char *value, const size_t valuelen)
 {
-	int	val;
+	long	val;
 	double	ret;
 	char	*str_end;
 
@@ -356,7 +368,19 @@ static int	voltronic_qs_hex_input_output_voltage(item_t *item, char *value, cons
 	val = strtol(item->value, &str_end, 16) * strtol(str_end, NULL, 16) / 51;
 	ret = val / 256.0;
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	snprintf(value, valuelen, item->dfl, ret);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 
 	return 0;
 }
@@ -369,7 +393,19 @@ static int	voltronic_qs_hex_load(item_t *item, char *value, const size_t valuele
 		return -1;
 	}
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	snprintf(value, valuelen, item->dfl, strtol(item->value, NULL, 16));
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 
 	return 0;
 }
@@ -391,7 +427,19 @@ static int	voltronic_qs_hex_frequency(item_t *item, char *value, const size_t va
 	ret = val2 / val1;
 	ret = ret > 99.9 ? 99.9 : ret;
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	snprintf(value, valuelen, item->dfl, ret);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 
 	return 0;
 }
@@ -399,7 +447,7 @@ static int	voltronic_qs_hex_frequency(item_t *item, char *value, const size_t va
 /* Battery voltage */
 static int	voltronic_qs_hex_battery_voltage(item_t *item, char *value, const size_t valuelen)
 {
-	int	val1, val2;
+	long	val1, val2;
 	char	*str_end;
 
 	if (strspn(item->value, "0123456789ABCDEFabcdef ") != strlen(item->value)) {
@@ -410,7 +458,19 @@ static int	voltronic_qs_hex_battery_voltage(item_t *item, char *value, const siz
 	val1 = strtol(item->value, &str_end, 16);
 	val2 = strtol(str_end, NULL, 16);
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	snprintf(value, valuelen, item->dfl, (val1 * val2) / 510.0);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 
 	return 0;
 }
@@ -418,7 +478,7 @@ static int	voltronic_qs_hex_battery_voltage(item_t *item, char *value, const siz
 /* Ratings bits */
 static int	voltronic_qs_hex_process_ratings_bits(item_t *item, char *value, const size_t valuelen)
 {
-	int	val;
+	long	val;
 	double	ret;
 
 	if (strspn(item->value, "01") != strlen(item->value)) {
@@ -478,7 +538,19 @@ static int	voltronic_qs_hex_process_ratings_bits(item_t *item, char *value, cons
 		return -1;
 	}
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	snprintf(value, valuelen, item->dfl, ret);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 
 	return 0;
 }
