@@ -27,7 +27,9 @@
 /* C-libary includes */
 #include <sys/file.h>
 #include <sys/stat.h>
+#ifndef WIN32
 #include <sys/ioctl.h>
+#endif
 #include <sys/types.h>
 
 /* nut includes */
