@@ -30,6 +30,7 @@
 
 #include "main.h"
 #include "serial.h"
+#include "nut_stdint.h"
 
 #include "powerp-txt.h"
 
@@ -576,7 +577,7 @@ static ssize_t powpan_initups(void)
 		}
 
 		if (ret < 46) {
-			upsdebugx(2, "Expected 46 bytes, but only got %zd", ret);
+			upsdebugx(2, "Expected 46 bytes, but only got %" PRIiSIZE, ret);
 			continue;
 		}
 
