@@ -23,12 +23,14 @@
 #include "main.h"
 #include "attribute.h"
 
+#ifndef WIN32
 #include <grp.h>
 #include <pwd.h>
+#include <sys/ioctl.h>
+#endif
 #include <ctype.h>
 #include <sys/file.h>
 #include <sys/types.h>
-#include <sys/ioctl.h>
 #include <unistd.h>
 
 #ifdef HAVE_UU_LOCK
@@ -45,8 +47,10 @@ static void ser_open_error(const char *port)
 static void ser_open_error(const char *port)
 {
 	struct	stat	fs;
+#ifndef WIN32
 	struct	passwd	*user;
 	struct	group	*group;
+#endif
 
 	printf("\n");
 
@@ -60,6 +64,8 @@ static void ser_open_error(const char *port)
 		fatalx(EXIT_FAILURE, "Fatal error: unusable configuration");
 	}
 
+/* TODO */
+#ifndef WIN32
 	user = getpwuid(getuid());
 
 	if (user)
@@ -77,6 +83,7 @@ static void ser_open_error(const char *port)
 	if (group)
 		printf("Serial port group: %s (%d)\n",
 			group->gr_name, (int) fs.st_gid);
+#endif
 
 	printf("     Mode of port: %04o\n\n", (int) fs.st_mode & 07777);
 
@@ -89,12 +96,17 @@ static void ser_open_error(const char *port)
 	fatalx(EXIT_FAILURE, "Fatal error: unusable configuration");
 }
 
-static void lock_set(int fd, const char *port)
+static void lock_set(TYPE_FD_SER fd, const char *port)
 {
 	int	ret;
 
-	if (fd < 0)
+	if (INVALID_FD_SER(fd)) {
+#ifndef WIN32
 		fatal_with_errno(EXIT_FAILURE, "lock_set: programming error: fd = %d", fd);
+#else
+		fatal_with_errno(EXIT_FAILURE, "lock_set: programming error: struct = %p", fd);
+#endif
+	}
 
 	if (do_lock_port == 0)
 		return;
@@ -124,20 +136,23 @@ static void lock_set(int fd, const char *port)
 
 #else
 
+	NUT_UNUSED_VARIABLE(port);
+	ret = 0; /* Make compiler happy */
+	ret = ret;
 	upslog_with_errno(LOG_WARNING, "Warning: no locking method is available");
 
 #endif
 }
 
 /* Non fatal version of ser_open */
-int ser_open_nf(const char *port)
+TYPE_FD_SER ser_open_nf(const char *port)
 {
-	int	fd;
+	TYPE_FD_SER	fd;
 
 	fd = open(port, O_RDWR | O_NOCTTY | O_EXCL | O_NONBLOCK);
 
-	if (fd < 0) {
-		return -1;
+	if (INVALID_FD_SER(fd)) {
+		return ERROR_FD_SER;
 	}
 
 	lock_set(fd, port);
@@ -145,19 +160,19 @@ int ser_open_nf(const char *port)
 	return fd;
 }
 
-int ser_open(const char *port)
+TYPE_FD_SER ser_open(const char *port)
 {
-	int res;
+	TYPE_FD_SER res;
 
 	res = ser_open_nf(port);
-	if(res == -1) {
+	if (INVALID_FD_SER(res)) {
 		ser_open_error(port);
 	}
 
 	return res;
 }
 
-int ser_set_speed_nf(int fd, const char *port, speed_t speed)
+int ser_set_speed_nf(TYPE_FD_SER fd, const char *port, speed_t speed)
 {
 	struct	termios	tio;
 	NUT_UNUSED_VARIABLE(port);
@@ -186,7 +201,7 @@ int ser_set_speed_nf(int fd, const char *port, speed_t speed)
 	return 0;
 }
 
-int ser_set_speed(int fd, const char *port, speed_t speed)
+int ser_set_speed(TYPE_FD_SER fd, const char *port, speed_t speed)
 {
 	int res;
 
@@ -198,7 +213,8 @@ int ser_set_speed(int fd, const char *port, speed_t speed)
 	return 0;
 }
 
-static int ser_set_control(int fd, int line, int state)
+#ifndef WIN32
+static int ser_set_control(TYPE_FD_SER fd, int line, int state)
 {
 	if (state) {
 		return ioctl(fd, TIOCMBIS, &line);
@@ -206,18 +222,54 @@ static int ser_set_control(int fd, int line, int state)
 		return ioctl(fd, TIOCMBIC, &line);
 	}
 }
+#endif
 
-int ser_set_dtr(int fd, int state)
+int ser_set_dtr(TYPE_FD_SER fd, int state)
 {
+#ifndef WIN32
 	return ser_set_control(fd, TIOCM_DTR, state);
+#else
+	DWORD action;
+
+	if (state == 0) {
+		action = CLRDTR;
+	}
+	else {
+		action = SETDTR;
+	}
+
+	/* Success */
+	if (EscapeCommFunction(fd->handle,action) != 0) {
+		return 0;
+	}
+
+	return -1;
+#endif
 }
 
-int ser_set_rts(int fd, int state)
+int ser_set_rts(TYPE_FD_SER fd, int state)
 {
+#ifndef WIN32
 	return ser_set_control(fd, TIOCM_RTS, state);
+#else
+	DWORD action;
+
+	if(state == 0) {
+		action = CLRRTS;
+	}
+	else {
+		action = SETRTS;
+	}
+	/* Success */
+	if( EscapeCommFunction(fd->handle,action) != 0) {
+		return 0;
+	}
+	return -1;
+#endif
 }
 
-static int ser_get_control(int fd, int line)
+#ifndef WIN32
+static int ser_get_control(TYPE_FD_SER fd, int line)
 {
 	int	flags;
 
@@ -225,31 +277,53 @@ static int ser_get_control(int fd, int line)
 
 	return (flags & line);
 }
+#endif
 
-int ser_get_dsr(int fd)
+int ser_get_dsr(TYPE_FD_SER fd)
 {
+#ifndef WIN32
 	return ser_get_control(fd, TIOCM_DSR);
+#else
+	int flags;
+
+	w32_getcomm(fd->handle, &flags);
+	return (flags & TIOCM_DSR);
+#endif
 }
 
-int ser_get_cts(int fd)
+int ser_get_cts(TYPE_FD_SER fd)
 {
+#ifndef WIN32
 	return ser_get_control(fd, TIOCM_CTS);
+#else
+	int flags;
+
+	w32_getcomm(fd->handle, &flags);
+	return (flags & TIOCM_CTS);
+#endif
 }
 
-int ser_get_dcd(int fd)
+int ser_get_dcd(TYPE_FD_SER fd)
 {
+#ifndef WIN32
 	return ser_get_control(fd, TIOCM_CD);
+#else
+	int flags;
+
+	w32_getcomm(fd->handle, &flags);
+	return (flags & TIOCM_CD);
+#endif
 }
 
-int ser_flush_io(int fd)
+int ser_close(TYPE_FD_SER fd, const char *port)
 {
-	return tcflush(fd, TCIOFLUSH);
-}
-
-int ser_close(int fd, const char *port)
-{
-	if (fd < 0)
+	if (INVALID_FD_SER(fd)) {
+#ifndef WIN32
 		fatal_with_errno(EXIT_FAILURE, "ser_close: programming error: fd=%d port=%s", fd, port);
+#else
+		fatal_with_errno(EXIT_FAILURE, "ser_close: programming error: struct=%p port=%s", fd, port);
+#endif
+	}
 
 	if (close(fd) != 0)
 		return -1;
@@ -262,12 +336,12 @@ int ser_close(int fd, const char *port)
 	return 0;
 }
 
-ssize_t ser_send_char(int fd, unsigned char ch)
+ssize_t ser_send_char(TYPE_FD_SER fd, unsigned char ch)
 {
 	return ser_send_buf_pace(fd, 0, &ch, 1);
 }
 
-static ssize_t send_formatted(int fd, const char *fmt, va_list va, useconds_t d_usec)
+static ssize_t send_formatted(TYPE_FD_SER fd, const char *fmt, va_list va, useconds_t d_usec)
 {
 	int	ret;
 	char	buf[LARGEBUF];
@@ -294,7 +368,7 @@ static ssize_t send_formatted(int fd, const char *fmt, va_list va, useconds_t d_
 }
 
 /* send the results of the format string with d_usec delay after each char */
-ssize_t ser_send_pace(int fd, useconds_t d_usec, const char *fmt, ...)
+ssize_t ser_send_pace(TYPE_FD_SER fd, useconds_t d_usec, const char *fmt, ...)
 {
 	ssize_t	ret;
 	va_list	ap;
@@ -321,7 +395,7 @@ ssize_t ser_send_pace(int fd, useconds_t d_usec, const char *fmt, ...)
 }
 
 /* send the results of the format string with no delay */
-ssize_t ser_send(int fd, const char *fmt, ...)
+ssize_t ser_send(TYPE_FD_SER fd, const char *fmt, ...)
 {
 	ssize_t	ret;
 	va_list	ap;
@@ -348,13 +422,13 @@ ssize_t ser_send(int fd, const char *fmt, ...)
 }
 
 /* send buflen bytes from buf with no delay */
-ssize_t ser_send_buf(int fd, const void *buf, size_t buflen)
+ssize_t ser_send_buf(TYPE_FD_SER fd, const void *buf, size_t buflen)
 {
 	return ser_send_buf_pace(fd, 0, buf, buflen);
 }
 
 /* send buflen bytes from buf with d_usec delay after each char */
-ssize_t ser_send_buf_pace(int fd, useconds_t d_usec, const void *buf,
+ssize_t ser_send_buf_pace(TYPE_FD_SER fd, useconds_t d_usec, const void *buf,
 	size_t buflen)
 {
 	ssize_t	ret = 0;
@@ -376,7 +450,7 @@ ssize_t ser_send_buf_pace(int fd, useconds_t d_usec, const void *buf,
 	return sent;
 }
 
-ssize_t ser_get_char(int fd, void *ch, time_t d_sec, useconds_t d_usec)
+ssize_t ser_get_char(TYPE_FD_SER fd, void *ch, time_t d_sec, useconds_t d_usec)
 {
 	/* Per standard below, we can cast here, because required ranges are
 	 * effectively the same (and signed -1 for suseconds_t), and at most long:
@@ -385,7 +459,7 @@ ssize_t ser_get_char(int fd, void *ch, time_t d_sec, useconds_t d_usec)
 	return select_read(fd, ch, 1, d_sec, (suseconds_t)d_usec);
 }
 
-ssize_t ser_get_buf(int fd, void *buf, size_t buflen, time_t d_sec, useconds_t d_usec)
+ssize_t ser_get_buf(TYPE_FD_SER fd, void *buf, size_t buflen, time_t d_sec, useconds_t d_usec)
 {
 	memset(buf, '\0', buflen);
 
@@ -393,7 +467,7 @@ ssize_t ser_get_buf(int fd, void *buf, size_t buflen, time_t d_sec, useconds_t d
 }
 
 /* keep reading until buflen bytes are received or a timeout occurs */
-ssize_t ser_get_buf_len(int fd, void *buf, size_t buflen, time_t d_sec, useconds_t d_usec)
+ssize_t ser_get_buf_len(TYPE_FD_SER fd, void *buf, size_t buflen, time_t d_sec, useconds_t d_usec)
 {
 	ssize_t	ret;
 	ssize_t	recv;
@@ -418,7 +492,7 @@ ssize_t ser_get_buf_len(int fd, void *buf, size_t buflen, time_t d_sec, useconds
 
 /* reads a line up to <endchar>, discarding anything else that may follow,
    with callouts to the handler if anything matches the alertset */
-ssize_t ser_get_line_alert(int fd, void *buf, size_t buflen, char endchar,
+ssize_t ser_get_line_alert(TYPE_FD_SER fd, void *buf, size_t buflen, char endchar,
 	const char *ignset, const char *alertset, void handler(char ch),
 	time_t d_sec, useconds_t d_usec)
 {
@@ -463,14 +537,14 @@ ssize_t ser_get_line_alert(int fd, void *buf, size_t buflen, char endchar,
 }
 
 /* as above, only with no alertset handling (just a wrapper) */
-ssize_t ser_get_line(int fd, void *buf, size_t buflen, char endchar,
+ssize_t ser_get_line(TYPE_FD_SER fd, void *buf, size_t buflen, char endchar,
 	const char *ignset, time_t d_sec, useconds_t d_usec)
 {
 	return ser_get_line_alert(fd, buf, buflen, endchar, ignset, "", NULL,
 		d_sec, d_usec);
 }
 
-ssize_t ser_flush_in(int fd, const char *ignset, int verbose)
+ssize_t ser_flush_in(TYPE_FD_SER fd, const char *ignset, int verbose)
 {
 	ssize_t	ret, extra = 0;
 	char	ch;
@@ -494,6 +568,11 @@ ssize_t ser_flush_in(int fd, const char *ignset, int verbose)
 	return extra;
 }
 
+int ser_flush_io(TYPE_FD_SER fd)
+{
+	return tcflush(fd, TCIOFLUSH);
+}
+
 void ser_comm_fail(const char *fmt, ...)
 {
 	int	ret;
