@@ -123,9 +123,10 @@
 
 #include "main.h"
 #include "serial.h"
+#include "nut_stdint.h"
 
-#define DRIVER_NAME		"Tripp Lite SmartOnline driver"
-#define DRIVER_VERSION	"0.06"
+#define DRIVER_NAME	"Tripp Lite SmartOnline driver"
+#define DRIVER_VERSION	"0.07"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -192,9 +193,9 @@ static struct {
 #define RELAY_OFF                    "ROF" /* set */
 #define RELAY_ON                     "RON" /* set */
 #define ATX_RESUME                   "RSM" /* set */
-#define SHUTDOWN_ACTION              "SDA" /* set */
-#define SHUTDOWN_RESTART             "SDR" /* set */
-#define SHUTDOWN_TYPE                "SDT" /* poll/set */
+#define TSU_SHUTDOWN_ACTION          "SDA" /* set */
+#define TSU_SHUTDOWN_RESTART         "SDR" /* set */
+#define TSU_SHUTDOWN_TYPE            "SDT" /* poll/set */
 #define RELAY_STATUS                 "SOL" /* poll/set */
 #define SELECT_OUTPUT_VOLTAGE        "SOV" /* poll/set */
 #define STATUS_ALARM                 "STA" /* poll */
@@ -235,7 +236,7 @@ static ssize_t do_command(char type, const char *command, const char *parameters
 		return -1;
 	}
 
-	upsdebugx(3, "do_command: %zd bytes sent [%s] -> OK", ret, buffer);
+	upsdebugx(3, "do_command: %" PRIiSIZE " bytes sent [%s] -> OK", ret, buffer);
 
 	ret = ser_get_buf_len(upsfd, (unsigned char *)buffer, 4, 3, 0);
 	if (ret < 0) {
@@ -248,9 +249,10 @@ static ssize_t do_command(char type, const char *command, const char *parameters
 	}
 
 	buffer[ret] = '\0';
-	upsdebugx(3, "do_command: %zd byted read [%s]", ret, buffer);
+	upsdebugx(3, "do_command: %" PRIiSIZE " byted read [%s]", ret, buffer);
 
 	if (!strcmp(buffer, "~00D")) {
+		int	c;
 
 		ret = ser_get_buf_len(upsfd, (unsigned char *)buffer, 3, 3, 0);
 		if (ret < 0) {
@@ -263,9 +265,9 @@ static ssize_t do_command(char type, const char *command, const char *parameters
 		}
 
 		buffer[ret] = '\0';
-		upsdebugx(3, "do_command: %zd bytes read [%s]", ret, buffer);
+		upsdebugx(3, "do_command: %" PRIiSIZE " bytes read [%s]", ret, buffer);
 
-		int c = atoi(buffer);
+		c = atoi(buffer);
 		if (c < 0) {
 			upsdebugx(3, "do_command: response not expected to be a negative count!");
 			return -1;
@@ -297,7 +299,7 @@ static ssize_t do_command(char type, const char *command, const char *parameters
 		}
 
 		response[ret] = '\0';
-		upsdebugx(3, "do_command: %zd bytes read [%s]", ret, response);
+		upsdebugx(3, "do_command: %" PRIiSIZE " bytes read [%s]", ret, response);
 
 		/* Tripp Lite pads their string responses with spaces.
 		   I don't like that, so I remove them.  This is safe to
@@ -422,7 +424,7 @@ static int get_sensitivity(void) {
 
 	if (do_command(POLL, VOLTAGE_SENSITIVITY, "", response) <= 0)
 		return 0;
-	for (i = 0; i < sizeof(sensitivity) / sizeof(sensitivity[0]); i++) {
+	for (i = 0; i < SIZEOF_ARRAY(sensitivity); i++) {
 		if (sensitivity[i].code == atoi(response)) {
 			dstate_setinfo("input.sensitivity", "%s",
 			               sensitivity[i].name);
@@ -437,7 +439,7 @@ static void set_sensitivity(const char *val) {
 	char parm[20];
 	unsigned int i;
 
-	for (i = 0; i < sizeof(sensitivity) / sizeof(sensitivity[0]); i++) {
+	for (i = 0; i < SIZEOF_ARRAY(sensitivity); i++) {
 		if (!strcasecmp(val, sensitivity[i].name)) {
 			snprintf(parm, sizeof(parm), "%u", i);
 			do_command(SET, VOLTAGE_SENSITIVITY, parm, NULL);
@@ -486,31 +488,31 @@ static int instcmd(const char *cmdname, const char *extra)
 	}
 	if (!strcasecmp(cmdname, "shutdown.reboot")) {
 		auto_reboot(1);
-		do_command(SET, SHUTDOWN_RESTART, "1", NULL);
-		do_command(SET, SHUTDOWN_ACTION, "10", NULL);
+		do_command(SET, TSU_SHUTDOWN_RESTART, "1", NULL);
+		do_command(SET, TSU_SHUTDOWN_ACTION, "10", NULL);
 		return STAT_INSTCMD_HANDLED;
 	}
 	if (!strcasecmp(cmdname, "shutdown.reboot.graceful")) {
 		auto_reboot(1);
-		do_command(SET, SHUTDOWN_RESTART, "1", NULL);
-		do_command(SET, SHUTDOWN_ACTION, "60", NULL);
+		do_command(SET, TSU_SHUTDOWN_RESTART, "1", NULL);
+		do_command(SET, TSU_SHUTDOWN_ACTION, "60", NULL);
 		return STAT_INSTCMD_HANDLED;
 	}
 	if (!strcasecmp(cmdname, "shutdown.return")) {
 		auto_reboot(1);
-		do_command(SET, SHUTDOWN_RESTART, "1", NULL);
-		do_command(SET, SHUTDOWN_ACTION, "10", NULL);
+		do_command(SET, TSU_SHUTDOWN_RESTART, "1", NULL);
+		do_command(SET, TSU_SHUTDOWN_ACTION, "10", NULL);
 		return STAT_INSTCMD_HANDLED;
 	}
 #if 0 /* doesn't seem to work */
 	if (!strcasecmp(cmdname, "shutdown.stayoff")) {
 		auto_reboot(0);
-		do_command(SET, SHUTDOWN_ACTION, "10", NULL);
+		do_command(SET, TSU_SHUTDOWN_ACTION, "10", NULL);
 		return STAT_INSTCMD_HANDLED;
 	}
 #endif
 	if (!strcasecmp(cmdname, "shutdown.stop")) {
-		do_command(SET, SHUTDOWN_ACTION, "0", NULL);
+		do_command(SET, TSU_SHUTDOWN_ACTION, "0", NULL);
 		return STAT_INSTCMD_HANDLED;
 	}
 	if (!strcasecmp(cmdname, "test.battery.start")) {
@@ -658,8 +660,7 @@ void upsdrv_initinfo(void)
 	}
 	if (get_sensitivity()) {
 		dstate_setflags("input.sensitivity", ST_FLAG_RW);
-		for (i = 0; i < sizeof(sensitivity) / sizeof(sensitivity[0]);
-		     i++)
+		for (i = 0; i < SIZEOF_ARRAY(sensitivity); i++)
 			dstate_addenum("input.sensitivity", "%s",
 			               sensitivity[i].name);
 	}
@@ -809,8 +810,7 @@ void upsdrv_updateinfo(void)
 		size_t	trsize;
 
 		r = atoi(response);
-		trsize = sizeof(test_result_names) /
-			sizeof(test_result_names[0]);
+		trsize = SIZEOF_ARRAY(test_result_names);
 
 		if ((r < 0) || (r >= (int) trsize))
 			r = 0;
@@ -854,9 +854,9 @@ void upsdrv_shutdown(void)
 	/* in case the power is on, tell it to automatically reboot.  if
 	   it is off, this has no effect. */
 	snprintf(parm, sizeof(parm), "%d", 1); /* delay before reboot, in minutes */
-	do_command(SET, SHUTDOWN_RESTART, parm, NULL);
+	do_command(SET, TSU_SHUTDOWN_RESTART, parm, NULL);
 	snprintf(parm, sizeof(parm), "%d", 5); /* delay before shutdown, in seconds */
-	do_command(SET, SHUTDOWN_ACTION, parm, NULL);
+	do_command(SET, TSU_SHUTDOWN_ACTION, parm, NULL);
 }
 
 void upsdrv_help(void)
