@@ -17,6 +17,7 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#include "config.h"	/* must be first */
 #include "common.h"
 #include "usb-common.h"
 
@@ -54,27 +55,6 @@ int is_usb_device_supported(usb_device_id_t *usb_device_id_list, USBDevice_t *de
 /* ---------------------------------------------------------------------- */
 /* matchers */
 
-/* helper function: version of strcmp that tolerates NULL
- * pointers. NULL is considered to come before all other strings
- * alphabetically.
- */
-static int strcmp_null(char *s1, char *s2)
-{
-	if (s1 == NULL && s2 == NULL) {
-		return 0;
-	}
-
-	if (s1 == NULL) {
-		return -1;
-	}
-
-	if (s2 == NULL) {
-		return 1;
-	}
-
-	return strcmp(s1, s2);
-}
-
 /* private callback function for exact matches
  */
 static int match_function_exact(USBDevice_t *hd, void *privdata)
@@ -119,6 +99,15 @@ static int match_function_exact(USBDevice_t *hd, void *privdata)
 		return 0;
 	}
 #endif
+#if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+  #ifdef DEBUG_EXACT_MATCH_BUSPORT
+	if (strcmp_null(hd->BusPort, data->BusPort) != 0) {
+		upsdebugx(2, "%s: failed match of %s: %s != %s",
+		    __func__, "BusPort", hd->BusPort, data->BusPort);
+		return 0;
+	}
+  #endif
+#endif
 #ifdef DEBUG_EXACT_MATCH_DEVICE
 	if (strcmp_null(hd->Device, data->Device) != 0) {
 		upsdebugx(2, "%s: failed match of %s: %s != %s",
@@ -193,114 +182,9 @@ void USBFreeExactMatcher(USBDeviceMatcher_t *matcher)
 	free(matcher);
 }
 
-/* Private function for compiling a regular expression. On success,
- * store the compiled regular expression (or NULL) in *compiled, and
- * return 0. On error with errno set, return -1. If the supplied
- * regular expression is unparseable, return -2 (an error message can
- * then be retrieved with regerror(3)). Note that *compiled will be an
- * allocated value, and must be freed with regfree(), then free(), see
- * regex(3). As a special case, if regex==NULL, then set
- * *compiled=NULL (regular expression NULL is intended to match
- * anything).
- */
-static int compile_regex(regex_t **compiled, char *regex, int cflags)
-{
-	int	r;
-	regex_t	*preg;
-
-	if (regex == NULL) {
-		*compiled = NULL;
-		return 0;
-	}
-
-	preg = malloc(sizeof(*preg));
-	if (!preg) {
-		return -1;
-	}
-
-	r = regcomp(preg, regex, cflags);
-	if (r) {
-		free(preg);
-		return -2;
-	}
-
-	*compiled = preg;
-
-	return 0;
-}
-
-/* Private function for regular expression matching. Check if the
- * entire string str (minus any initial and trailing whitespace)
- * matches the compiled regular expression preg. Return 1 if it
- * matches, 0 if not. Return -1 on error with errno set. Special
- * cases: if preg==NULL, it matches everything (no contraint).  If
- * str==NULL, then it is treated as "".
- */
-static int match_regex(regex_t *preg, char *str)
-{
-	int	r;
-	size_t	len = 0;
-	char	*string;
-	regmatch_t	match;
-
-	if (!preg) {
-		return 1;
-	}
-
-	if (!str) {
-		string = xstrdup("");
-	} else {
-		/* skip leading whitespace */
-		for (len = 0; len < strlen(str); len++) {
-
-			if (!strchr(" \t\n", str[len])) {
-				break;
-			}
-		}
-
-		string = xstrdup(str+len);
-
-		/* skip trailing whitespace */
-		for (len = strlen(string); len > 0; len--) {
-
-			if (!strchr(" \t\n", string[len-1])) {
-				break;
-			}
-		}
-
-		string[len] = '\0';
-	}
-
-	/* test the regular expression */
-	r = regexec(preg, string, 1, &match, 0);
-	free(string);
-	if (r) {
-		return 0;
-	}
-
-	/* check that the match is the entire string */
-	if ((match.rm_so != 0) || (match.rm_eo != (int)len)) {
-		return 0;
-	}
-
-	return 1;
-}
-
-/* Private function, similar to match_regex, but the argument being
- * matched is a (hexadecimal) number, rather than a string. It is
- * converted to a 4-digit hexadecimal string. */
-static int match_regex_hex(regex_t *preg, int n)
-{
-	char	buf[10];
-
-	snprintf(buf, sizeof(buf), "%04x", n);
-
-	return match_regex(preg, buf);
-}
-
 /* private data type: hold a set of compiled regular expressions. */
 typedef struct regex_matcher_data_s {
-	regex_t	*regex[7];
+	regex_t	*regex[USBMATCHER_REGEXP_ARRAY_LIMIT];
 } regex_matcher_data_t;
 
 /* private callback function for regex matches */
@@ -311,6 +195,9 @@ static int match_function_regex(USBDevice_t *hd, void *privdata)
 
 	upsdebugx(3, "%s: matching a device...", __func__);
 
+	/* NOTE: Here and below the "detailed" logging is commented away
+	 * because data->regex[] elements are not strings anymore! */
+
 	r = match_regex_hex(data->regex[0], hd->VendorID);
 	if (r != 1) {
 /*
@@ -387,6 +274,19 @@ static int match_function_regex(USBDevice_t *hd, void *privdata)
 		    __func__, "Device", hd->Device);
 		return r;
 	}
+
+#if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+	r = match_regex(data->regex[7], hd->BusPort);
+	if (r != 1) {
+/*
+		upsdebugx(2, "%s: failed match of %s: %s !~ %s",
+		    __func__, "Device", hd->Device, data->regex[6]);
+*/
+		upsdebugx(2, "%s: failed match of %s: %s",
+		    __func__, "Bus Port", hd->BusPort);
+		return r;
+	}
+#endif
 	return 1;
 }
 
@@ -423,7 +323,7 @@ int USBNewRegexMatcher(USBDeviceMatcher_t **matcher, char **regex, int cflags)
 	m->privdata = (void *)data;
 	m->next = NULL;
 
-	for (i=0; i<7; i++) {
+	for (i=0; i < USBMATCHER_REGEXP_ARRAY_LIMIT; i++) {
 		r = compile_regex(&data->regex[i], regex[i], cflags);
 		if (r == -2) {
 			r = i+1;
@@ -450,7 +350,7 @@ void USBFreeRegexMatcher(USBDeviceMatcher_t *matcher)
 
 	data = (regex_matcher_data_t *)matcher->privdata;
 
-	for (i = 0; i < 7; i++) {
+	for (i = 0; i < USBMATCHER_REGEXP_ARRAY_LIMIT; i++) {
 		if (!data->regex[i]) {
 			continue;
 		}
