@@ -28,18 +28,25 @@
  */
 
 #define DRIVER_NAME	"Generic HID driver"
-#define DRIVER_VERSION		"0.47"
+#define DRIVER_VERSION	"0.52"
 
-#include "main.h"
+#define HU_VAR_WAITBEFORERECONNECT "waitbeforereconnect"
+
+#include "main.h"	/* Must be first, includes "config.h" */
+#include "nut_stdint.h"
 #include "libhid.h"
 #include "usbhid-ups.h"
 #include "hidparser.h"
 #include "hidtypes.h"
+#include "common.h"
+#ifdef WIN32
+#include "wincompat.h"
+#endif
 
 /* include all known subdrivers */
 #include "mge-hid.h"
 
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 	/* explore stub goes first, others alphabetically */
 	#include "explore-hid.h"
 	#include "apc-hid.h"
@@ -56,15 +63,16 @@
 	#include "powervar-hid.h"
 	#include "salicru-hid.h"
 	#include "tripplite-hid.h"
-#endif
+#endif	/* !SHUT_MODE => USB */
 
-/* Reference list of avaiable subdrivers */
+/* Reference list of available subdrivers */
 static subdriver_t *subdriver_list[] = {
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 	&explore_subdriver,
-#endif
+#endif	/* !SHUT_MODE => USB */
+	/* mge-hid.c supports both SHUT and USB */
 	&mge_subdriver,
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 	&apc_subdriver,
 	&arduino_subdriver,
 	&belkin_subdriver,
@@ -79,7 +87,7 @@ static subdriver_t *subdriver_list[] = {
 	&powervar_subdriver,
 	&salicru_subdriver,
 	&tripplite_subdriver,
-#endif
+#endif	/* !SHUT_MODE => USB */
 	NULL
 };
 
@@ -91,11 +99,11 @@ upsdrv_info_t upsdrv_info = {
 	"Arjen de Korte <adkorte-guest@alioth.debian.org>\n" \
 	"John Stamp <kinsayder@hotmail.com>",
 	/*FIXME: link the subdrivers? do the same as for the mibs! */
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 	DRV_STABLE,
-#else
+#else	/* SHUT_MODE */
 	DRV_EXPERIMENTAL,
-#endif
+#endif	/* SHUT_MODE / USB */
 	{ &comm_upsdrv_info, NULL }
 };
 
@@ -106,38 +114,21 @@ typedef enum {
 	HU_WALKMODE_FULL_UPDATE
 } walkmode_t;
 
-/* Compatibility layer between libusb 0.1 and 1.0, for errno/return codes */
-#if WITH_LIBUSB_0_1 || (defined SHUT_MODE)
- #define ERROR_BUSY	-EBUSY
- #define ERROR_NO_DEVICE -ENODEV
- #define ERROR_ACCESS -EACCES
- #define ERROR_IO -EIO
- #define ERROR_NOT_FOUND -ENOENT
- #define ERROR_TIMEOUT -ETIMEDOUT
- #define ERROR_OVERFLOW -EOVERFLOW
- #define ERROR_PIPE -EPIPE
-#else /* for libusb 1.0 */
- #define ERROR_BUSY	LIBUSB_ERROR_BUSY
- #define ERROR_NO_DEVICE LIBUSB_ERROR_NO_DEVICE
- #define ERROR_ACCESS LIBUSB_ERROR_ACCESS
- #define ERROR_IO LIBUSB_ERROR_IO
- #define ERROR_NOT_FOUND LIBUSB_ERROR_NOT_FOUND
- #define ERROR_TIMEOUT LIBUSB_ERROR_TIMEOUT
- #define ERROR_OVERFLOW LIBUSB_ERROR_OVERFLOW
- #define ERROR_PIPE LIBUSB_ERROR_PIPE
-#endif
-
 /* pointer to the active subdriver object (changed in callback() function) */
 static subdriver_t *subdriver = NULL;
 
 /* Global vars */
 static HIDDevice_t *hd = NULL;
-static HIDDevice_t curDevice = { 0x0000, 0x0000, NULL, NULL, NULL, NULL, 0, NULL };
+static HIDDevice_t curDevice = { 0x0000, 0x0000, NULL, NULL, NULL, NULL, 0, NULL
+#if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+	, NULL
+#endif
+};
 static HIDDeviceMatcher_t *subdriver_matcher = NULL;
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 static HIDDeviceMatcher_t *exact_matcher = NULL;
 static HIDDeviceMatcher_t *regex_matcher = NULL;
-#endif
+#endif	/* !SHUT_MODE => USB */
 static int pollfreq = DEFAULT_POLLFREQ;
 static unsigned ups_status = 0;
 static bool_t data_has_changed = FALSE; /* for SEMI_STATIC data polling */
@@ -146,6 +137,7 @@ bool_t use_interrupt_pipe = TRUE;
 #else
 bool_t use_interrupt_pipe = FALSE;
 #endif
+static size_t interrupt_pipe_EIO_count = 0; /* How many times we had I/O errors since last reconnect? */
 static time_t lastpoll; /* Timestamp the last polling */
 hid_dev_handle_t udev = HID_DEV_HANDLE_CLOSED;
 
@@ -156,6 +148,13 @@ hid_dev_handle_t udev = HID_DEV_HANDLE_CLOSED;
  */
 static int onlinedischarge = 0;
 
+/**
+ * Some UPS models (e.g. APC were seen to do so) report OL & DISCHRG when they
+ * are in calibration mode. This usually happens after a few seconds reporting
+ * an "OFF" state as well, while the hardware is switching to on-battery mode.
+ */
+static int onlinedischarge_calibration = 0;
+
 /* support functions */
 static hid_info_t *find_nut_info(const char *varname);
 static hid_info_t *find_hid_info(const HIDData_t *hiddata);
@@ -176,7 +175,7 @@ static double interval(void);
 /* global variables */
 HIDDesc_t	*pDesc = NULL;		/* parsed Report Descriptor */
 reportbuf_t	*reportbuf = NULL;	/* buffer for most recent reports */
-
+int disable_fix_report_desc = 0; /* by default we apply fix-ups for broken USB encoding, etc. */
 
 /* --------------------------------------------------------------- */
 /* Struct & data for boolean processing                            */
@@ -204,7 +203,7 @@ static status_lkp_t status_info[] = {
 	{ "bypassauto", STATUS(BYPASSAUTO) },
 	{ "bypassman", STATUS(BYPASSMAN) },
 	{ "off", STATUS(OFF) },
-	{ "cal", STATUS(CAL) },
+	{ "cal", STATUS(CALIB) },
 	{ "overheat", STATUS(OVERHEAT) },
 	{ "commfault", STATUS(COMMFAULT) },
 	{ "depleted", STATUS(DEPLETED) },
@@ -423,6 +422,12 @@ info_lkp_t on_off_info[] = {
    done with result! */
 static const char *date_conversion_fun(double value)
 {
+	/* Per  spec https://www.usb.org/sites/default/files/pdcv11.pdf (page 38):
+	 * 4.2.6 Battery Settings -> ManufacturerDate
+	 *   The date the pack was manufactured in a packed integer.
+	 *   The date is packed in the following fashion:
+	 *   (year – 1980)*512 + month*32 + day.
+	 */
 	static char buf[32];
 	long year, month, day;
 
@@ -534,16 +539,137 @@ info_lkp_t kelvin_celsius_conversion[] = {
 	{ 0, NULL, kelvin_celsius_conversion_fun, NULL }
 };
 
+static subdriver_t *match_function_subdriver_name(int fatal_mismatch) {
+	char	*subdrv = getval("subdriver");
+	subdriver_t	*info = NULL;
+
+	/* Pick up the subdriver name if set explicitly */
+	if (subdrv) {
+		int	i, flag_HAVE_LIBREGEX = 0;
+#if (defined HAVE_LIBREGEX && HAVE_LIBREGEX)
+		int	res;
+		size_t	len;
+		regex_t	*regex_ptr = NULL;
+		flag_HAVE_LIBREGEX = 1;
+#endif
+
+		upsdebugx(2,
+			"%s: matching a subdriver by explicit "
+			"name%s: '%s'...",
+			__func__,
+			flag_HAVE_LIBREGEX ?	"/regex" : "",
+			subdrv);
+
+		/* First try exact match for strings like "TrippLite HID 0.85"
+		 * Not likely to hit (due to versions etc.), but worth a try :)
+		 */
+		for (i=0; subdriver_list[i] != NULL; i++) {
+			if (strcmp_null(subdrv, subdriver_list[i]->name) == 0) {
+				info = subdriver_list[i];
+				goto found;
+			}
+		}
+
+#if (defined HAVE_LIBREGEX && HAVE_LIBREGEX)
+		/* Then try a case-insensitive regex like "tripplite.*"
+		 * if so provided by caller */
+		upsdebugx(2, "%s: retry matching by regex 'as is'", __func__);
+		res = compile_regex(&regex_ptr, subdrv, REG_ICASE | REG_EXTENDED);
+		if (res == 0 && regex_ptr != NULL) {
+			for (i=0; subdriver_list[i] != NULL; i++) {
+				res = match_regex(regex_ptr, subdriver_list[i]->name);
+				if (res == 1) {
+					free(regex_ptr);
+					info = subdriver_list[i];
+					goto found;
+				}
+			}
+		}
+
+		if (regex_ptr) {
+			free(regex_ptr);
+			regex_ptr = NULL;
+		}
+
+		/* Then try a case-insensitive regex like "tripplite.*"
+		 * with automatically added ".*" */
+		len = strlen(subdrv);
+		if (
+			(len < 3 || (subdrv[len-2] != '.' && subdrv[len-1] != '*'))
+			&& len < (LARGEBUF-3)
+		) {
+			char	buf[LARGEBUF];
+			upsdebugx(2, "%s: retry matching by regex with added '.*'", __func__);
+			snprintf(buf, sizeof(buf), "%s.*", subdrv);
+			res = compile_regex(&regex_ptr, buf, REG_ICASE | REG_EXTENDED);
+			if (res == 0 && regex_ptr != NULL) {
+				for (i=0; subdriver_list[i] != NULL; i++) {
+					res = match_regex(regex_ptr, subdriver_list[i]->name);
+					if (res == 1) {
+						free(regex_ptr);
+						info = subdriver_list[i];
+						goto found;
+					}
+				}
+			}
+
+			if (regex_ptr) {
+				free(regex_ptr);
+				regex_ptr = NULL;
+			}
+		}
+#endif	/* HAVE_LIBREGEX */
+
+		if (fatal_mismatch) {
+			fatalx(EXIT_FAILURE,
+				"Configuration requested subdriver '%s' but none matched",
+				subdrv);
+		} else {
+			upslogx(LOG_WARNING,
+				"Configuration requested subdriver '%s' but none matched; "
+				"will try USB matching by other fields",
+				subdrv);
+		}
+	}
+
+	/* No match (and non-fatal mismatch mode), or no
+	 * "subdriver" was specified in configuration */
+	return NULL;
+
+found:
+	upsdebugx(2, "%s: found a match: %s", __func__, info->name);
+	if (!getval("vendorid") || !getval("productid")) {
+		if (fatal_mismatch) {
+			fatalx(EXIT_FAILURE,
+				"When specifying a subdriver, "
+				"'vendorid' and 'productid' "
+				"are mandatory.");
+		} else {
+			upslogx(LOG_WARNING,
+				"When specifying a subdriver, "
+				"'vendorid' and 'productid' "
+				"are highly recommended.");
+		}
+	}
+
+	return info;
+}
+
 /*!
  * subdriver matcher: only useful for USB mode
  * as SHUT is only supported by MGE UPS SYSTEMS units
  */
 
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 static int match_function_subdriver(HIDDevice_t *d, void *privdata) {
-	int i;
+	int	i;
 	NUT_UNUSED_VARIABLE(privdata);
 
+	if (match_function_subdriver_name(1)) {
+		/* This driver can handle this device. Guessing so... */
+		return 1;
+	}
+
 	upsdebugx(2, "%s (non-SHUT mode): matching a device...", __func__);
 
 	for (i=0; subdriver_list[i] != NULL; i++) {
@@ -563,7 +689,7 @@ static HIDDeviceMatcher_t subdriver_matcher_struct = {
 	NULL,
 	NULL
 };
-#endif
+#endif	/* !SHUT_MODE => USB */
 
 /* ---------------------------------------------
  * driver functions implementations
@@ -768,12 +894,25 @@ void upsdrv_shutdown(void)
 		return;
 	}
 
-	fatalx(EXIT_FAILURE, "Shutdown failed!");
+	upslogx(LOG_ERR, "Shutdown failed!");
+	set_exit_flag(-1);
 }
 
 void upsdrv_help(void)
 {
-	/* FIXME: to be completed */
+	size_t i;
+	printf("\nAcceptable values for 'subdriver' via -x or ups.conf "
+		"in this driver (exact names here, case-insensitive "
+		"sub-strings may be used, as well as regular expressions): ");
+
+	for (i = 0; subdriver_list[i] != NULL; i++) {
+		if (i>0)
+			printf(", ");
+		printf("\"%s\"", subdriver_list[i]->name);
+	}
+	printf("\n\n");
+
+	printf("Read The Fine Manual ('man 8 usbhid-ups')\n");
 }
 
 void upsdrv_makevartable(void)
@@ -807,7 +946,15 @@ void upsdrv_makevartable(void)
 	addvar(VAR_FLAG, "onlinedischarge",
 		"Set to treat discharging while online as being offline");
 
-#ifndef SHUT_MODE
+	addvar(VAR_FLAG, "onlinedischarge_calibration",
+		"Set to treat discharging while online as doing calibration");
+
+	addvar(VAR_FLAG, "disable_fix_report_desc",
+		"Set to disable fix-ups for broken USB encoding, etc. which we apply by default on certain vendors/products");
+
+#if !((defined SHUT_MODE) && SHUT_MODE)
+	addvar(VAR_VALUE, "subdriver", "Explicit USB HID subdriver selection");
+
 	/* allow -x vendor=X, vendorid=X, product=X, productid=X, serial=X */
 	nut_usb_addvars();
 
@@ -819,10 +966,13 @@ void upsdrv_makevartable(void)
 		"Don't use polling, only use interrupt pipe");
 	addvar(VAR_VALUE, "interruptsize",
 		"Number of bytes to read from interrupt pipe");
-#else
+	addvar(VAR_VALUE, HU_VAR_WAITBEFORERECONNECT,
+		"Seconds to wait before trying to reconnect");
+
+#else	/* SHUT_MODE */
 	addvar(VAR_VALUE, "notification",
-		"Set notification type, (ignored, only for backward compatibility)");
-#endif
+		"Set notification type (ignored, only for backward compatibility)");
+#endif	/* SHUT_MODE / USB */
 }
 
 #define	MAX_EVENT_NUM	32
@@ -846,7 +996,12 @@ void upsdrv_updateinfo(void)
 			return;
 		}
 
-		upsdebugx(1, "Got to reconnect!\n");
+		upsdebugx(1, "Got to reconnect!");
+		if (use_interrupt_pipe == TRUE && interrupt_pipe_EIO_count > 0) {
+			upsdebugx(0, "\nReconnecting. If you saw \"nut_libusb_get_interrupt: Input/Output Error\" "
+				"or similar message in the log above, try setting \"pollonly\" flag in \"ups.conf\" "
+				"options section for this driver!\n");
+		}
 
 		if (!reconnect_ups()) {
 			lastpoll = now;
@@ -855,6 +1010,7 @@ void upsdrv_updateinfo(void)
 		}
 
 		hd = &curDevice;
+		interrupt_pipe_EIO_count = 0;
 
 		if (hid_ups_walk(HU_WALKMODE_INIT) == FALSE) {
 			hd = NULL;
@@ -864,26 +1020,34 @@ void upsdrv_updateinfo(void)
 #ifdef DEBUG
 	interval();
 #endif
+
 	/* Get HID notifications on Interrupt pipe first */
 	if (use_interrupt_pipe == TRUE) {
 		evtCount = HIDGetEvents(udev, event, MAX_EVENT_NUM);
 		switch (evtCount)
 		{
-		case ERROR_BUSY:      /* Device or resource busy */
+		case LIBUSB_ERROR_BUSY:      /* Device or resource busy */
 			upslog_with_errno(LOG_CRIT, "Got disconnected by another driver");
 			goto fallthrough_reconnect;
 #if WITH_LIBUSB_0_1 /* limit to libusb 0.1 implementation */
 		case -EPERM:		/* Operation not permitted */
 #endif
-		case ERROR_NO_DEVICE: /* No such device */
-		case ERROR_ACCESS:    /* Permission denied */
-		case ERROR_IO:        /* I/O error */
-#if WITH_LIBUSB_0_1 /* limit to libusb 0.1 implementation */
-		case -ENXIO:		/* No such device or address */
+		case LIBUSB_ERROR_NO_DEVICE: /* No such device */
+		case LIBUSB_ERROR_ACCESS:    /* Permission denied */
+#if WITH_LIBUSB_0_1         /* limit to libusb 0.1 implementation */
+		case -ENXIO:		    /* No such device or address */
 #endif
-		case ERROR_NOT_FOUND: /* No such file or directory */
+		case LIBUSB_ERROR_NOT_FOUND: /* No such file or directory */
+		case LIBUSB_ERROR_NO_MEM:    /* Insufficient memory */
 		fallthrough_reconnect:
 			/* Uh oh, got to reconnect! */
+			dstate_setinfo("driver.state", "reconnect.trying");
+			hd = NULL;
+			return;
+		case LIBUSB_ERROR_IO:        /* I/O error */
+			/* Uh oh, got to reconnect, with a special suggestion! */
+			dstate_setinfo("driver.state", "reconnect.trying");
+			interrupt_pipe_EIO_count++;
 			hd = NULL;
 			return;
 		default:
@@ -912,10 +1076,10 @@ void upsdrv_updateinfo(void)
 
 		/* Skip Input reports, if we don't use the Feature report */
 		found_data = FindObject_with_Path(pDesc, &(event[i]->Path), interrupt_only ? ITEM_INPUT:ITEM_FEATURE);
-		if(!found_data && !interrupt_only) {
+		if (!found_data && !interrupt_only) {
 			found_data = FindObject_with_Path(pDesc, &(event[i]->Path), ITEM_INPUT);
 		}
-		if(!found_data) {
+		if (!found_data) {
 			upsdebugx(2, "Could not find event as either ITEM_INPUT or ITEM_FEATURE?");
 			continue;
 		}
@@ -931,7 +1095,7 @@ void upsdrv_updateinfo(void)
 	upsdebugx(1, "took %.3f seconds handling interrupt reports...\n",
 		interval());
 #endif
-	/* clear status buffer before begining */
+	/* clear status buffer before beginning */
 	status_init();
 
 	/* Do a full update (polling) every pollfreq
@@ -1000,12 +1164,7 @@ void upsdrv_initups(void)
 	int ret;
 	char *val;
 
-	upsdebugx(2, "Initializing an USB-connected UPS with library %s " \
-		"(NUT subdriver name='%s' ver='%s')",
-		dstate_getinfo("driver.version.usb"),
-		comm_driver->name, comm_driver->version );
-
-#ifdef SHUT_MODE
+#if (defined SHUT_MODE) && SHUT_MODE
 	/*!
 	 * SHUT is a serial protocol, so it needs
 	 * only the device path
@@ -1013,10 +1172,16 @@ void upsdrv_initups(void)
 	upsdebugx(1, "upsdrv_initups (SHUT)...");
 
 	subdriver_matcher = device_path;
-#else
-	char *regex_array[7];
+#else	/* !SHUT_MODE => USB */
+	char *regex_array[USBMATCHER_REGEXP_ARRAY_LIMIT];
 
 	upsdebugx(1, "upsdrv_initups (non-SHUT)...");
+
+	upsdebugx(2, "Initializing an USB-connected UPS with library %s " \
+		"(NUT subdriver name='%s' ver='%s')",
+		dstate_getinfo("driver.version.usb"),
+		comm_driver->name, comm_driver->version );
+
 	warn_if_bad_usb_port_filename(device_path);
 
 	subdriver_matcher = &subdriver_matcher_struct;
@@ -1039,6 +1204,13 @@ void upsdrv_initups(void)
 	regex_array[4] = getval("serial");
 	regex_array[5] = getval("bus");
 	regex_array[6] = getval("device");
+#if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+	regex_array[7] = getval("busport");
+#else
+	if (getval("busport")) {
+		upslogx(LOG_WARNING, "\"busport\" is configured for the device, but is not actually handled by current build combination of NUT and libusb (ignored)");
+	}
+#endif
 
 	ret = USBNewRegexMatcher(&regex_matcher, regex_array, REG_ICASE | REG_EXTENDED);
 	switch(ret)
@@ -1063,11 +1235,11 @@ void upsdrv_initups(void)
 
 	/* link the matchers */
 	subdriver_matcher->next = regex_matcher;
-#endif /* SHUT_MODE */
+#endif /* SHUT_MODE / USB */
 
 	/* Search for the first supported UPS matching the
 	   regular expression (USB) or device_path (SHUT) */
-	ret = comm_driver->open(&udev, &curDevice, subdriver_matcher, &callback);
+	ret = comm_driver->open_dev(&udev, &curDevice, subdriver_matcher, &callback);
 	if (ret < 1)
 		fatalx(EXIT_FAILURE, "No matching HID UPS found");
 
@@ -1087,6 +1259,14 @@ void upsdrv_initups(void)
 		onlinedischarge = 1;
 	}
 
+	if (testvar("onlinedischarge_calibration")) {
+		onlinedischarge_calibration = 1;
+	}
+
+	if (testvar("disable_fix_report_desc")) {
+		disable_fix_report_desc = 1;
+	}
+
 	val = getval("interruptsize");
 	if (val) {
 		int ipv = atoi(val);
@@ -1146,10 +1326,10 @@ void upsdrv_cleanup(void)
 {
 	upsdebugx(1, "upsdrv_cleanup...");
 
-	comm_driver->close(udev);
+	comm_driver->close_dev(udev);
 	Free_ReportDesc(pDesc);
 	free_report_buffer(reportbuf);
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 	USBFreeExactMatcher(exact_matcher);
 	USBFreeRegexMatcher(regex_matcher);
 
@@ -1158,7 +1338,10 @@ void upsdrv_cleanup(void)
 	free(curDevice.Serial);
 	free(curDevice.Bus);
 	free(curDevice.Device);
-#endif
+# if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+	free(curDevice.BusPort);
+# endif
+#endif	/* !SHUT_MODE => USB */
 }
 
 /**********************************************************************
@@ -1215,9 +1398,9 @@ static int callback(
 {
 	int i;
 	const char *mfr = NULL, *model = NULL, *serial = NULL;
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 	int ret;
-#endif
+#endif	/* !SHUT_MODE => USB */
 	upsdebugx(2, "Report Descriptor size = %" PRI_NUT_USB_CTRL_CHARBUFSIZE, rdlen);
 
 #if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) )
@@ -1261,13 +1444,17 @@ static int callback(
 	}
 
 	/* select the subdriver for this device */
-	for (i=0; subdriver_list[i] != NULL; i++) {
-		if (subdriver_list[i]->claim(hd)) {
-			break;
+	subdriver = match_function_subdriver_name(0);
+	if (!subdriver) {
+		for (i=0; subdriver_list[i] != NULL; i++) {
+			if (subdriver_list[i]->claim(hd)) {
+				break;
+			}
 		}
+
+		subdriver = subdriver_list[i];
 	}
 
-	subdriver = subdriver_list[i];
 	if (!subdriver) {
 		upsdebugx(1, "Manufacturer not supported!");
 		return 0;
@@ -1280,7 +1467,7 @@ static int callback(
 	}
 	HIDDumpTree(udev, arghd, subdriver->utab);
 
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 	/* create a new matcher for later matching */
 	USBFreeExactMatcher(exact_matcher);
 	ret = USBNewExactMatcher(&exact_matcher, hd);
@@ -1290,7 +1477,7 @@ static int callback(
 	}
 
 	regex_matcher->next = exact_matcher;
-#endif /* SHUT_MODE */
+#endif	/* !SHUT_MODE => USB */
 
 	/* apply subdriver specific formatting */
 	mfr = subdriver->format_mfr(hd);
@@ -1343,6 +1530,9 @@ int fix_report_desc(HIDDevice_t *arg_pDev, HIDDesc_t *arg_pDesc) {
 	NUT_UNUSED_VARIABLE(arg_pDev);
 	NUT_UNUSED_VARIABLE(arg_pDesc);
 
+/* Implementations should honor the user's toggle:
+ *	if (disable_fix_report_desc) return 0;
+ */
 	return 0;
 }
 
@@ -1353,11 +1543,11 @@ static bool_t hid_ups_walk(walkmode_t mode)
 	double		value;
 	int		retcode;
 
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 	/* extract the VendorId for further testing */
 	int vendorID = curDevice.VendorID;
 	int productID = curDevice.ProductID;
-#endif
+#endif	/* !SHUT_MODE => USB */
 
 	/* 3 modes: HU_WALKMODE_INIT, HU_WALKMODE_QUICK_UPDATE
 	 * and HU_WALKMODE_FULL_UPDATE */
@@ -1365,13 +1555,14 @@ static bool_t hid_ups_walk(walkmode_t mode)
 	/* Device data walk ----------------------------- */
 	for (item = subdriver->hid2nut; item->info_type != NULL; item++) {
 
-#ifdef SHUT_MODE
+#if (defined SHUT_MODE) && SHUT_MODE
 		/* Check if we are asked to stop (reactivity++) in SHUT mode.
 		 * In USB mode, looping through this takes well under a second,
 		 * so any effort to improve reactivity here is wasted. */
 		if (exit_flag != 0)
 			return TRUE;
-#endif
+#endif	/* SHUT_MODE */
+
 		/* filter data according to mode */
 		switch (mode)
 		{
@@ -1465,34 +1656,43 @@ static bool_t hid_ups_walk(walkmode_t mode)
 # pragma GCC diagnostic pop
 #endif
 
-#ifndef SHUT_MODE
+#if !((defined SHUT_MODE) && SHUT_MODE)
 		/* skip report 0x54 for Tripplite SU3000LCD2UHV due to firmware bug */
 		if ((vendorID == 0x09ae) && (productID == 0x1330)) {
 			if (item->hiddata && (item->hiddata->ReportID == 0x54)) {
 				continue;
 			}
 		}
-#endif
+#endif	/* !SHUT_MODE => USB */
 
 		retcode = HIDGetDataValue(udev, item->hiddata, &value, poll_interval);
 
 		switch (retcode)
 		{
-		case ERROR_BUSY:      /* Device or resource busy */
+		case LIBUSB_ERROR_BUSY:      /* Device or resource busy */
 			upslog_with_errno(LOG_CRIT, "Got disconnected by another driver");
 			goto fallthrough_reconnect;
+
 #if WITH_LIBUSB_0_1 /* limit to libusb 0.1 implementation */
 		case -EPERM:		/* Operation not permitted */
 #endif
-		case ERROR_NO_DEVICE: /* No such device */
-		case ERROR_ACCESS:    /* Permission denied */
-		case ERROR_IO:        /* I/O error */
-#if WITH_LIBUSB_0_1 /* limit to libusb 0.1 implementation */
-		case -ENXIO:		/* No such device or address */
+		case LIBUSB_ERROR_NO_DEVICE: /* No such device */
+		case LIBUSB_ERROR_ACCESS:    /* Permission denied */
+#if WITH_LIBUSB_0_1           /* limit to libusb 0.1 implementation */
+		case -ENXIO:		  /* No such device or address */
 #endif
-		case ERROR_NOT_FOUND: /* No such file or directory */
+		case LIBUSB_ERROR_NOT_FOUND: /* No such file or directory */
+		case LIBUSB_ERROR_NO_MEM:    /* Insufficient memory */
 		fallthrough_reconnect:
 			/* Uh oh, got to reconnect! */
+			dstate_setinfo("driver.state", "reconnect.trying");
+			hd = NULL;
+			return FALSE;
+
+		case LIBUSB_ERROR_IO:        /* I/O error */
+			/* Uh oh, got to reconnect, with a special suggestion! */
+			dstate_setinfo("driver.state", "reconnect.trying");
+			interrupt_pipe_EIO_count++;
 			hd = NULL;
 			return FALSE;
 
@@ -1502,14 +1702,19 @@ static bool_t hid_ups_walk(walkmode_t mode)
 		case 0:
 			continue;
 
-		case ERROR_TIMEOUT:   /* Connection timed out */
-		case ERROR_OVERFLOW:  /* Value too large for defined data type */
-#if EPROTO && WITH_LIBUSB_0_1
+		case LIBUSB_ERROR_TIMEOUT:   /* Connection timed out */
+/* libusb win32 does not know EPROTO and EOVERFLOW,
+ * it only returns EIO for any IO errors */
+#ifndef WIN32
+		case LIBUSB_ERROR_OVERFLOW:  /* Value too large for defined data type */
+# if EPROTO && WITH_LIBUSB_0_1
 		case -EPROTO:		/* Protocol error */
+# endif
 #endif
-		case ERROR_PIPE:      /* Broken pipe */
+		case LIBUSB_ERROR_PIPE:      /* Broken pipe */
 		default:
 			/* Don't know what happened, try again later... */
+		   upsdebugx(1, "HIDGetDataValue unknown retcode '%i'", retcode);
 			continue;
 		}
 
@@ -1564,21 +1769,44 @@ static bool_t hid_ups_walk(walkmode_t mode)
 static int reconnect_ups(void)
 {
 	int ret;
+	char	*val;
+	int wait_before_reconnect = 0;
 
-	upsdebugx(4, "==================================================");
-	upsdebugx(4, "= device has been disconnected, try to reconnect =");
-	upsdebugx(4, "==================================================");
+	dstate_setinfo("driver.state", "reconnect.trying");
+
+	/* Init time to wait before trying to reconnect (seconds) */
+	val = getval(HU_VAR_WAITBEFORERECONNECT);
+	if (val) {
+		wait_before_reconnect = atoi(val);
+	}
 
 	/* Try to close the previous handle */
-	if (udev)
-		comm_driver->close(udev);
+	if (udev == HID_DEV_HANDLE_CLOSED) {
+		upsdebugx(4, "Not closing comm_driver previous handle: already closed");
+	} else {
+		upsdebugx(4, "Closing comm_driver previous handle");
+		comm_driver->close_dev(udev);
+		udev = HID_DEV_HANDLE_CLOSED;
+	}
 
-	ret = comm_driver->open(&udev, &curDevice, subdriver_matcher, NULL);
+	upsdebugx(4, "===================================================================");
+	if (wait_before_reconnect > 0 ) {
+		upsdebugx(4, " device has been disconnected, trying to reconnect in %i seconds", wait_before_reconnect);
+		sleep(wait_before_reconnect);
+		upsdebugx(4, " trying to reconnect");
+	} else {
+		upsdebugx(4, " device has been disconnected, try to reconnect");
+	}
+	upsdebugx(4, "===================================================================");
 
+	upsdebugx(4, "Opening comm_driver ...");
+	ret = comm_driver->open_dev(&udev, &curDevice, subdriver_matcher, NULL);
+	upsdebugx(4, "Opening comm_driver returns ret=%i", ret);
 	if (ret > 0) {
 		return 1;
 	}
 
+	dstate_setinfo("driver.state", "quiet");
 	return 0;
 }
 
@@ -1642,28 +1870,47 @@ static void ups_status_set(void)
 		dstate_delinfo("input.transfer.reason");
 	}
 
+	/* Report calibration mode first, because it looks like OFF or OB
+	 * for many implementations (literally, it is a temporary OB state
+	 * managed by the UPS hardware to become OL later... if it guesses
+	 * correctly when to do so), and may cause false alarms for us to
+	 * raise FSD urgently. So we first let upsmon know it is just a drill.
+	 */
+	if (ups_status & STATUS(CALIB)) {
+		status_set("CAL");		/* calibration */
+	}
 
 	if (!(ups_status & STATUS(ONLINE))) {
 		status_set("OB");		/* on battery */
 	} else if ((ups_status & STATUS(DISCHRG))) {
-			/* if online */
+		/* if online but discharging */
+		if (onlinedischarge_calibration) {
+			/* if we treat OL+DISCHRG as calibrating */
+			status_set("CAL");	/* calibration */
+		}
+
 		if (onlinedischarge) {
 			/* if we treat OL+DISCHRG as being offline */
 			status_set("OB");	/* on battery */
-		} else {
-			if (!(ups_status & STATUS(CAL))) {
+		}
+
+		if (!onlinedischarge && !onlinedischarge_calibration) {
+			if (!(ups_status & STATUS(CALIB))) {
 				/* if in OL+DISCHRG unknowingly, warn user */
 				upslogx(LOG_WARNING, "%s: seems that UPS [%s] is in OL+DISCHRG state now. "
-				"Is it calibrating or do you perhaps want to set 'onlinedischarge' option? "
-				"Some UPS models (e.g. CyberPower UT series) emit OL+DISCHRG when offline.",
+				"Is it calibrating (perhaps you want to set 'onlinedischarge_calibration' option)? "
+				"Note that some UPS models (e.g. CyberPower UT series) emit OL+DISCHRG when "
+				"in fact offline/on-battery (perhaps you want to set 'onlinedischarge' option).",
 				__func__, upsname);
 			}
 			/* if we're calibrating */
 			status_set("OL");	/* on line */
 		}
 	} else if ((ups_status & STATUS(ONLINE))) {
+		/* if simply online */
 		status_set("OL");
 	}
+
 	if ((ups_status & STATUS(DISCHRG)) &&
 		!(ups_status & STATUS(DEPLETED))) {
 		status_set("DISCHRG");		/* discharging */
@@ -1693,9 +1940,6 @@ static void ups_status_set(void)
 	if (ups_status & STATUS(OFF)) {
 		status_set("OFF");		/* ups is off */
 	}
-	if (ups_status & STATUS(CAL)) {
-		status_set("CAL");		/* calibration */
-	}
 }
 
 /* find info element definition in info array
