@@ -24,27 +24,28 @@
 
 #include "xppc-mib.h"
 
-#define XPPC_MIB_VERSION  "0.2"
+#define XPPC_MIB_VERSION  "0.4"
 
 #define XPPC_SYSOID       ".1.3.6.1.4.1.935"
 
 /* To create a value lookup structure (as needed on the 2nd line of the example
  * below), use the following kind of declaration, outside of the present snmp_info_t[]:
- * static info_lkp_t onbatt_info[] = {
- * 	{ 1, "OB" },
- * 	{ 2, "OL" },
- * 	{ 0, NULL }
+ * static info_lkp_t xpcc_onbatt_info[] = {
+ * 	{ 1, "OB", NULL, NULL },
+ * 	{ 2, "OL", NULL, NULL },
+ * 	{ 0, NULL, NULL, NULL }
  * };
  */
 
 /* upsBaseBatteryStatus */
 static info_lkp_t xpcc_onbatt_info[] = {
-	{ 1, "" },	/* unknown */
-	{ 2, "" },	/* batteryNormal */
-	{ 3, "LB" }	/* batteryLow */
+	{ 1, "", NULL, NULL },	/* unknown */
+	{ 2, "", NULL, NULL },	/* batteryNormal */
+	{ 3, "LB", NULL, NULL },	/* batteryLow */
+	{ 0, NULL, NULL, NULL }
 };
 
-/* 
+/*
 upsBaseOutputStatus OBJECT-TYPE
 	SYNTAX INTEGER {
 			unknown(1),
@@ -58,15 +59,16 @@ upsBaseOutputStatus OBJECT-TYPE
 			onBuck(9) }
 */
 static info_lkp_t xpcc_power_info[] = {
-	{ 1, "" },	/* unknown */
-	{ 2, "OL" },	/* onLine */
-	{ 3, "OB" },	/* onBattery */
-	{ 4, "OL BOOST" },	/* onBoost */
-	{ 5, "OFF" },	/* sleeping */
-	{ 6, "BYPASS"},	/* onBypass */
-	{ 7, "" },	/* rebooting */
-	{ 8, "OFF" },	/* standBy */
-	{ 9, "OL TRIM"}	/* onBuck */
+	{ 1, "", NULL, NULL },	/* unknown */
+	{ 2, "OL", NULL, NULL },	/* onLine */
+	{ 3, "OB", NULL, NULL },	/* onBattery */
+	{ 4, "OL BOOST", NULL, NULL },	/* onBoost */
+	{ 5, "OFF", NULL, NULL },	/* sleeping */
+	{ 6, "BYPASS", NULL, NULL },	/* onBypass */
+	{ 7, "", NULL, NULL },	/* rebooting */
+	{ 8, "OFF", NULL, NULL },	/* standBy */
+	{ 9, "OL TRIM", NULL, NULL },	/* onBuck */
+	{ 0, NULL, NULL, NULL }
 };
 
 /* XPPC Snmp2NUT lookup table */
@@ -83,22 +85,27 @@ static snmp_info_t xppc_mib[] = {
 	 *	dfl: default value
 	 *	flags: snmp-ups internal flags (FIXME: ...)
 	 *	oid2info: lookup table between OID and NUT values
-	 *	setvar: variable to set for SU_FLAG_SETINT
 	 *
 	 * Example:
 	 * { "input.voltage", 0, 0.1, ".1.3.6.1.4.1.705.1.6.2.1.2.1", "", SU_INPUT_1, NULL },
-	 * { "ups.status", ST_FLAG_STRING, SU_INFOSIZE, ".1.3.6.1.4.1.705.1.7.3.0", "", SU_FLAG_OK | SU_STATUS_BATT, onbatt_info },
+	 * { "ups.status", ST_FLAG_STRING, SU_INFOSIZE, ".1.3.6.1.4.1.705.1.7.3.0", "", SU_FLAG_OK | SU_STATUS_BATT, xpcc_onbatt_info },
 	 *
 	 * To create a value lookup structure (as needed on the 2nd line), use the
 	 * following kind of declaration, outside of the present snmp_info_t[]:
-	 * static info_lkp_t onbatt_info[] = {
+	 * static info_lkp_t xpcc_onbatt_info[] = {
 	 * 	{ 1, "OB" },
 	 * 	{ 2, "OL" },
 	 * 	{ 0, NULL }
 	 * };
 	 */
-        { "ups.mfr", ST_FLAG_STRING, SU_INFOSIZE, NULL, "Tripp Lite / Phoenixtec",
-                SU_FLAG_STATIC | SU_FLAG_ABSENT | SU_FLAG_OK, NULL },
+
+	/* standard MIB items */
+	{ "device.description", ST_FLAG_STRING | ST_FLAG_RW, SU_INFOSIZE, ".1.3.6.1.2.1.1.1.0", NULL, SU_FLAG_OK, NULL },
+	{ "device.contact", ST_FLAG_STRING | ST_FLAG_RW, SU_INFOSIZE, ".1.3.6.1.2.1.1.4.0", NULL, SU_FLAG_OK, NULL },
+	{ "device.location", ST_FLAG_STRING | ST_FLAG_RW, SU_INFOSIZE, ".1.3.6.1.2.1.1.6.0", NULL, SU_FLAG_OK, NULL },
+
+	{ "ups.mfr", ST_FLAG_STRING, SU_INFOSIZE, NULL, "Tripp Lite / Phoenixtec",
+		SU_FLAG_STATIC | SU_FLAG_ABSENT | SU_FLAG_OK, NULL },
 
 	/* upsBaseIdentModel.0 = STRING: "Intelligent" */
 	{ "ups.model", ST_FLAG_STRING, SU_INFOSIZE, ".1.3.6.1.4.1.935.1.1.1.1.1.1.0", "Generic Phoenixtec SNMP device", SU_FLAG_OK, NULL },
@@ -123,4 +130,4 @@ static snmp_info_t xppc_mib[] = {
 	{ NULL, 0, 0, NULL, NULL, 0, NULL }
 };
 
-mib2nut_info_t	xppc = { "xppc", XPPC_MIB_VERSION, NULL, NULL, xppc_mib, XPPC_SYSOID };
+mib2nut_info_t	xppc = { "xppc", XPPC_MIB_VERSION, NULL, NULL, xppc_mib, XPPC_SYSOID, NULL };
