@@ -9,6 +9,7 @@
 #       is to just pick out some strings relevant for tracking config changes.
 #
 # Copyright (C) 2016-2020 Eaton
+# Copyright (C) 2020-2023 Jim Klimov <jimklimov+nut@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -76,7 +77,7 @@
 
 # Directory where NUT configs are located, e.g. /etc/nut or /etc/ups
 # Set at package configuration, compiled into daemons and drivers
-prefix="/usr/local"
+prefix="/usr/local/ups"
 [ -n "${NUT_CONFPATH-}" ] || NUT_CONFPATH="${prefix}/etc"
 # Technically this should be a distribution-dependent value configured
 # during package build. But everyone has it the same from systemd defaults:
@@ -92,7 +93,7 @@ if [ -n "$ZSH_VERSION" ]; then
 #    IFS="`printf ' \t\r\n'`" ; export IFS
 fi
 
-if set | egrep '^(shell|version|t?csh)' | egrep 't?csh' >/dev/null ; then
+if set | grep -E '^(shell|version|t?csh)' | grep -E 't?csh' >/dev/null ; then
     echo "FATAL: csh or tcsh is not supported in this script" >&2
     exit 1
 fi
@@ -170,7 +171,7 @@ fi
 # Cache needed bits of ups.conf to speed up later parsing. Note that these
 # data are needed for most operations, and populated by upslist_readFile()
 UPSCONF_DATA=""
-# Subset of normalized data above that only has sections, drivers and ports
+# Subset of normalized data above that only has sections, drivers and ports (SDP)
 UPSCONF_DATA_SDP=""
 
 # List of configured UPSes in the config-file
@@ -442,7 +443,7 @@ upsconf_getValue() {
         RES_L=0
         VALUE=""
 
-        LINE="`echo "$SECTION_CONTENT" | egrep '(^'"$1"'=|^'"$1"'$)'`" \
+        LINE="`echo "$SECTION_CONTENT" | grep -E '(^'"$1"'=|^'"$1"'$)'`" \
         && VALUE="$(echo "$LINE" | sed -e "s,^$1=,," -e 's,^\"\(.*\)\"$,\1,' -e "s,^'\(.*\)'$,\1,")" \
         || RES_L=$?
 
@@ -478,8 +479,45 @@ upsconf_getDriverMedia() {
     # particular system's physics, both serial and network media may need USB).
     CURR_DRV="`upsconf_getDriver "$1"`" || return $?
     case "$CURR_DRV" in
-        *netxml*|*snmp*|*ipmi*|*powerman*|*-mib*|*avahi*|*apcupsd*)
+        *netxml*|*snmp*|*ipmi*|*powerman*|*-mib*|*avahi*)
             printf '%s\n%s\n' "$CURR_DRV" "network" ; return ;;
+        *apcupsd-ups*)
+            # Relay from a nearby apcupsd network server into NUT ecosystem:
+            CURR_PORT="`upsconf_getPort "$1"`" || CURR_PORT=""
+            case "$CURR_PORT" in
+                *localhost*|*127.0.0.1*|*::1*)
+                    printf '%s\n%s\n' "$CURR_DRV" "network-localhost" ; return ;;
+                *)
+                    printf '%s\n%s\n' "$CURR_DRV" "network" ; return ;;
+            esac
+            ;;
+        *apc_modbus*)
+            CURR_PORT="`upsconf_getPort "$1"`" || CURR_PORT=""
+            CURR_PORTTYPE="`upsconf_getValue "$1" 'porttype'`" || CURR_PORTTYPE=""
+            case "$CURR_PORTTYPE" in
+                *usb*)
+                    printf '%s\n%s\n' "$CURR_DRV" "usb" ; return ;;
+                *serial*)
+                    printf '%s\n%s\n' "$CURR_DRV" "serial" ; return ;;
+                *)  # default depends on driver build (against libmodbus
+                    # version with or without support for usb)
+                    # TOTHINK: Check for presence of config values like
+                    # vendorid (USB) or baud (Serial)? They are optional
+                    # with reasonable defaults anyway...
+                    case "$CURR_PORT" in
+                        *auto*)
+                            printf '%s\n%s\n' "$CURR_DRV" "usb" ; return ;;
+                        /*)
+                            printf '%s\n%s\n' "$CURR_DRV" "serial" ; return ;;
+                        *localhost*|*127.0.0.1*|*::1*)
+                            printf '%s\n%s\n' "$CURR_DRV" "network-localhost" ; return ;;
+                        *)
+                            printf '%s\n%s\n' "$CURR_DRV" "network" ; return ;;
+                    esac
+                    # returns are above, but just in case - have a fallback:
+                    printf '%s\n%s\n' "$CURR_DRV" "" ; return ;;
+            esac
+            ;;
         *usb*)
             printf '%s\n%s\n' "$CURR_DRV" "usb" ; return ;;
         nutdrv_qx) # May be direct serial or USB
@@ -489,7 +527,7 @@ upsconf_getDriverMedia() {
                     printf '%s\n%s\n' "$CURR_DRV" "usb" ; return ;;
                 /dev/*)
                     # See drivers/nutdrv_qx.c :: upsdrv_initups() for a list
-                    if [ -n "`upsconf_getValue "$1" 'subdriver' 'vendorid' 'productid' 'vendor' 'product' 'serial' 'bus' 'langid_fix'`" ] \
+                    if [ -n "`upsconf_getValue "$1" 'subdriver' 'vendorid' 'productid' 'vendor' 'product' 'serial' 'bus' 'busport' 'langid_fix'`" ] \
                     ; then
                         printf '%s\n%s\n' "$CURR_DRV" "usb" ; return
                     else
@@ -625,7 +663,7 @@ smf_refreshSupervizor() {
 }
 smf_listInstances_raw() {
     # Newer versions have pattern matching; older SMF might not have this luxury
-    /usr/bin/svcs -a -H -o fmri | egrep '/nut-driver:'
+    /usr/bin/svcs -a -H -o fmri | grep -E '/nut-driver:'
 }
 smf_listInstances() {
     smf_listInstances_raw | sed 's/^.*://' | sort -n
@@ -666,14 +704,14 @@ smf_setSavedMD5() {
     /usr/sbin/svcadm refresh "${TARGET_FMRI}" || return
 }
 smf_restart_upsd() {
-    echo "Restarting NUT data server to make sure it knows new configuration..."
+    echo "Reloading or restarting NUT data server to make sure it knows new configuration..."
     /usr/sbin/svcadm enable "nut-server" 2>/dev/null
     /usr/sbin/svcadm clear "nut-server" 2>/dev/null
     /usr/sbin/svcadm refresh "nut-server" || \
     /usr/sbin/svcadm restart "nut-server"
 }
 smf_restart_drv() {
-    echo "Restarting NUT driver instance '$1' to make sure it knows new configuration..."
+    echo "Reloading or restarting NUT driver instance '$1' to make sure it knows new configuration..."
     /usr/sbin/svcadm enable "nut-driver:$1" 2>/dev/null
     /usr/sbin/svcadm clear "nut-driver:$1" 2>/dev/null
     /usr/sbin/svcadm refresh "nut-driver:$1" || \
@@ -756,7 +794,7 @@ systemd_refreshSupervizor() {
     /bin/systemctl daemon-reload
 }
 systemd_listInstances_raw() {
-    /bin/systemctl show 'nut-driver@*' -p Id | egrep '=nut-driver' | sed 's,^Id=,,'
+    /bin/systemctl show 'nut-driver@*' -p Id | grep -E '=nut-driver' | sed 's,^Id=,,'
 }
 systemd_listInstances() {
     systemd_listInstances_raw | sed -e 's/^.*@//' -e 's/\.service$//' | sort -n
@@ -788,7 +826,7 @@ systemd_restart_upsd() {
         *) return 0 ;;
     esac
 
-    echo "Restarting NUT data server to make sure it knows new configuration..."
+    echo "Reloading or restarting NUT data server to make sure it knows new configuration..."
     # Note: reload is a better way to go about this, so the
     # data service is not interrupted by re-initialization
     # of the daemon. But systemd/systemctl sometimes stalls...
@@ -804,9 +842,12 @@ systemd_restart_drv() {
         *) return 0 ;;
     esac
 
-    echo "Restarting NUT driver instance '$1' to make sure it knows new configuration..."
+    echo "Reloading or restarting NUT driver instance '$1' to make sure it knows new configuration..."
 
     # Full restart, e.g. in case we changed the user= in configs
+    # however let "reload" try, in case we changed something that
+    # can be updated "on the fly", like the "debug_min" setting.
+    $TIMEOUT_CMD $TIMEOUT_ARGS /bin/systemctl reload-or-restart "nut-driver@$1" || \
     $TIMEOUT_CMD $TIMEOUT_ARGS /bin/systemctl restart "nut-driver@$1"
 }
 
@@ -824,15 +865,15 @@ upslist_normalizeFile_filter() {
     # are dropped. Note that brackets with spaces inside, and brackets
     # that do not start the non-whitespace payload of the line, are not
     # sections.
-    egrep -v '(^$|^#)' | \
+    grep -E -v '(^$|^#)' | \
     sed -e 's,^['"$TABCHAR"'\ ]*,,' \
         -e 's,^\#.*$,,' \
         -e 's,['"$TABCHAR"'\ ]*$,,' \
         -e 's,^\([^=\ '"$TABCHAR"']*\)['"$TABCHAR"'\ ]*=['"$TABCHAR"'\ ]*,\1=,g' \
         -e 's,=\"\([^\ '"$TABCHAR"']*\)\"$,=\1,' \
         -e 's,^\(\[[^]'"$TABCHAR"'\ ]*\]\)['"$TABCHAR"'\ ]*\(#.*\)*$,\1,' \
-    | egrep -v '^$' \
-    | egrep '([\[\=]|^[^ '"$TABCHAR"']*$|^[^ '"$TABCHAR"']*[ '"$TABCHAR"']*\#.*$)'
+    | grep -E -v '^$' \
+    | grep -E '([\[\=]|^[^ '"$TABCHAR"']*$|^[^ '"$TABCHAR"']*[ '"$TABCHAR"']*#.*$)'
 }
 
 upslist_normalizeFile() {
@@ -859,7 +900,7 @@ upslist_normalizeFile() {
     # for faster parsing when determining driver-required media etc.
     UPSCONF_DATA="$(upslist_normalizeFile_filter < "$UPSCONF")" \
         && [ -n "$UPSCONF_DATA" ] \
-        && UPSCONF_DATA_SDP="`egrep '^(\[.*\]|driver=|port=)' << EOF
+        && UPSCONF_DATA_SDP="`grep -E '^(\[.*\]|driver=|port=)' << EOF
 $UPSCONF_DATA
 EOF`" \
         ||  { echo "Error reading the '$UPSCONF' file or it does not declare any device configurations: nothing left after normalization" >&2
@@ -890,7 +931,7 @@ upslist_readFile() {
 
     if [ -n "$UPSCONF_DATA" ] ; then
         # Note that section-name brackets should contain a single token
-        UPSLIST_FILE="$(echo "$UPSCONF_DATA_SDP" | egrep '^\[[^'"$TABCHAR"'\ ]*\]$' | sed 's,^\[\(.*\)\]$,\1,' | sort -n)" \
+        UPSLIST_FILE="$(echo "$UPSCONF_DATA_SDP" | grep -E '^\[[^'"$TABCHAR"'\ ]*\]$' | sed 's,^\[\(.*\)\]$,\1,' | sort -n)" \
             || UPSLIST_FILE=""
         if [ -z "$UPSLIST_FILE" ] ; then
             echo "Error reading the '$UPSCONF' file or it does not declare any device configurations: no section declarations in parsed normalized contents" >&2
@@ -986,8 +1027,21 @@ nut_driver_enumerator_main() {
 
     if [ -n "$NEW_CHECKSUM" ]; then
         for UPSS in $NEW_CHECKSUM ; do
-            echo "Dropping old ${SERVICE_FRAMEWORK} service instance ${UPSS} whose section in config file has changed..." >&2
-            $hook_unregisterInstance "$UPSS"
+            CURR_DRV="`upsconf_getDriver "$UPSS"`" || CURR_DRV=""
+            DO_UNREGISTER=yes
+            if [ -n "$CURR_DRV" ] ; then
+                # If reload is handled and does not complain,
+                # we are OK to proceed without re-defining
+                # and restarting the driver service instance.
+                "/usr/local/ups/bin/$CURR_DRV" -a "$UPSS" -c reload-or-error >/dev/null 2>/dev/null \
+                && DO_UNREGISTER=no
+            fi
+            if [ "$DO_UNREGISTER" = yes ] ; then
+                echo "Dropping old ${SERVICE_FRAMEWORK} service instance ${UPSS} whose section in config file has changed..." >&2
+                $hook_unregisterInstance "$UPSS"
+            else
+                echo "Keeping ${SERVICE_FRAMEWORK} service instance ${UPSS} whose section in config file has changed: live reload sufficed" >&2
+            fi
         done
         upslist_readSvcs "after updating for new config section checksums"
     fi
@@ -1298,7 +1352,7 @@ while [ $# -gt 0 ]; do
             case "$SVC" in
                 MD5_*) ;; # fall through to the bulk of code
                 *)  upslist_readFile_once || exit $?
-                    echo "$UPSLIST_FILE" | egrep "^$SVC\$"
+                    echo "$UPSLIST_FILE" | grep -E "^$SVC\$"
                     exit $?
                     ;;
             esac
