@@ -36,6 +36,10 @@ int	num_ups = 0;
  * than that would not have effect, can only have more).
  */
 int	nut_debug_level_global = -1;
+/* Debug level specified via command line - we revert to
+ * it when reloading if there was no DEBUG_MIN in upsd.conf
+ */
+int nut_debug_level_args = 0;
 
 /* add another UPS for monitoring from ups.conf */
 static void ups_create(const char *fn, const char *name, const char *desc)
@@ -60,6 +64,19 @@ static void ups_create(const char *fn, const char *name, const char *desc)
 
 	temp->stale = 1;
 	temp->retain = 1;
+#ifdef WIN32
+	memset(&temp->read_overlapped,0,sizeof(temp->read_overlapped));
+	memset(temp->buf,0,sizeof(temp->buf));
+	temp->read_overlapped.hEvent = CreateEvent(NULL, /* Security */
+						FALSE, /* auto-reset*/
+						FALSE, /* initial state = non signaled */
+						NULL /* no name */);
+	if(temp->read_overlapped.hEvent == NULL ) {
+		upslogx(LOG_ERR, "Can't create event for UPS [%s]",
+			name);
+		return;
+	}
+#endif
 	temp->sock_fd = sstate_connect(temp);
 
 	/* preload this to the current time to avoid false staleness */
@@ -100,8 +117,12 @@ static void ups_update(const char *fn, const char *name, const char *desc)
 		sstate_cmdfree(temp);
 		pconf_finish(&temp->sock_ctx);
 
+#ifndef WIN32
 		close(temp->sock_fd);
-		temp->sock_fd = -1;
+#else
+		CloseHandle(temp->sock_fd);
+#endif
+		temp->sock_fd = ERROR_FD;
 		temp->dumpdone = 0;
 
 		/* now redefine the filename and wrap up */
@@ -209,8 +230,16 @@ static int parse_upsd_conf_args(size_t numargs, char **arg)
 
 	/* STATEPATH <dir> */
 	if (!strcmp(arg[0], "STATEPATH")) {
+		const char *sp = getenv("NUT_STATEPATH");
+		if (sp && strcmp(sp, arg[1])) {
+			/* Only warn if the two strings are not equal */
+			upslogx(LOG_WARNING,
+				"Ignoring STATEPATH='%s' from configuration file, "
+				"in favor of NUT_STATEPATH='%s' environment variable",
+				NUT_STRARG(arg[1]), NUT_STRARG(sp));
+		}
 		free(statepath);
-		statepath = xstrdup(arg[1]);
+		statepath = xstrdup(sp ? sp : arg[1]);
 		return 1;
 	}
 
@@ -373,6 +402,13 @@ void load_upsdconf(int reloading)
 				"Applying debug_min=%d from upsd.conf",
 				nut_debug_level_global);
 			nut_debug_level = nut_debug_level_global;
+		} else {
+			/* DEBUG_MIN is absent or commented-away in ups.conf */
+			upslogx(LOG_INFO,
+				"Applying debug level %d from "
+				"original command line arguments",
+				nut_debug_level_args);
+			nut_debug_level = nut_debug_level_args;
 		}
 	}
 
@@ -491,8 +527,12 @@ static void delete_ups(upstype_t *target)
 			else
 				last->next = ptr->next;
 
-			if (ptr->sock_fd != -1)
+			if (VALID_FD(ptr->sock_fd))
+#ifndef WIN32
 				close(ptr->sock_fd);
+#else
+				CloseHandle(ptr->sock_fd);
+#endif
 
 			/* release memory */
 			sstate_infofree(ptr);
@@ -553,7 +593,7 @@ void conf_reload(void)
 	}
 
 	/* reload from ups.conf */
-	read_upsconf();
+	read_upsconf(1);		/* 1 = may abort upon fundamental errors */
 	upsconf_add(1);			/* 1 = reloading */
 
 	/* now reread upsd.conf */
