@@ -21,7 +21,11 @@
 #ifndef NUT_STYPE_H_SEEN
 #define NUT_STYPE_H_SEEN 1
 
+#include "common.h"
+
+#ifndef WIN32
 #include <netdb.h>
+#endif
 
 #ifndef NI_MAXHOST
 #define NI_MAXHOST      1025
@@ -40,7 +44,10 @@ extern "C" {
 typedef struct stype_s {
 	char	*addr;
 	char	*port;
-	int	sock_fd;
+	TYPE_FD_SOCK	sock_fd;
+#ifdef WIN32
+	HANDLE  Event;
+#endif
 	struct stype_s	*next;
 } stype_t;
 
