@@ -30,15 +30,28 @@
 #include "netcmds.h"
 #include "upsconf.h"
 
-#include <sys/un.h>
-#include <sys/socket.h>
-#include <netdb.h>
+#ifndef WIN32
+# include <sys/un.h>
+# include <sys/socket.h>
+# include <netdb.h>
 
-#ifdef HAVE_SYS_SIGNAL_H
-#include <sys/signal.h>
-#endif
-#ifdef HAVE_SIGNAL_H
-#include <signal.h>
+# ifdef HAVE_SYS_SIGNAL_H
+#  include <sys/signal.h>
+# endif
+# ifdef HAVE_SIGNAL_H
+#  include <signal.h>
+/* #include <poll.h> */
+# endif
+#else
+/* Those 2 files for support of getaddrinfo, getnameinfo and freeaddrinfo
+   on Windows 2000 and older versions */
+# include <ws2tcpip.h>
+# include <wspiapi.h>
+/* This override network system calls to adapt to Windows specificity */
+# define W32_NETWORK_CALL_OVERRIDE
+# include "wincompat.h"
+# undef W32_NETWORK_CALL_OVERRIDE
+# include <getopt.h>
 #endif
 
 #include "user.h"
@@ -78,7 +91,7 @@ nfds_t	maxconn = 0;
 /* preloaded to STATEPATH in main, can be overridden via upsd.conf */
 char	*statepath = NULL;
 
-/* preloaded to DATADIR in main, can be overridden via upsd.conf */
+/* preloaded to NUT_DATADIR in main(), can be overridden via upsd.conf */
 char	*datapath = NULL;
 
 /* everything else */
@@ -96,6 +109,10 @@ typedef enum {
 	DRIVER = 1,
 	CLIENT,
 	SERVER
+#ifdef WIN32
+	,NAMED_PIPE
+#endif
+
 } handler_type_t;
 
 typedef struct {
@@ -103,7 +120,6 @@ typedef struct {
 	void		*data;
 } handler_t;
 
-
 /* Commands and settings status tracking */
 
 /* general enable/disable status info for commands and settings
@@ -124,9 +140,13 @@ typedef struct tracking_s {
 
 static tracking_t	*tracking_list = NULL;
 
-
+#ifndef WIN32
 	/* pollfd  */
 static struct pollfd	*fds = NULL;
+#else
+static HANDLE		*fds = NULL;
+static HANDLE		mutex = INVALID_HANDLE_VALUE;
+#endif
 static handler_t	*handler = NULL;
 
 	/* pid file */
@@ -139,6 +159,9 @@ static int	reload_flag = 0, exit_flag = 0;
 /* Ref: RFC 4122 https://tools.ietf.org/html/rfc4122#section-4.1.2 */
 #define UUID4_BYTESIZE 16
 
+#ifdef HAVE_SYSTEMD
+# define SERVICE_UNIT_NAME "nut-server.service"
+#endif
 
 static const char *inet_ntopW (struct sockaddr_storage *s)
 {
@@ -213,22 +236,191 @@ void listen_add(const char *addr, const char *port)
 	server = xcalloc(1, sizeof(*server));
 	server->addr = xstrdup(addr);
 	server->port = xstrdup(port);
-	server->sock_fd = -1;
-	server->next = firstaddr;
+	server->sock_fd = ERROR_FD_SOCK;
+	server->next = NULL;
 
-	firstaddr = server;
+	if (firstaddr) {
+		stype_t	*tmp;
+		for (tmp = firstaddr; tmp->next; tmp = tmp->next);
+		tmp->next = server;
+	} else {
+		firstaddr = server;
+	}
 
 	upsdebugx(3, "listen_add: added %s:%s", server->addr, server->port);
 }
 
+/* Close the connection if needed and free the allocated memory.
+ * WARNING: it is up to the caller to rewrite the "next" pointer
+ * in whoever points to this server instance (if needed)! */
+static void stype_free(stype_t *server)
+{
+	if (VALID_FD_SOCK(server->sock_fd)) {
+		close(server->sock_fd);
+	}
+
+	free(server->addr);
+	free(server->port);
+	free(server);
+}
+
 /* create a listening socket for tcp connections */
 static void setuptcp(stype_t *server)
 {
+#ifdef WIN32
+	WSADATA WSAdata;
+	WSAStartup(2,&WSAdata);
+	atexit((void(*)(void))WSACleanup);
+#endif
 	struct addrinfo		hints, *res, *ai;
 	int	v = 0, one = 1;
 
+	if (VALID_FD_SOCK(server->sock_fd)) {
+		/* Alredy bound, e.g. thanks to 'LISTEN *' handling and injection
+		 * into the list we loop over */
+		upsdebugx(6, "setuptcp: SKIP bind to %s port %s: entry already initialized",
+			server->addr, server->port);
+		return;
+	}
+
 	upsdebugx(3, "setuptcp: try to bind to %s port %s", server->addr, server->port);
 
+	/* Special handling note for `LISTEN * <port>` directive with the
+	 * literal asterisk on systems with RFC-3493 (no relation!) support
+	 * for "IPv4-mapped addresses": it is possible (and technically
+	 * suffices) to LISTEN on "::" (aka "::0" or "0:0:0:0:0:0:0:0") and
+	 * also get an IPv4 any-address listener automatically. More so,
+	 * they would conflict and listening on one such socket precludes
+	 * listening on the other. On other systems (or with disabled
+	 * mapping so IPv6 really means "IPv6 only") we need both sockets.
+	 * NUT asks the system for "IPv6 only" mode when listening on any
+	 * sort of IPv6 addresses; it is however up to the system to implement
+	 * that ability and comply with our request.
+	 * Here we jump through some hoops:
+	 * * Try to get IPv6 any-address (unless constrained by CLI to IPv4);
+	 * * Try to get IPv4 any-address (unless constrained by CLI to IPv6),
+	 *   log information for the sysadmin that it might conflict with the
+	 *   IPv6 listener (IFF we have just opened one);
+	 * * Remember the one or two linked-list entries used, to release later.
+	 */
+	if (!strcmp(server->addr, "*")) {
+		stype_t	*serverAnyV4 = NULL, *serverAnyV6 = NULL;
+		int	canhaveAnyV4 = 0, canhaveAnyV6 = 0;
+
+		/* Note: default opt_af==AF_UNSPEC so not constrained to only one protocol */
+		if (opt_af != AF_INET6) {
+			/* Not constrained to IPv6 */
+			upsdebugx(1, "%s: handling 'LISTEN * %s' with IPv4 any-address support",
+				__func__, server->port);
+			serverAnyV4 = xcalloc(1, sizeof(*serverAnyV4));
+			serverAnyV4->addr = xstrdup("0.0.0.0");
+			serverAnyV4->port = xstrdup(server->port);
+			serverAnyV4->sock_fd = ERROR_FD_SOCK;
+			serverAnyV4->next = NULL;
+		}
+
+		if (opt_af != AF_INET) {
+			/* Not constrained to IPv4 */
+			upsdebugx(1, "%s: handling 'LISTEN * %s' with IPv6 any-address support",
+				__func__, server->port);
+			serverAnyV6 = xcalloc(1, sizeof(*serverAnyV6));
+			serverAnyV6->addr = xstrdup("::0");
+			serverAnyV6->port = xstrdup(server->port);
+			serverAnyV6->sock_fd = ERROR_FD_SOCK;
+			serverAnyV6->next = NULL;
+		}
+
+		if (serverAnyV6) {
+			setuptcp(serverAnyV6);
+			if (VALID_FD_SOCK(serverAnyV6->sock_fd)) {
+				canhaveAnyV6 = 1;
+			} else {
+				upsdebugx(3,
+					"%s: Could not bind to %s:%s trying to handle a 'LISTEN *' directive",
+					__func__, serverAnyV6->addr, serverAnyV6->port);
+			}
+		}
+
+		if (serverAnyV4) {
+			/* Try to get this listener if we can (no IPv4-mapped
+			 * IPv6 support was in force on this platform or its
+			 * configuration in some way that setsockopt(IPV6_V6ONLY)
+			 * failed to cancel).
+			 */
+			upsdebugx(3, "%s: try taking IPv4 'ANY'%s",
+				__func__,
+				canhaveAnyV6 ? " (if dual-stack IPv6 'ANY' did not grab it)" : "");
+			setuptcp(serverAnyV4);
+			if (VALID_FD_SOCK(serverAnyV4->sock_fd)) {
+				canhaveAnyV4 = 1;
+			} else {
+				upsdebugx(3,
+					"%s: Could not bind to IPv4 %s:%s%s",
+					__func__, serverAnyV4->addr, serverAnyV4->port,
+					canhaveAnyV6 ? (" after trying to bind to IPv6: "
+						"assuming dual-stack support on this "
+						"system could not be disabled") : "");
+			}
+		}
+
+		if (!canhaveAnyV4 && !canhaveAnyV6) {
+			fatalx(EXIT_FAILURE,
+				"Handling of 'LISTEN * %s' directive failed to bind to 'ANY' address",
+				server->port);
+		}
+
+		/* Finalize our findings and reset to normal operation
+		 * Note that at least one of these addresses is usable
+		 * and we keep it (and replace original "server" entry
+		 * keeping its place in the list).
+		 */
+		free(server->addr);
+		free(server->port);
+		if (canhaveAnyV4) {
+			upsdebugx(3, "%s: remembering IPv4 'ANY' instead of 'LISTEN *'", __func__);
+			server->addr = serverAnyV4->addr;
+			server->port = serverAnyV4->port;
+			server->sock_fd = serverAnyV4->sock_fd;
+			/* ...and keep whatever server->next there was */
+
+			/* Free the ghost, all needed info was relocated */
+			free(serverAnyV4);
+		} else {
+			if (serverAnyV4) {
+				/* Free any contents there were too */
+				stype_free(serverAnyV4);
+			}
+		}
+		serverAnyV4 = NULL;
+
+		if (canhaveAnyV6) {
+			if (canhaveAnyV4) {
+				/* "server" already populated by excerpts from V4, attach to it */
+				upsdebugx(3, "%s: also remembering IPv6 'ANY' instead of 'LISTEN *'", __func__);
+				serverAnyV6->next = server->next;
+				server->next = serverAnyV6;
+			} else {
+				/* Only retain V6 info */
+				upsdebugx(3, "%s: remembering IPv6 'ANY' instead of 'LISTEN *'", __func__);
+				server->addr = serverAnyV6->addr;
+				server->port = serverAnyV6->port;
+				server->sock_fd = serverAnyV6->sock_fd;
+				/* ...and keep whatever server->next there was */
+
+				/* Free the ghost, all needed info was relocated */
+				free(serverAnyV6);
+			}
+		} else {
+			if (serverAnyV6) {
+				/* Free any contents there were too */
+				stype_free(serverAnyV6);
+			}
+		}
+		serverAnyV6 = NULL;
+
+		return;
+	}
+
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_flags		= AI_PASSIVE;
 	hints.ai_family		= opt_af;
@@ -244,9 +436,9 @@ static void setuptcp(stype_t *server)
 	}
 
 	for (ai = res; ai; ai = ai->ai_next) {
-		int sock_fd;
+		TYPE_FD_SOCK sock_fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 
-		if ((sock_fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0) {
+		if (INVALID_FD_SOCK(sock_fd)) {
 			upsdebug_with_errno(3, "setuptcp: socket");
 			continue;
 		}
@@ -255,12 +447,26 @@ static void setuptcp(stype_t *server)
 			fatal_with_errno(EXIT_FAILURE, "setuptcp: setsockopt");
 		}
 
+		/* Ordinarily we request that IPv6 listeners handle only IPv6
+		 * and not IPv4 mapped addresses - if the OS would honour that.
+		 * TOTHINK: Does any platform need `#ifdef IPV6_V6ONLY` given
+		 * that we apparently already have AF_INET6 OS support everywhere?
+		 */
+		if (ai->ai_family == AF_INET6) {
+			if (setsockopt(sock_fd, IPPROTO_IPV6, IPV6_V6ONLY, (void *)&one, sizeof(one)) != 0) {
+				upsdebug_with_errno(3, "setuptcp: setsockopt IPV6_V6ONLY");
+				/* ack, ignore */
+			}
+		}
+
 		if (bind(sock_fd, ai->ai_addr, ai->ai_addrlen) < 0) {
 			upsdebug_with_errno(3, "setuptcp: bind");
 			close(sock_fd);
 			continue;
 		}
 
+/* WSAEventSelect automatically set the socket to nonblocking mode */
+#ifndef WIN32
 		if ((v = fcntl(sock_fd, F_GETFL, 0)) == -1) {
 			fatal_with_errno(EXIT_FAILURE, "setuptcp: fcntl(get)");
 		}
@@ -268,6 +474,7 @@ static void setuptcp(stype_t *server)
 		if (fcntl(sock_fd, F_SETFL, v | O_NDELAY) == -1) {
 			fatal_with_errno(EXIT_FAILURE, "setuptcp: fcntl(set)");
 		}
+#endif
 
 		if (listen(sock_fd, 16) < 0) {
 			upsdebug_with_errno(3, "setuptcp: listen");
@@ -275,15 +482,39 @@ static void setuptcp(stype_t *server)
 			continue;
 		}
 
+		if (ai->ai_next) {
+			char ipaddrbuf[SMALLBUF];
+			const char *ipaddr;
+			snprintf(ipaddrbuf, sizeof(ipaddrbuf), " as ");
+			ipaddr = inet_ntop(ai->ai_family, ai->ai_addr,
+				ipaddrbuf + strlen(ipaddrbuf),
+				sizeof(ipaddrbuf));
+			upslogx(LOG_WARNING,
+				"setuptcp: bound to %s%s but there seem to be "
+				"further (ignored) addresses resolved for this name",
+				server->addr,
+				ipaddr == NULL ? "" : ipaddrbuf);
+		}
+
 		server->sock_fd = sock_fd;
 		break;
 	}
 
+#ifdef WIN32
+		server->Event = CreateEvent(NULL, /* Security */
+				FALSE, /* auto-reset */
+				FALSE, /* initial state */
+				NULL); /* no name */
+
+		/* Associate socket event to the socket via its Event object */
+		WSAEventSelect( server->sock_fd, server->Event, FD_ACCEPT );
+#endif
+
 	freeaddrinfo(res);
 
 	/* leave up to the caller, server_load(), to fail silently if there is
 	 * no other valid LISTEN interface */
-	if (server->sock_fd < 0) {
+	if (INVALID_FD_SOCK(server->sock_fd)) {
 		upslogx(LOG_ERR, "not listening on %s port %s", server->addr, server->port);
 	} else {
 		upslogx(LOG_INFO, "listening on %s port %s", server->addr, server->port);
@@ -323,6 +554,10 @@ static void client_disconnect(nut_ctype_t *client)
 	shutdown(client->sock_fd, 2);
 	close(client->sock_fd);
 
+#ifdef WIN32
+	CloseHandle(client->Event);
+#endif
+
 	if (client->loginups) {
 		declogins(client->loginups);
 	}
@@ -388,7 +623,10 @@ int sendback(nut_ctype_t *client, const char *fmt, ...)
 		res = write(client->sock_fd, ans, len);
 	}
 
-	upsdebugx(2, "write: [destfd=%d] [len=%zu] [%s]", client->sock_fd, len, str_rtrim(ans, '\n'));
+	{ /* scoping */
+		char * s = str_rtrim(ans, '\n');
+		upsdebugx(2, "write: [destfd=%d] [len=%" PRIuSIZE "] [%s]", client->sock_fd, len, s);
+	}
 
 	if (res < 0 || len != (size_t)res) {
 		upslog_with_errno(LOG_NOTICE, "write() failed for %s", client->addr);
@@ -435,7 +673,15 @@ void kick_login_clients(const char *upsname)
 /* make sure a UPS is sane - connected, with fresh data */
 int ups_available(const upstype_t *ups, nut_ctype_t *client)
 {
-	if (ups->sock_fd < 0) {
+	if (!ups) {
+		/* Should never happen, but handle this
+		 * just in case instead of segfaulting */
+		upsdebugx(1, "%s: ERROR, called with a NULL ups pointer", __func__);
+		send_err(client, NUT_ERR_FEATURE_NOT_SUPPORTED);
+		return 0;
+	}
+
+	if (INVALID_FD(ups->sock_fd)) {
 		send_err(client, NUT_ERR_DRIVER_NOT_CONNECTED);
 		return 0;
 	}
@@ -546,6 +792,16 @@ static void client_connect(stype_t *server)
 
 	client->tracking = 0;
 
+#ifdef WIN32
+	client->Event = CreateEvent(NULL, /* Security, */
+				FALSE,    /* auto-reset */
+				FALSE,    /* initial state */
+				NULL);    /* no name */
+
+	/* Associate socket event to the socket via its Event object */
+	WSAEventSelect( client->sock_fd, client->Event, FD_READ );
+#endif
+
 	pconf_init(&client->ctx, NULL);
 
 	if (firstclient) {
@@ -622,13 +878,16 @@ void server_load(void)
 {
 	stype_t	*server;
 
-	/* default behaviour if no LISTEN addres has been specified */
+	/* default behaviour if no LISTEN address has been specified */
 	if (!firstaddr) {
+		/* Note: default opt_af==AF_UNSPEC so not constrained to only one protocol */
 		if (opt_af != AF_INET) {
+			upsdebugx(1, "%s: No LISTEN configuration provided, will try IPv6 localhost", __func__);
 			listen_add("::1", string_const(PORT));
 		}
 
 		if (opt_af != AF_INET6) {
+			upsdebugx(1, "%s: No LISTEN configuration provided, will try IPv4 localhost", __func__);
 			listen_add("127.0.0.1", string_const(PORT));
 		}
 	}
@@ -638,7 +897,7 @@ void server_load(void)
 	}
 
 	/* check if we have at least 1 valid LISTEN interface */
-	if (firstaddr->sock_fd < 0) {
+	if (INVALID_FD_SOCK(firstaddr->sock_fd)) {
 		fatalx(EXIT_FAILURE, "no listening interface available");
 	}
 }
@@ -650,14 +909,7 @@ void server_free(void)
 	/* cleanup server fds */
 	for (server = firstaddr; server; server = snext) {
 		snext = server->next;
-
-		if (server->sock_fd != -1) {
-			close(server->sock_fd);
-		}
-
-		free(server->addr);
-		free(server->port);
-		free(server);
+		stype_free(server);
 	}
 
 	firstaddr = NULL;
@@ -684,8 +936,14 @@ static void driver_free(void)
 
 		unext = ups->next;
 
-		if (ups->sock_fd != -1) {
+		if (VALID_FD(ups->sock_fd)) {
+#ifndef WIN32
 			close(ups->sock_fd);
+#else
+			DisconnectNamedPipe(ups->sock_fd);
+			CloseHandle(ups->sock_fd);
+#endif
+			ups->sock_fd = ERROR_FD;
 		}
 
 		sstate_infofree(ups);
@@ -724,38 +982,51 @@ static void upsd_cleanup(void)
 
 	free(fds);
 	free(handler);
+
+#ifdef WIN32
+	if (mutex != INVALID_HANDLE_VALUE) {
+		ReleaseMutex(mutex);
+		CloseHandle(mutex);
+	}
+#endif
 }
 
 static void poll_reload(void)
 {
+#ifndef WIN32
 	long	ret;
+	size_t	maxalloc;
 
 	ret = sysconf(_SC_OPEN_MAX);
 
 	if ((intmax_t)ret < (intmax_t)maxconn) {
 		fatalx(EXIT_FAILURE,
 			"Your system limits the maximum number of connections to %ld\n"
-			"but you requested %jd. The server won't start until this\n"
+			"but you requested %" PRIdMAX ". The server won't start until this\n"
 			"problem is resolved.\n", ret, (intmax_t)maxconn);
 	}
 
 	if (1 > maxconn) {
 		fatalx(EXIT_FAILURE,
-			"You requested %jd as maximum number of connections.\n"
+			"You requested %" PRIdMAX " as maximum number of connections.\n"
 			"The server won't start until this problem is resolved.\n", (intmax_t)maxconn);
 	}
 
 	/* How many items can we stuff into the array? */
-	size_t maxalloc = SIZE_MAX / sizeof(void *);
+	maxalloc = SIZE_MAX / sizeof(void *);
 	if ((uintmax_t)maxalloc < (uintmax_t)maxconn) {
 		fatalx(EXIT_FAILURE,
-			"You requested %jd as maximum number of connections, but we can only allocate %zu.\n"
+			"You requested %" PRIdMAX " as maximum number of connections, but we can only allocate %" PRIuSIZE ".\n"
 			"The server won't start until this problem is resolved.\n", (intmax_t)maxconn, maxalloc);
 	}
 
 	/* The checks above effectively limit that maxconn is in size_t range */
 	fds = xrealloc(fds, (size_t)maxconn * sizeof(*fds));
 	handler = xrealloc(handler, (size_t)maxconn * sizeof(*handler));
+#else
+	fds = xrealloc(fds, (size_t)MAXIMUM_WAIT_OBJECTS * sizeof(*fds));
+	handler = xrealloc(handler, (size_t)MAXIMUM_WAIT_OBJECTS * sizeof(*handler));
+#endif
 }
 
 /* instant command and setvar status tracking */
@@ -971,38 +1242,66 @@ int nut_uuid_v4(char *uuid_str)
 		nut_uuid[12], nut_uuid[13], nut_uuid[14], nut_uuid[15]);
 }
 
+static void set_exit_flag(int sig)
+{
+	exit_flag = sig;
+}
+
+static void set_reload_flag(int sig)
+{
+	NUT_UNUSED_VARIABLE(sig);
+	reload_flag = 1;
+}
+
 /* service requests and check on new data */
 static void mainloop(void)
 {
+#ifndef WIN32
 	int	ret;
-	nfds_t	i, nfds = 0;
+	nfds_t	i;
+#else
+	DWORD	ret;
+	pipe_conn_t * conn;
+#endif
 
+	nfds_t	nfds = 0;
 	upstype_t	*ups;
 	nut_ctype_t		*client, *cnext;
 	stype_t		*server;
 	time_t	now;
 
+	upsnotify(NOTIFY_STATE_WATCHDOG, NULL);
+
 	time(&now);
 
 	if (reload_flag) {
+		upsnotify(NOTIFY_STATE_RELOADING, NULL);
 		conf_reload();
 		poll_reload();
 		reload_flag = 0;
+		upsnotify(NOTIFY_STATE_READY, NULL);
 	}
 
 	/* cleanup instcmd/setvar status tracking entries if needed */
 	tracking_cleanup();
 
+#ifndef WIN32
 	/* scan through driver sockets */
 	for (ups = firstups; ups && (nfds < maxconn); ups = ups->next) {
 
 		/* see if we need to (re)connect to the socket */
-		if (ups->sock_fd < 0) {
-			upsdebugx(1, "%s: UPS [%s] is not currently connected",
+		if (INVALID_FD(ups->sock_fd)) {
+			upsdebugx(1, "%s: UPS [%s] is not currently connected, "
+				"trying to reconnect",
 				__func__, ups->name);
 			ups->sock_fd = sstate_connect(ups);
-			upsdebugx(1, "%s: UPS [%s] is now connected as FD %d",
-				__func__, ups->name, ups->sock_fd);
+			if (INVALID_FD(ups->sock_fd)) {
+				upsdebugx(1, "%s: UPS [%s] is still not connected (FD %d)",
+					__func__, ups->name, ups->sock_fd);
+			} else {
+				upsdebugx(1, "%s: UPS [%s] is now connected as FD %d",
+					__func__, ups->name, ups->sock_fd);
+			}
 			continue;
 		}
 
@@ -1064,7 +1363,7 @@ static void mainloop(void)
 		nfds++;
 	}
 
-	upsdebugx(2, "%s: polling %jd filedescriptors", __func__, (intmax_t)nfds);
+	upsdebugx(2, "%s: polling %" PRIdMAX " filedescriptors", __func__, (intmax_t)nfds);
 
 	ret = poll(fds, nfds, 2000);
 
@@ -1175,6 +1474,196 @@ static void mainloop(void)
 			continue;
 		}
 	}
+#else
+	/* scan through driver sockets */
+	for (ups = firstups; ups && (nfds < maxconn); ups = ups->next) {
+
+		/* see if we need to (re)connect to the socket */
+		if (INVALID_FD(ups->sock_fd)) {
+			upsdebugx(1, "%s: UPS [%s] is not currently connected, "
+				"trying to reconnect",
+				__func__, ups->name);
+			ups->sock_fd = sstate_connect(ups);
+			if (INVALID_FD(ups->sock_fd)) {
+				upsdebugx(1, "%s: UPS [%s] is still not connected (FD %d)",
+					__func__, ups->name, ups->sock_fd);
+			} else {
+				upsdebugx(1, "%s: UPS [%s] is now connected as FD %d",
+					__func__, ups->name, ups->sock_fd);
+			}
+			continue;
+		}
+
+		/* throw some warnings if it's not feeding us data any more */
+		if (sstate_dead(ups, maxage)) {
+			ups_data_stale(ups);
+		} else {
+			ups_data_ok(ups);
+		}
+
+		/* FIXME: Is the conditional needed? We got here... */
+		if (VALID_FD(ups->sock_fd)) {
+			fds[nfds] = ups->read_overlapped.hEvent;
+
+			handler[nfds].type = DRIVER;
+			handler[nfds].data = ups;
+
+			nfds++;
+		}
+	}
+
+	/* scan through client sockets */
+	for (client = firstclient; client; client = cnext) {
+
+		cnext = client->next;
+
+		if (difftime(now, client->last_heard) > 60) {
+			/* shed clients after 1 minute of inactivity */
+			client_disconnect(client);
+			continue;
+		}
+
+		if (nfds >= maxconn) {
+			/* ignore clients that we are unable to handle */
+			continue;
+		}
+
+		fds[nfds] = client->Event;
+
+		handler[nfds].type = CLIENT;
+		handler[nfds].data = client;
+
+		nfds++;
+	}
+
+	/* scan through server sockets */
+	for (server = firstaddr; server && (nfds < maxconn); server = server->next) {
+
+		if (INVALID_FD_SOCK(server->sock_fd)) {
+			continue;
+		}
+
+		fds[nfds] = server->Event;
+
+		handler[nfds].type = SERVER;
+		handler[nfds].data = server;
+
+		nfds++;
+	}
+
+	/* Wait on the read IO on named pipe  */
+	for (conn = pipe_connhead; conn; conn = conn->next) {
+		fds[nfds] = conn->overlapped.hEvent;
+		handler[nfds].type = NAMED_PIPE;
+		handler[nfds].data = (void *)conn;
+		nfds++;
+	}
+	/* Add the new named pipe connected event */
+	fds[nfds] = pipe_connection_overlapped.hEvent;
+	handler[nfds].type = NAMED_PIPE;
+	handler[nfds].data = NULL;
+	nfds++;
+
+	upsdebugx(2, "%s: wait for %d filedescriptors", __func__, nfds);
+
+	/* https://docs.microsoft.com/en-us/windows/win32/api/synchapi/nf-synchapi-waitformultipleobjects */
+	ret = WaitForMultipleObjects(nfds,fds,FALSE,2000);
+
+	upsdebugx(6, "%s: wait for filedescriptors done: %" PRIu64, __func__, ret);
+
+	if (ret == WAIT_TIMEOUT) {
+		upsdebugx(2, "%s: no data available", __func__);
+		return;
+	}
+
+	if (ret == WAIT_FAILED) {
+		DWORD err = GetLastError();
+		err = err; /* remove compile time warning */
+		upslog_with_errno(LOG_ERR, "%s", __func__);
+		upsdebugx(2, "%s: wait failed: code 0x%" PRIx64, __func__, err);
+		return;
+	}
+
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) )
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS
+# pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE
+# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#endif
+	if (ret >= WAIT_ABANDONED_0 && ret <= WAIT_ABANDONED_0 + nfds - 1) {
+		/* One abandoned mutex object that satisfied the wait? */
+		ret = ret - WAIT_ABANDONED_0;
+		upsdebugx(5, "%s: got abandoned FD array item: %" PRIu64, __func__, nfds, ret);
+		/* FIXME: Should this be handled somehow? Cleanup? Abort?.. */
+	} else
+	if (ret >= WAIT_OBJECT_0 && ret <= WAIT_OBJECT_0 + nfds - 1) {
+		/* Which one handle was triggered this time? */
+		/* Note: WAIT_OBJECT_0 may be currently defined as 0,
+		 * but docs insist on checking and shifting the range */
+		ret = ret - WAIT_OBJECT_0;
+		upsdebugx(5, "%s: got event on FD array item: %" PRIu64, __func__, nfds, ret);
+	}
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) )
+# pragma GCC diagnostic pop
+#endif
+
+	if (ret >= nfds) {
+		/* Array indexes are [0..nfds-1] */
+		upsdebugx(2, "%s: unexpected response to query about data available: %" PRIu64, __func__, ret);
+		return;
+	}
+
+	upsdebugx(6, "%s: requesting handler[%" PRIu64 "]", __func__, ret);
+	upsdebugx(6, "%s: handler.type=%d handler.data=%p", __func__, handler[ret].type, handler[ret].data);
+
+	switch(handler[ret].type) {
+		case DRIVER:
+			upsdebugx(4, "%s: calling sstate_readline() for DRIVER", __func__);
+			sstate_readline((upstype_t *)handler[ret].data);
+			break;
+		case CLIENT:
+			upsdebugx(4, "%s: calling client_readline() for CLIENT", __func__);
+			client_readline((nut_ctype_t *)handler[ret].data);
+			break;
+		case SERVER:
+			upsdebugx(4, "%s: calling client_connect() for SERVER", __func__);
+			client_connect((stype_t *)handler[ret].data);
+			break;
+		case NAMED_PIPE:
+			/* a new pipe connection has been signaled */
+			if (fds[ret] == pipe_connection_overlapped.hEvent) {
+				upsdebugx(4, "%s: calling pipe_connect() for NAMED_PIPE", __func__);
+				pipe_connect();
+			}
+			/* one of the read event handle has been signaled */
+			else {
+				upsdebugx(4, "%s: calling pipe_ready() for NAMED_PIPE", __func__);
+				pipe_conn_t * conn = handler[ret].data;
+				if ( pipe_ready(conn) ) {
+					if (!strncmp(conn->buf, SIGCMD_STOP, sizeof(SIGCMD_STOP))) {
+						set_exit_flag(1);
+					}
+					else if (!strncmp(conn->buf, SIGCMD_RELOAD, sizeof(SIGCMD_RELOAD))) {
+						set_reload_flag(1);
+					}
+					else {
+						upslogx(LOG_ERR,"Unknown signal"
+						       );
+					}
+
+					upsdebugx(4, "%s: calling pipe_disconnect() for NAMED_PIPE", __func__);
+					pipe_disconnect(conn);
+				}
+			}
+			break;
+		default:
+			upsdebugx(2, "%s: <unknown> has data available", __func__);
+			break;
+	}
+#endif
 }
 
 static void help(const char *arg_progname)
@@ -1190,35 +1679,29 @@ static void help(const char *arg_progname)
 	printf("		commands:\n");
 	printf("		 - reload: reread configuration files\n");
 	printf("		 - stop: stop process and exit\n");
+#ifndef WIN32
 	printf("  -P <pid>	send the signal above to specified PID (bypassing PID file)\n");
+#endif
 	printf("  -D		raise debugging level (and stay foreground by default)\n");
 	printf("  -F		stay foregrounded even if no debugging is enabled\n");
 	printf("  -FF		stay foregrounded and still save the PID file\n");
 	printf("  -B		stay backgrounded even if debugging is bumped\n");
-	printf("  -h		display this help\n");
+	printf("  -h		display this help text\n");
+	printf("  -V		display the version of this software\n");
 	printf("  -r <dir>	chroots to <dir>\n");
 	printf("  -q		raise log level threshold\n");
 	printf("  -u <user>	switch to <user> (if started as root)\n");
-	printf("  -V		display the version of this software\n");
 	printf("  -4		IPv4 only\n");
 	printf("  -6		IPv6 only\n");
 
+	nut_report_config_flags();
+
 	exit(EXIT_SUCCESS);
 }
 
-static void set_reload_flag(int sig)
-{
-	NUT_UNUSED_VARIABLE(sig);
-	reload_flag = 1;
-}
-
-static void set_exit_flag(int sig)
-{
-	exit_flag = sig;
-}
-
 static void setup_signals(void)
 {
+#ifndef WIN32
 	struct sigaction	sa;
 
 	sigemptyset(&sa.sa_mask);
@@ -1245,10 +1728,14 @@ static void setup_signals(void)
 	/* handle reloading */
 	sa.sa_handler = set_reload_flag;
 	sigaction(SIGHUP, &sa, NULL);
+#else
+	pipe_create(UPSD_PIPE_NAME);
+#endif
 }
 
 void check_perms(const char *fn)
 {
+#ifndef WIN32
 	int	ret;
 	struct stat	st;
 
@@ -1262,21 +1749,48 @@ void check_perms(const char *fn)
 	if (st.st_mode & (S_IROTH | S_IXOTH)) {
 		upslogx(LOG_WARNING, "%s is world readable", fn);
 	}
+#else
+	NUT_UNUSED_VARIABLE(fn);
+#endif
 }
 
 int main(int argc, char **argv)
 {
-	int	i, cmd = 0, cmdret = 0, foreground = -1;
+	int	i, cmdret = 0, foreground = -1;
+#ifndef WIN32
+	int	cmd = 0;
+	pid_t	oldpid = -1;
+#else
+	const char * cmd = NULL;
+#endif
 	char	*chroot_path = NULL;
 	const char	*user = RUN_AS_USER;
 	struct passwd	*new_uid = NULL;
-	pid_t	oldpid = -1;
 
 	progname = xbasename(argv[0]);
 
 	/* yes, xstrdup - the conf handlers call free on this later */
 	statepath = xstrdup(dflt_statepath());
-	datapath = xstrdup(DATADIR);
+#ifndef WIN32
+	datapath = xstrdup(NUT_DATADIR);
+#else
+	datapath = getfullpath(PATH_SHARE);
+
+	/* remove trailing .exe */
+	char * drv_name;
+	drv_name = (char *)xbasename(argv[0]);
+	char * name = strrchr(drv_name,'.');
+	if( name != NULL ) {
+		if(strcasecmp(name, ".exe") == 0 ) {
+			progname = strdup(drv_name);
+			char * t = strrchr(progname,'.');
+			*t = 0;
+		}
+	}
+	else {
+		progname = drv_name;
+	}
+#endif
 
 	/* set up some things for later */
 	snprintf(pidfn, sizeof(pidfn), "%s/%s.pid", altpidpath(), progname);
@@ -1307,27 +1821,34 @@ int main(int argc, char **argv)
 				break;
 
 			case 'V':
-				/* do nothing - we already printed the banner */
+				/* Note - we already printed the banner for program name */
+				nut_report_config_flags();
+
 				exit(EXIT_SUCCESS);
 
 			case 'c':
-				if (!strncmp(optarg, "reload", strlen(optarg)))
+				if (!strncmp(optarg, "reload", strlen(optarg))) {
 					cmd = SIGCMD_RELOAD;
-				if (!strncmp(optarg, "stop", strlen(optarg)))
+				} else
+				if (!strncmp(optarg, "stop", strlen(optarg))) {
 					cmd = SIGCMD_STOP;
+				}
 
 				/* bad command given */
 				if (cmd == 0)
 					help(progname);
 				break;
 
+#ifndef WIN32
 			case 'P':
 				if ((oldpid = parsepid(optarg)) < 0)
 					help(progname);
 				break;
+#endif
 
 			case 'D':
 				nut_debug_level++;
+				nut_debug_level_args++;
 				break;
 			case 'F':
 				if (foreground > 0) {
@@ -1363,44 +1884,132 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (cmd) {
-		if (oldpid < 0) {
-			cmdret = sendsignalfn(pidfn, cmd);
-		} else {
-			cmdret = sendsignalpid(oldpid, cmd);
-		}
-		exit((cmdret == 0)?EXIT_SUCCESS:EXIT_FAILURE);
+	{ /* scoping */
+		char *s = getenv("NUT_DEBUG_LEVEL");
+		int l;
+		if (s && str_to_int(s, &l, 10)) {
+			if (l > 0 && nut_debug_level_args < 1) {
+				upslogx(LOG_INFO, "Defaulting debug verbosity to NUT_DEBUG_LEVEL=%d "
+					"since none was requested by command-line options", l);
+				nut_debug_level = l;
+				nut_debug_level_args = l;
+			}	/* else follow -D settings */
+		}	/* else nothing to bother about */
 	}
 
-	/* otherwise, we are being asked to start.
-	 * so check if a previous instance is running by sending signal '0'
-	 * (Ie 'kill <pid> 0') */
+	/* Note: "cmd" may be non-trivial to command that instance by
+	 * explicit PID number or lookup in PID file (error if absent).
+	 * Otherwise, we are being asked to start and "cmd" is 0/NULL -
+	 * for probing whether a competing older instance of this program
+	 * is running (error if it is).
+	 */
+#ifndef WIN32
+	/* If cmd == 0 we are starting and check if a previous instance
+	 * is running by sending signal '0' (i.e. 'kill <pid> 0' equivalent)
+	 */
+
 	if (oldpid < 0) {
-		cmdret = sendsignalfn(pidfn, 0);
+		cmdret = sendsignalfn(pidfn, cmd);
 	} else {
-		cmdret = sendsignalpid(oldpid, 0);
+		cmdret = sendsignalpid(oldpid, cmd);
 	}
+#else	/* if WIN32 */
+	if (cmd) {
+		/* Command the running daemon, it should be there */
+		cmdret = sendsignal(UPSD_PIPE_NAME, cmd);
+	} else {
+		/* Starting new daemon, check for competition */
+		mutex = CreateMutex(NULL, TRUE, UPSD_PIPE_NAME);
+		if (mutex == NULL) {
+			if (GetLastError() != ERROR_ACCESS_DENIED) {
+				fatalx(EXIT_FAILURE,
+					"Can not create mutex %s : %d.\n",
+					UPSD_PIPE_NAME, (int)GetLastError());
+			}
+		}
+
+		cmdret = -1; /* unknown, maybe ok */
+		if (GetLastError() == ERROR_ALREADY_EXISTS
+		||  GetLastError() == ERROR_ACCESS_DENIED
+		) {
+			cmdret = 0; /* known conflict */
+		}
+	}
+#endif	/* WIN32 */
+
 	switch (cmdret) {
 	case 0:
-		printf("Fatal error: A previous upsd instance is already running!\n");
-		printf("Either stop the previous instance first, or use the 'reload' command.\n");
-		exit(EXIT_FAILURE);
+		if (cmd) {
+			upsdebugx(1, "Signaled old daemon OK");
+		} else {
+			printf("Fatal error: A previous upsd instance is already running!\n");
+			printf("Either stop the previous instance first, or use the 'reload' command.\n");
+			exit(EXIT_FAILURE);
+		}
+		break;
 
 	case -3:
 	case -2:
+		/* if starting new daemon, no competition running -
+		 *    maybe OK (or failed to detect it => problem)
+		 * if signaling old daemon - certainly have a problem
+		 */
 		upslogx(LOG_WARNING, "Could not %s PID file '%s' "
 			"to see if previous upsd instance is "
-			"already running!\n",
+			"already running!",
 			(cmdret == -3 ? "find" : "parse"),
 			pidfn);
 		break;
 
 	case -1:
+	case 1:	/* WIN32 */
 	default:
-		/* Just failed to send signal, no competitor running */
+		/* if cmd was nontrivial - speak up below, else be quiet */
+		upsdebugx(1, "Just failed to send signal, no daemon was running");
 		break;
 	}
 
+	if (cmd) {
+		/* We were signalling a daemon, successfully or not - exit now... */
+		if (cmdret != 0) {
+			/* sendsignal*() above might have logged more details
+			 * for troubleshooting, e.g. about lack of PID file
+			 */
+			upslogx(LOG_NOTICE, "Failed to signal the currently running daemon (if any)");
+#ifndef WIN32
+# ifdef HAVE_SYSTEMD
+			switch (cmd) {
+			case SIGCMD_RELOAD:
+				upslogx(LOG_NOTICE, "Try 'systemctl reload %s'%s",
+					SERVICE_UNIT_NAME,
+					(oldpid < 0 ? " or add '-P $PID' argument" : ""));
+				break;
+			case SIGCMD_STOP:
+				upslogx(LOG_NOTICE, "Try 'systemctl stop %s'%s",
+					SERVICE_UNIT_NAME,
+					(oldpid < 0 ? " or add '-P $PID' argument" : ""));
+				break;
+			default:
+				upslogx(LOG_NOTICE, "Try 'systemctl <command> %s'%s",
+					SERVICE_UNIT_NAME,
+					(oldpid < 0 ? " or add '-P $PID' argument" : ""));
+				break;
+			}
+			/* ... or edit nut-server.service locally to start `upsd -FF`
+			 * and so save the PID file for ability to manage the daemon
+			 * beside the service framework, possibly confusing things...
+			 */
+# else
+			if (oldpid < 0) {
+				upslogx(LOG_NOTICE, "Try to add '-P $PID' argument");
+			}
+# endif
+#endif	/* not WIN32 */
+		}
+
+		exit((cmdret == 0) ? EXIT_SUCCESS : EXIT_FAILURE);
+	}
+
 	argc -= optind;
 	argv += optind;
 
@@ -1424,9 +2033,13 @@ int main(int argc, char **argv)
 		chroot_start(chroot_path);
 	}
 
+#ifndef WIN32
 	/* default to system limit (may be overridden in upsd.conf) */
 	/* FIXME: Check for overflows (and int size of nfds_t vs. long) - see get_max_pid_t() for example */
 	maxconn = (nfds_t)sysconf(_SC_OPEN_MAX);
+#else
+	maxconn = 64;  /*FIXME : arbitrary value, need adjustement */
+#endif
 
 	/* handle upsd.conf */
 	load_upsdconf(0);	/* 0 = initial */
@@ -1465,16 +2078,19 @@ int main(int argc, char **argv)
 	server_load();
 
 	become_user(new_uid);
-
+#ifndef WIN32
 	if (chdir(statepath)) {
 		fatal_with_errno(EXIT_FAILURE, "Can't chdir to %s", statepath);
+	} else {
+		upsdebugx(1, "chdired into statepath %s for driver sockets", statepath);
 	}
+#endif
 
 	/* check statepath perms */
 	check_perms(statepath);
 
 	/* handle ups.conf */
-	read_upsconf();
+	read_upsconf(1);	/* 1 = may abort upon fundamental errors */
 	upsconf_add(0);		/* 0 = initial */
 	poll_reload();
 
@@ -1484,6 +2100,8 @@ int main(int argc, char **argv)
 		} else {
 			fatalx(EXIT_FAILURE, "Fatal error: at least one UPS must be defined in ups.conf");
 		}
+	} else {
+		upslogx(LOG_INFO, "Found %d UPS defined in ups.conf", num_ups);
 	}
 
 	/* try to bring in the var/cmd descriptions */
@@ -1508,13 +2126,16 @@ int main(int argc, char **argv)
 	/* initialize SSL (keyfile must be readable by nut user) */
 	ssl_init();
 
+	upsnotify(NOTIFY_STATE_READY_WITH_PID, NULL);
+
 	while (!exit_flag) {
+		/* Note: mainloop() calls upsnotify(NOTIFY_STATE_WATCHDOG, NULL); */
 		mainloop();
 	}
 
-	ssl_cleanup();
-
 	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
+	upsnotify(NOTIFY_STATE_STOPPING, "Signal %d: exiting", exit_flag);
+
+	ssl_cleanup();
 	return EXIT_SUCCESS;
 }
-
