@@ -31,14 +31,20 @@
 
 #include "common.h"
 
+#ifndef WIN32
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#endif
 
 #include "timehead.h"
 
 #include <sys/file.h>
-#include <poll.h> /* nfds_t */
+#ifdef HAVE_POLL_H
+# include <poll.h> /* nfds_t */
+#else
+typedef unsigned long int nfds_t;
+#endif
 
 #include "parseconf.h"
 #include "nut_ctype.h"
@@ -98,9 +104,13 @@ extern upstype_t	*firstups;
 extern nut_ctype_t	*firstclient;
 
 /* map commands onto signals */
-
+#ifndef WIN32
 #define SIGCMD_STOP	SIGTERM
 #define SIGCMD_RELOAD	SIGHUP
+#else
+#define SIGCMD_STOP    COMMAND_STOP
+#define SIGCMD_RELOAD  COMMAND_RELOAD
+#endif
 
 /* awkward way to make a string out of a numeric constant */
 
