@@ -23,6 +23,7 @@
 #define NUT_UPSTYPE_H_SEEN 1
 
 #include "parseconf.h"
+#include "common.h"
 
 #ifdef __cplusplus
 /* *INDENT-OFF* */
@@ -35,8 +36,11 @@ typedef struct upstype_s {
 	char			*name;
 	char			*fn;
 	char			*desc;
-
-	int			sock_fd;
+	TYPE_FD			sock_fd;
+#ifdef WIN32
+	char 			buf[SMALLBUF];
+	OVERLAPPED		read_overlapped;
+#endif
 	int			stale;
 	int			dumpdone;
 	int			data_ok;
