@@ -1,6 +1,5 @@
-/* device.h: definition of a container describing a NUT device
- * 
- *  Copyright (C) 2011 - Frederic Bohe <fredericbohe@eaton.com>
+/*
+ *  Copyright (C) 2011 - EATON
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,6 +15,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
+
+/*! \file nutscan-device.h
+    \brief definition of a container describing a NUT discovered device
+    \author Frederic Bohe <fredericbohe@eaton.com>
+*/
+
 #ifndef SCAN_DEVICE
 #define SCAN_DEVICE
 
@@ -36,7 +41,7 @@ extern "C" {
 	(assert(0 < (type) && (type) < TYPE_END), nutscan_device_type_strings[type - 1])
 
 typedef enum nutscan_device_type {
-	TYPE_NONE=0,
+	TYPE_NONE = 0,
 	TYPE_USB,
 	TYPE_SNMP,
 	TYPE_XML,
@@ -65,9 +70,9 @@ typedef struct nutscan_device {
 	struct nutscan_device * next;
 } nutscan_device_t;
 
-nutscan_device_t * nutscan_new_device();
+nutscan_device_t * nutscan_new_device(void);
 void nutscan_free_device(nutscan_device_t * device);
-void nutscan_add_option_to_device(nutscan_device_t * device,char * option, char * value);
+void nutscan_add_option_to_device(nutscan_device_t * device, char * option, char * value);
 nutscan_device_t * nutscan_add_device_to_device(nutscan_device_t * first, nutscan_device_t * second);
 
 /**
