@@ -23,12 +23,24 @@
 
 #include "config.h" /* must be first */
 
+#include "nut_stdint.h"
+#include "common.h"
 #include "nutscan-ip.h"
 #include <stdio.h>
-#include "common.h"
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <netdb.h>
+#ifndef WIN32
+# include <sys/socket.h>
+# include <netdb.h>
+#else
+/* Those 2 files for support of getaddrinfo, getnameinfo and freeaddrinfo
+   on Windows 2000 and older versions */
+# include <ws2tcpip.h>
+# include <wspiapi.h>
+# ifndef AI_NUMERICSERV
+#  define AI_NUMERICSERV NI_NUMERICSERV
+# endif
+# include "wincompat.h"
+#endif
 
 static void increment_IPv6(struct in6_addr * addr)
 {
@@ -257,6 +269,7 @@ int nutscan_cidr_to_ip(const char * cidr, char ** start_ip, char ** stop_ip)
 	nutscan_ip_iter_t ip;
 	int mask_val;
 	int mask_byte;
+	int ret;
 	uint32_t mask_bit; /* 32-bit IPv4 address bitmask */
 	char host[SMALLBUF];
 	struct addrinfo hints;
@@ -266,6 +279,9 @@ int nutscan_cidr_to_ip(const char * cidr, char ** start_ip, char ** stop_ip)
 
 	*start_ip = NULL;
 	*stop_ip = NULL;
+#ifdef WIN32
+        WSADATA WSAdata;
+#endif
 
 	cidr_tok = strdup(cidr);
 	first_ip = strdup(strtok_r(cidr_tok, "/", &saveptr));
@@ -307,13 +323,23 @@ int nutscan_cidr_to_ip(const char * cidr, char ** start_ip, char ** stop_ip)
 	hints.ai_family = AF_INET;
 
 	ip.type = IPv4;
-	/* Detecting IPv4 vs IPv6 */
-	if (getaddrinfo(first_ip, NULL, &hints, &res) != 0) {
-		/*Try IPv6 detection */
+
+#ifdef WIN32
+        WSAStartup(2,&WSAdata);
+        atexit((void(*)(void))WSACleanup);
+#endif
+
+	if ((ret = getaddrinfo(first_ip, NULL, &hints, &res)) != 0) {
+		/* EAI_ADDRFAMILY? */
+		upsdebugx(5, "%s: getaddrinfo() failed for AF_INET (IPv4): %d",
+			__func__, ret);
+
+		/* Try IPv6 detection */
 		ip.type = IPv6;
 		hints.ai_family = AF_INET6;
-		int ret;
 		if ((ret = getaddrinfo(first_ip, NULL, &hints, &res)) != 0) {
+			upsdebugx(5, "%s: getaddrinfo() failed for AF_INET6 (IPv6): %d",
+				__func__, ret);
 			free(first_ip);
 			return 0;
 		}
