@@ -53,9 +53,11 @@ static int (*nut_usb_get_string_simple)(libusb_device_handle *dev, int index,
  static ssize_t (*nut_usb_get_device_list)(libusb_context *ctx,	libusb_device ***list);
  static void (*nut_usb_free_device_list)(libusb_device **list, int unref_devices);
  static uint8_t (*nut_usb_get_bus_number)(libusb_device *dev);
+ static uint8_t (*nut_usb_get_device_address)(libusb_device *dev);
+ static uint8_t (*nut_usb_get_port_number)(libusb_device *dev);
  static int (*nut_usb_get_device_descriptor)(libusb_device *dev,
 	struct libusb_device_descriptor *desc);
-#else
+#else /* => WITH_LIBUSB_0_1 */
  #define USB_INIT_SYMBOL "usb_init"
  #define USB_OPEN_SYMBOL "usb_open"
  #define USB_CLOSE_SYMBOL "usb_close"
@@ -63,10 +65,14 @@ static int (*nut_usb_get_string_simple)(libusb_device_handle *dev, int index,
  static libusb_device_handle * (*nut_usb_open)(struct usb_device *dev);
  static void (*nut_usb_init)(void);
  static int (*nut_usb_find_busses)(void);
+# ifndef WIN32
  static struct usb_bus * (*nut_usb_busses);
+# else
+ static struct usb_bus * (*nut_usb_get_busses)(void);
+# endif	/* WIN32 */
  static int (*nut_usb_find_devices)(void);
  static char * (*nut_usb_strerror)(void);
-#endif
+#endif /* WITH_LIBUSB_1_0 */
 
 /* return 0 on error; visible externally */
 int nutscan_load_usb_library(const char *libname_path);
@@ -96,6 +102,7 @@ int nutscan_load_usb_library(const char *libname_path)
 			dl_error = lt_dlerror();
 			goto err;
 	}
+	lt_dlerror();      /* Clear any existing error */
 
 	*(void **) (&nut_usb_init) = lt_dlsym(dl_handle, USB_INIT_SYMBOL);
 	if ((dl_error = lt_dlerror()) != NULL) {
@@ -107,7 +114,6 @@ int nutscan_load_usb_library(const char *libname_path)
 			goto err;
 	}
 
-	lt_dlerror();      /* Clear any existing error */
 	*(void **) (&nut_usb_close) = lt_dlsym(dl_handle, USB_CLOSE_SYMBOL);
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
@@ -124,22 +130,50 @@ int nutscan_load_usb_library(const char *libname_path)
 			goto err;
 	}
 
-	*(void **) (&nut_usb_get_device_list) = lt_dlsym(dl_handle, "libusb_get_device_list");
+	*(void **) (&nut_usb_get_device_list) = lt_dlsym(dl_handle,
+					"libusb_get_device_list");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
 
-	*(void **) (&nut_usb_free_device_list) = lt_dlsym(dl_handle, "libusb_free_device_list");
+	*(void **) (&nut_usb_free_device_list) = lt_dlsym(dl_handle,
+					"libusb_free_device_list");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
 
-	*(void **) (&nut_usb_get_bus_number) = lt_dlsym(dl_handle, "libusb_get_bus_number");
+	*(void **) (&nut_usb_get_bus_number) = lt_dlsym(dl_handle,
+					"libusb_get_bus_number");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
 
-	*(void **) (&nut_usb_get_device_descriptor) = lt_dlsym(dl_handle, "libusb_get_device_descriptor");
+	/* Note: per https://nxmnpg.lemoda.net/3/libusb_get_device_address there
+	 * was a libusb_get_port_path() equivalent with different arguments, but
+	 * not for too long (libusb-1.0.12...1.0.16) and now it is deprecated.
+	 */
+	*(void **) (&nut_usb_get_device_address) = lt_dlsym(dl_handle,
+					"libusb_get_device_address");
+	if ((dl_error = lt_dlerror()) != NULL) {
+			goto err;
+	}
+
+	/* This method may be absent in some libusb versions, and we should
+	 * tolerate that! In run-time driver code see also blocks fenced by:
+	 *   #if (defined WITH_USB_BUSPORT) && (WITH_USB_BUSPORT)
+	 */
+	*(void **) (&nut_usb_get_port_number) = lt_dlsym(dl_handle,
+					"libusb_get_port_number");
+	if ((dl_error = lt_dlerror()) != NULL) {
+			fprintf(stderr,
+				"While loading USB library (%s), failed to find libusb_get_port_number() : %s. "
+				"The \"busport\" USB matching option will be disabled.\n",
+				libname_path, dl_error);
+			nut_usb_get_port_number = NULL;
+	}
+
+	*(void **) (&nut_usb_get_device_descriptor) = lt_dlsym(dl_handle,
+					"libusb_get_device_descriptor");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
@@ -150,17 +184,28 @@ int nutscan_load_usb_library(const char *libname_path)
 			goto err;
 	}
 #else /* for libusb 0.1 */
-	*(void **) (&nut_usb_find_busses) = lt_dlsym(dl_handle, "usb_find_busses");
+	*(void **) (&nut_usb_find_busses) = lt_dlsym(dl_handle,
+					"usb_find_busses");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
 
-	*(void **) (&nut_usb_busses) = lt_dlsym(dl_handle, "usb_busses");
+# ifndef WIN32
+	*(void **) (&nut_usb_busses) = lt_dlsym(dl_handle,
+					"usb_busses");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
+# else
+	*(void **) (&nut_usb_get_busses) = lt_dlsym(dl_handle,
+					"usb_get_busses");
+	if ((dl_error = lt_dlerror()) != NULL) {
+			goto err;
+	}
+# endif	/* WIN32 */
 
-	*(void **)(&nut_usb_find_devices) = lt_dlsym(dl_handle, "usb_find_devices");
+	*(void **)(&nut_usb_find_devices) = lt_dlsym(dl_handle,
+					"usb_find_devices");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
@@ -175,7 +220,9 @@ int nutscan_load_usb_library(const char *libname_path)
 	return 1;
 
 err:
-	fprintf(stderr, "Cannot load USB library (%s) : %s. USB search disabled.\n", libname_path, dl_error);
+	fprintf(stderr,
+		"Cannot load USB library (%s) : %s. USB search disabled.\n",
+		libname_path, dl_error);
 	dl_handle = (void *)1;
 	lt_dlexit();
 	return 0;
@@ -199,22 +246,52 @@ static char* is_usb_device_supported(usb_device_id_t *usb_device_id_list,
 }
 
 /* return NULL if error */
-nutscan_device_t * nutscan_scan_usb()
+nutscan_device_t * nutscan_scan_usb(void)
 {
 	int ret;
 	char string[256];
+	/* Items below are learned by libusbN version-specific API code
+	 * Keep in sync with items matched by drivers/libusb{0,1}.c
+	 * (nut)libusb_open methods, and fields of USBDevice_t struct
+	 * (drivers/usb-common.h).
+	 */
 	char *driver_name = NULL;
 	char *serialnumber = NULL;
 	char *device_name = NULL;
 	char *vendor_name = NULL;
 	uint8_t iManufacturer = 0, iProduct = 0, iSerialNumber = 0;
-	uint16_t VendorID;
-	uint16_t ProductID;
-	char *busname;
+	uint16_t VendorID = 0;
+	uint16_t ProductID = 0;
+	char *busname = NULL;
+	/* device_port physical meaning: connection port on that bus;
+	 *   different consumers plugged into same socket should have
+	 *   the same port value. However in practice such functionality
+	 *   depends on platform and HW involved and may mean logical
+	 *   enumeration results.
+	 * In libusb1 API: first libusb_get_port_numbers() earlier known
+	 *    as libusb_get_port_path() for physical port number on the bus, see
+	 *    https://libusb.sourceforge.io/api-1.0/group__libusb__dev.html#ga14879a0ea7daccdcddb68852d86c00c4
+	 *    later changed to logical libusb_get_bus_number() (which
+	 *    often yields same numeric value, except on systems that
+	 *    can not see or tell about physical topology)
+	 * In libusb0 API: "device filename"
+	 */
+	char *device_port = NULL;
+	/* bcdDevice: aka "Device release number" - note we currently do not match by it */
+	uint16_t bcdDevice = 0;
 #if WITH_LIBUSB_1_0
 	libusb_device *dev;
 	libusb_device **devlist;
-	uint8_t bus;
+	uint8_t bus_num;
+	/* Sort of like device_port above, but different (should be
+	 * more closely about physical port number than logical device
+	 * enumeration results). Uses libusb_get_port_number() where
+	 * available in libusb (and hoping the OS and HW honour it).
+	 */
+	char *bus_port = NULL;
+	ssize_t devcount = 0;
+	struct libusb_device_descriptor dev_desc;
+	int i;
 #else  /* => WITH_LIBUSB_0_1 */
 	struct usb_device *dev;
 	struct usb_bus *bus;
@@ -242,10 +319,6 @@ nutscan_device_t * nutscan_scan_usb()
 #endif /* WITH_LIBUSB_1_0 */
 
 #if WITH_LIBUSB_1_0
-	ssize_t devcount = 0;
-	struct libusb_device_descriptor dev_desc;
-	int i;
-
 	devcount = (*nut_usb_get_device_list)(NULL, &devlist);
 	if (devcount <= 0) {
 		(*nut_usb_exit)(NULL);
@@ -263,16 +336,53 @@ nutscan_device_t * nutscan_scan_usb()
 		iManufacturer = dev_desc.iManufacturer;
 		iProduct = dev_desc.iProduct;
 		iSerialNumber = dev_desc.iSerialNumber;
-		bus = (*nut_usb_get_bus_number)(dev);
+		bus_num = (*nut_usb_get_bus_number)(dev);
+
 		busname = (char *)malloc(4);
 		if (busname == NULL) {
 			(*nut_usb_free_device_list)(devlist, 1);
 			(*nut_usb_exit)(NULL);
 			fatal_with_errno(EXIT_FAILURE, "Out of memory");
 		}
-		snprintf(busname, 4, "%03d", bus);
+		snprintf(busname, 4, "%03d", bus_num);
+
+		device_port = (char *)malloc(4);
+		if (device_port == NULL) {
+			(*nut_usb_free_device_list)(devlist, 1);
+			(*nut_usb_exit)(NULL);
+			fatal_with_errno(EXIT_FAILURE, "Out of memory");
+		} else {
+			uint8_t device_addr = (*nut_usb_get_device_address)(dev);
+			if (device_addr > 0) {
+				snprintf(device_port, 4, "%03d", device_addr);
+			} else {
+				snprintf(device_port, 4, ".*");
+			}
+		}
+
+		if (nut_usb_get_port_number != NULL) {
+			bus_port = (char *)malloc(4);
+			if (bus_port == NULL) {
+				(*nut_usb_free_device_list)(devlist, 1);
+				(*nut_usb_exit)(NULL);
+				fatal_with_errno(EXIT_FAILURE, "Out of memory");
+			} else {
+				uint8_t port_num = (*nut_usb_get_port_number)(dev);
+				if (port_num > 0) {
+					snprintf(bus_port, 4, "%03d", port_num);
+				} else {
+					snprintf(bus_port, 4, ".*");
+				}
+			}
+		}
+
+		bcdDevice = dev_desc.bcdDevice;
 #else  /* => WITH_LIBUSB_0_1 */
+# ifndef WIN32
 	for (bus = (*nut_usb_busses); bus; bus = bus->next) {
+# else
+	for (bus = (*nut_usb_get_busses)(); bus; bus = bus->next) {
+# endif	/* WIN32 */
 		for (dev = bus->devices; dev; dev = dev->next) {
 
 			VendorID = dev->descriptor.idVendor;
@@ -282,6 +392,8 @@ nutscan_device_t * nutscan_scan_usb()
 			iProduct = dev->descriptor.iProduct;
 			iSerialNumber = dev->descriptor.iSerialNumber;
 			busname = bus->dirname;
+			device_port = dev->filename;
+			bcdDevice = dev->descriptor.bcdDevice;
 #endif
 			if ((driver_name =
 				is_usb_device_supported(usb_device_table,
@@ -291,9 +403,9 @@ nutscan_device_t * nutscan_scan_usb()
 #if WITH_LIBUSB_1_0
 				ret = (*nut_usb_open)(dev, &udev);
 				if (!udev || ret != LIBUSB_SUCCESS) {
-					fprintf(stderr,"Failed to open device "
-						"bus '%s', skipping: %s\n",
-						busname,
+					fprintf(stderr, "Failed to open device "
+						"bus '%s' device/port '%s' bus/port '%s', skipping: %s\n",
+						busname, device_port, bus_port,
 						(*nut_usb_strerror)(ret));
 
 					/* Note: closing is not applicable
@@ -302,7 +414,12 @@ nutscan_device_t * nutscan_scan_usb()
 					 * when e.g. permissions problem)
 					 */
 
-					free (busname);
+					free(busname);
+					free(device_port);
+					if (bus_port != NULL) {
+						free(bus_port);
+						bus_port = NULL;
+					}
 
 					continue;
 				}
@@ -311,8 +428,8 @@ nutscan_device_t * nutscan_scan_usb()
 				if (!udev) {
 					/* TOTHINK: any errno or similar to test? */
 					fprintf(stderr, "Failed to open device "
-						"bus '%s',skipping: %s\n",
-						busname,
+						"bus '%s' device/port '%s', skipping: %s\n",
+						busname, device_port,
 						(*nut_usb_strerror)());
 					continue;
 				}
@@ -328,6 +445,11 @@ nutscan_device_t * nutscan_scan_usb()
 							(*nut_usb_close)(udev);
 #if WITH_LIBUSB_1_0
 							free(busname);
+							free(device_port);
+							if (bus_port != NULL) {
+								free(bus_port);
+								bus_port = NULL;
+							}
 							(*nut_usb_free_device_list)(devlist, 1);
 							(*nut_usb_exit)(NULL);
 #endif	/* WITH_LIBUSB_1_0 */
@@ -347,6 +469,11 @@ nutscan_device_t * nutscan_scan_usb()
 							(*nut_usb_close)(udev);
 #if WITH_LIBUSB_1_0
 							free(busname);
+							free(device_port);
+							if (bus_port != NULL) {
+								free(bus_port);
+								bus_port = NULL;
+							}
 							(*nut_usb_free_device_list)(devlist, 1);
 							(*nut_usb_exit)(NULL);
 #endif	/* WITH_LIBUSB_1_0 */
@@ -367,6 +494,11 @@ nutscan_device_t * nutscan_scan_usb()
 							(*nut_usb_close)(udev);
 #if WITH_LIBUSB_1_0
 							free(busname);
+							free(device_port);
+							if (bus_port != NULL) {
+								free(bus_port);
+								bus_port = NULL;
+							}
 							(*nut_usb_free_device_list)(devlist, 1);
 							(*nut_usb_exit)(NULL);
 #endif	/* WITH_LIBUSB_1_0 */
@@ -386,6 +518,11 @@ nutscan_device_t * nutscan_scan_usb()
 					(*nut_usb_close)(udev);
 #if WITH_LIBUSB_1_0
 					free(busname);
+					free(device_port);
+					if (bus_port != NULL) {
+						free(bus_port);
+						bus_port = NULL;
+					}
 					(*nut_usb_free_device_list)(devlist, 1);
 					(*nut_usb_exit)(NULL);
 #endif	/* WITH_LIBUSB_1_0 */
@@ -436,6 +573,26 @@ nutscan_device_t * nutscan_scan_usb()
 					"bus",
 					busname);
 
+				nutscan_add_option_to_device(nut_dev,
+					"device",
+					device_port);
+
+#if WITH_LIBUSB_1_0
+				if (bus_port) {
+					nutscan_add_option_to_device(nut_dev,
+						"busport",
+						bus_port);
+					free(bus_port);
+					bus_port = NULL;
+				}
+#endif	/* WITH_LIBUSB_1_0 */
+
+				/* Not currently matched by drivers, hence commented for now: */
+				sprintf(string, "%04X", bcdDevice);
+				nutscan_add_option_to_device(nut_dev,
+					"###NOTMATCHED-YET###bcdDevice",
+					string);
+
 				current_nut_dev = nutscan_add_device_to_device(
 					current_nut_dev,
 					nut_dev);
@@ -449,6 +606,11 @@ nutscan_device_t * nutscan_scan_usb()
 	}
 #else	/* not WITH_LIBUSB_0_1 */
 		free(busname);
+		free(device_port);
+		if (bus_port != NULL) {
+			free(bus_port);
+			bus_port = NULL;
+		}
 	}
 
 	(*nut_usb_free_device_list)(devlist, 1);
@@ -457,9 +619,13 @@ nutscan_device_t * nutscan_scan_usb()
 
 	return nutscan_rewind_device(current_nut_dev);
 }
+
 #else /* not WITH_USB */
-nutscan_device_t * nutscan_scan_usb()
+
+/* stub function */
+nutscan_device_t * nutscan_scan_usb(void)
 {
 	return NULL;
 }
+
 #endif /* WITH_USB */
